/*  $Id: qprogres.h,v 1.3 1998/07/27 16:34:53 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

/////////////////// QProgressCtrl widget class //////////////////////
//
// for making Win95-style progress bars, as well as options for
// blocked/solid bar style, vertical/horizontal alignment, 
// bar/background colors
//
// Copyright (C) 1996 Tim D. Gilman
// This version of the class is the first release.  Please
// send comments/suggestions/bug reports to <tdgilman@best.com>
//
// This class was written using Qt 1.0 (http://www.troll.no)
//
// you may freely use this source code without permission for 
// non-commercial development.

#ifndef _PROGRESS_H
#define _PROGRESS_H

#include <qframe.h>
#include <qcolor.h>

typedef enum {
   horizBar,
   vertBar
} ProgressOrient;

typedef enum {
   blockedBar,
   solidBar
} ProgressStyle;

class QProgressCtrl : public QFrame {
   Q_OBJECT
 public:
   QProgressCtrl(QWidget *parent=0, const char *name=0);
   ~QProgressCtrl();
   
   void setOrientation(ProgressOrient t) { m_orient = t; }
   void setStyle(ProgressStyle t) { m_style = t; repaint(); }
   void setBarColor(QColor t) { m_barColor = new QColor(t); } 
   void setRange(int nLower=0, int nUpper=100);
   int minValue() { return m_nLower; }
   int maxValue() { return m_nUpper; } 
   int stepSize() { return m_nStep; }
   int value() { return m_nPos; }
   QColor barColor() { return *m_barColor; }
   int  setPos(int nPos);
   int  offsetPos(int nPos);
   int  setStep(int nStep=10);

 public slots:
   int  stepIt();

 protected:
   void resizeEvent( QResizeEvent * );
   
 private:
   void paintEvent( QPaintEvent * );
   void init();
   int  calcSolidBarLen(int nPos);
   int  calcBlockLen();
   int  calcNumBlocks(int nPos);
   void paintSolidBar(int nOldLen, int nLen);
   void paintBlockedBars(int nOldNumBlocks, int nNumBlocks);

   int  m_nLower, m_nUpper, m_nStep, m_nPos, m_nSolidLen, m_nBlockWidth,
	m_nNumBlocks;
   QColor *m_barColor;
   ProgressOrient m_orient;
   ProgressStyle m_style;
   
 private:
   QProgressCtrl( const QProgressCtrl & ) {}
   QProgressCtrl &operator=( const QProgressCtrl & ) { return *this; }
};

#endif // _PROGRESS_H
