/*  $Id: zProgressBar.cc,v 1.3 1998/07/27 16:34:53 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "zProgressBar.h"

zProgressBar::zProgressBar(QWidget *parent) 
					: BaseComponent(parent,"progbar")
{
	orient = new QString("Horizontal");
	type = new QString("Blocked");
	look = new QProgressCtrl(this,"foo");
	setLook(look);
	resize(330,33);
}

void 
zProgressBar::fillAttributes()
{
	ADDATTRIB("Starting Value", look->value(), W_TEXT);

        ADDATTRIB("Minimum Value", look->minValue(),W_TEXT);

        ADDATTRIB("Maximum Value", look->maxValue(),W_TEXT);

        ADDATTRIB("Incriment Size",look->stepSize(),W_TEXT);

        ADDATTRIB("Direction",orient->data(),W_LIST, 2,
        "Vertical", "Horizontal");

	ADDATTRIB("Bar Style",type->data(),W_LIST, 2,
	"Blocked", "Solid"); 

        ADDATTRIB("Name",(char *)myLook->name(),W_TEXT);

        ADDATTRIB("Tool Tip",tooltipData->data(),W_TEXT);

        ADDATTRIB("Left",x(),W_TEXT);

        ADDATTRIB("Top",y(),W_TEXT);

        ADDATTRIB("Width",width(),W_TEXT);

        ADDATTRIB("Height",height(),W_TEXT);

        ADDATTRIB("Background Colour",look->colorGroup().background(),W_COLOR);

        ADDATTRIB("Bar Colour",look->barColor(),W_COLOR);
}

void 
zProgressBar::interpretAttributes()
{
	QString i;

	look->setPos(attrib->giveIntValue("Starting Value"));

        look->setRange(attrib->giveIntValue("Minimum Value"),
                       attrib->giveIntValue("Maximum Value"));

	look->setStep(attrib->giveIntValue("Incriment Size"));

        setGeometry(attrib->giveIntValue("Left"),
                    attrib->giveIntValue("Top"),
                    attrib->giveIntValue("Width"),
                    attrib->giveIntValue("Height"));

        *orient = attrib->giveValue("Direction");
        if(*orient == "Vertical")
                look->setOrientation(vertBar);
        else
                look->setOrientation(horizBar);

        *tooltipData = attrib->giveValue("Tool Tip");

        changeName(attrib->giveValue("Name").data());

	look->setBackgroundColor(attrib->giveColor("Background Colour"));

	look->setBarColor(attrib->giveColor("Bar Colour"));

	*type = attrib->giveValue("Bar Style");
	if(*type == "Blocked")
		look->setStyle(blockedBar);
	else
		look->setStyle(solidBar);

}

void
zProgressBar::writeInits()
{
	QString i;

	i.sprintf("setPos(%d);\n",look->value());
	WRITEATTRIB("Starting Position", i);

	i.sprintf("setRange(%d,%d);\n",look->minValue(),look->maxValue());
	writeInParent(i);

	i.sprintf("setStep(%d);\n",look->stepSize());
	WRITEATTRIB("Incriment Size",i);

	i.sprintf("setGeometry(%d,%d,%d,%d);\n", x(), y(), width(), height());
	writeInParent(i);

	i.sprintf("setOrientation(%s);\n", 
		*orient == "Vertical" ? "vertBar" : "horizBar");
	WRITEATTRIB("Direction",i);

        if(!tooltipData->isEmpty())
        {
                QString p;
                if(noFuncs() == TRUE)
                        p = name();
                else
                        p = "this";

                i.sprintf(
                "QToolTip::add(%s,\"%s\");\n"
                        ,p.data(),tooltipData->data());
                writeLine(i);
        }

	writePalette();

        i.sprintf("setBarColor(QColor(%d,%d,%d));\n",
		look->barColor().red(), look->barColor().green(), 
		look->barColor().blue());
	writeInParent(i);

	i.sprintf("setStyle(%s);\n",
		*type == "Blocked" ? "blockedBar" : "solidBar");
	WRITEATTRIB("Bar Style",i);
}

void
zProgressBar::fillMembInfo()
{
        //fills generic virtual protected stuffs
        genericFillMembInfo();

        //empty destructor
        addMember("destructor()","","",0,2,1);

        //very generic constructor
        addMember("constructor(QWidget *parent=0, char *name=0)",
                        "init(parent,name)","",0,2,1);

	addMember("int stepIt()","","",0,4,0);
}

/******************************************************************************
 Uncomment if you intend to link this in dynamically, otherwise not necesary
******************************************************************************/
extern "C"
{
        zProgressBar *yourself(QWidget *parent)
        {
                return (new zProgressBar(parent));
        }
}
