/*  $Id: AttributeItem.cc,v 1.6 1998/07/27 16:34:54 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */


#include "main.h"
#include "AttributeItem.h"
#include <qstrlist.h>
#include <stdarg.h>
#include <stdlib.h>

int AttributeItem::wid = 0;

AttributeItem::AttributeItem(char *caption,
			 char *curVal, 
			 ATTRIB_TYPE tipe,
		         int numArgs,...)	:	QListBoxItem()
{
	clearNews();

	QStrList strings;

	va_list ap;
        va_start(ap,numArgs);
        for(int x=0; x < numArgs; x++) 	strings.append(va_arg(ap,char *));
	va_end(ap);

	initItem(caption, curVal, tipe, &strings);
}

AttributeItem::AttributeItem(char *caption,
			char *curVal,
			ATTRIB_TYPE tipe,
			QStrList *list)         :       QListBoxItem()
{
	clearNews();
	initItem(caption, curVal, tipe, list);
}

AttributeItem::AttributeItem(char *caption,
                         int curVal,
                         ATTRIB_TYPE tipe,
                         int numArgs,...)     :       QListBoxItem()
{		
	clearNews();

	if(tipe == W_TRUE)
        {
	    QStrList strings;
	    strings.append("True");
	    strings.append("False");

	    QString i;
             if(curVal == TRUE)
		 i = "True";
	     else
		 i = "False";
	     
	     initItem(caption,i.data(),tipe,&strings);
	}
	else
	{
	        QStrList strings;

	        va_list ap;
	        va_start(ap,numArgs);
	        for(int x=0; x < numArgs; x++)
	                strings.append(va_arg(ap,char *));
	        va_end(ap);
		
		QString i; i.sprintf("%d",curVal);
	        initItem(caption, i.data(), tipe, &strings);
	}
}

AttributeItem::AttributeItem(char *caption,
                         bool curVal,
                         ATTRIB_TYPE tipe)     :       QListBoxItem()
{
	clearNews();

	if(tipe == W_TRUE)
        {
	    QStrList strings;
	    strings.append("True");
	    strings.append("False");

	    QString i;
             if(curVal == TRUE)
		 i = "True";
	     else
		 i = "False";
	     
	     initItem(caption,i.data(),tipe,&strings);
	}
}

AttributeItem::AttributeItem(char *caption,
                         QColor curVal,
                         ATTRIB_TYPE tipe)     :       QListBoxItem()
{
	clearNews();

        if(tipe == W_COLOR)
        {
                optionType = tipe;
                myCaption = new QString(caption);
		currColor = curVal;
        }
}

AttributeItem::~AttributeItem()
{
        if(myCaption)      delete myCaption;
        delete myCurrentValue;
        if(strings)        delete [] strings;
}

AttributeItem::initItem(char *caption,
			char *curVal,
			ATTRIB_TYPE tipe,
			QStrList *list) 
{
	myCaption = new QString(caption);
	if(tipe == W_LIST || tipe == W_TRUE)
	{
		strings = new QString[list->count()];
        	for(int x=0; x < list->count(); x++)
                	strings[x] = list->at(x);
	}
	numItems = list->count();
	optionType = tipe;
	*myCurrentValue = curVal;
}

void
AttributeItem::clearNews()
{
    setWritten(0);
     myCaption = 0;
     myCurrentValue = new QString;
     strings = 0;
}

void
AttributeItem::paint(QPainter *p)
{
	p->setFont(QFont(configs->queryFontN(), configs->queryFontS()));
	QFontMetrics fm = p->fontMetrics();
	int y = fm.ascent() + fm.leading() / 2;

	if(written)
	{
		p->drawLine(3,configs->queryFontHeight()/2,
		   configs->queryFontWidth()/2,configs->queryFontHeight()-3);
		p->drawLine(configs->queryFontWidth()/2,
		   configs->queryFontHeight()-3,configs->queryFontWidth()-3,3);
	}
	p->drawText(configs->queryFontWidth(),y,(char *)myCaption->data());

	/* Erase text from caption beyond the vertical line */
	p->setBrush(QColor(192,192,192));
	p->fillRect((wid/2)+1,0,wid/2,configs->queryFontHeight(),
			QBrush(QColor(192,192,192),SolidPattern));

	/* Draw horizontal line on bottom of attribute 
	   p->drawLine(0,configs->queryFontHeight()+1,
		wid,configs->queryFontHeight()+1); */
//	for(int t = 0; t < (wid/2); t+=3) p->drawPoint(t,configs->queryFontHeight());
	p->drawLine(0,configs->queryFontHeight(),wid/2,configs->queryFontHeight());

	/* Draw vertical line on center */
     	p->drawLine(wid / 2,0,wid / 2,	configs->queryFontHeight()+3);

	if(optionType == W_COLOR)
	{
	    //border width
	    int border=2;
	    
	    //little border
	    p->setBrush("white");
	    p->drawRect((wid/2)+configs->queryFontWidth(),3,40,configs->queryFontHeight()-3);
			    
	    //square of colour
	    p->setBrush(currColor);
	    p->fillRect(((wid/2)+border)+configs->queryFontWidth(),3+border,40-(border*2),
			(configs->queryFontHeight()-3)-(border*2),
			QBrush(currColor,SolidPattern));
	}
	else
	{
	     p->setPen("red");
	     p->drawText((wid/2)+configs->queryFontWidth(),y,(char *)myCurrentValue->data());
	}
}

int
AttributeItem::height(const QListBox *) const
{
	return configs->queryFontHeight()+1;
}

void
AttributeItem::resizeHLines(int x)
{
	wid = x;
}

int
AttributeItem::width(const QListBox *lb) const
{
	if(!wid) wid = lb->width();
	return wid-5;
}

void
AttributeItem::setValue(QString p)
{
	if(optionType == W_COLOR)
	{
		int last = 0, next = 0;
		QString temp[3], lo = p.data(); 
		for(int count = 0; count < 3; count++)
		{
			next = lo.find(':',last);
			temp[count] = lo.mid(last,next-last);
			last = next+1;
			lo = p.data();
		}
		//Believe it or not it didn't work to do it anyother way
		//looks dumb but I guess whatever works...
		QColor c(temp[0].toInt(),temp[1].toInt(),temp[2].toInt());
		QColor d(currColor.red(),currColor.green(),currColor.blue());
		if(d != c) setWritten(1); currColor.setRgb(c.rgb());
	}
	else
	{
		if(p != *myCurrentValue) setWritten(1);
		*myCurrentValue = p; 
	}
}
