/*  $Id: Attributes.cc,v 1.9 1998/07/27 20:42:42 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "Attributes.h"
#include "components/BaseComp.h"
#include "components/CompIO.h"
#include "SigSlotEdit.h"
extern SigSlotEdit *sigSlot;

#include "SourceEdit.h"
extern SourceEdit *source;

#include <qfiledlg.h>
#include <qregexp.h>

void comment(ofstream &, char *);

Attributes::Attributes(QWidget *parent,char *name) : QWidget(parent,name)
{
    bc = 0; isSerial = -1; workingAttribute = 0;
    oldCapt = ""; validTextChange = TRUE;

    resize(500,400);
    ADDTOOLTIP(this,"Select attribute to change this area.");

    custCol = new QPushButton("Custom Colour",this,"custCol");
    ADDTOOLTIP(custCol,"Click to make a custom colour.");
    QObject::connect(custCol,SIGNAL(clicked()),this,SLOT(gotoColors()));
    custCol->setGeometry(0,25,width(),25);
    custCol->hide();

    textBox = new QLineEdit(this,"lined");
    ADDTOOLTIP(textBox,"Enter new value here.");
    QObject::connect(textBox,SIGNAL(textChanged(const char *)),
		     this,SLOT(textChangeAttribute(const char *)));
    textBox->setGeometry(0,25,width(),25);
    textBox->hide();

    browser = new QPushButton("Browse",this,"browd");
    ADDTOOLTIP(browser,"Click to browse filesystem for specific file.");
    QObject::connect(browser,SIGNAL(clicked()),this,SLOT(gotoFiles()));
    browser->setGeometry(width()-40,25,40,25);
    browser->hide();

    men = new QComboBox(FALSE,this,"combod");
    ADDTOOLTIP(men,"Select value from the list as new value.");
    QObject::connect(men,SIGNAL(activated(int)),
		     this,SLOT(menuChangeAttribute(int)));
    men->setGeometry(0,25,width(),25);
    men->hide();
	 
    objectList = new QComboBox(FALSE,this,"objlist");
    ADDTOOLTIP(objectList,"Select object you wish to edit.");
    objectList->setGeometry(0,0,width()-25,25);
    QObject::connect(objectList,SIGNAL(activated(int)),
		     this,SLOT(choseObject(int)));
    objectList->setEnabled(FALSE);

    hintButt = new Hint("Attributes:\nTo use this dialog select the object you wish to edit\nin the top most combo box. Then select the attribute you\nwish to edit in the big listbox. The area beneath the\nobject list will change to suit the kind of attribute you wish to\nchange, simply do as the popup says in that area of\nthe dialog and you're done. Double clicking on may types\nwill lead to actions being take automatically, for example\nlists will cycle through on double click. Attributes with\na check to the left of them have changed and will be written,\nothers have been left as default.",this,"hintButt");
    hintButt->move(width()-25,0);

    attributeList = new myListBox(this,"attriblist",1);
    ADDTOOLTIP(attributeList,"Select attribute to edit.");
    attributeList->setGeometry(0,50,width(),height()-80);
    QObject::connect(attributeList,SIGNAL(highlighted(int)),
		     this,SLOT(changeToAttribute(int)));
    QObject::connect(attributeList,SIGNAL(clicked(int)),
		     this, SLOT(changeToAttribute(int)));
    QObject::connect(attributeList,SIGNAL(dblClicked(int)),
		     this, SLOT(dblClick(int)));
	
    setCaption("Attributes");
}

void 
Attributes::resizeEvent(QResizeEvent *)
{
    objectList->resize(width()-25,objectList->height());
    men->resize(width(),men->height());
    custCol->resize(width(),custCol->height());
    textBox->resize(width(),textBox->height());
    browser->move(width()-40,browser->y());
    attributeList->resize(width(),height()-50);
    hintButt->move(width()-25,0);
    if(attributeList->count() > 0) 
    {
	AttributeItem *p = (AttributeItem *)attributeList->zItem(0);
	p->resizeHLines(width());
	attributeList->repaint();
    }
}

void
Attributes::applyf()
{
    if(workingAttribute) workingAttribute->setWritten(1);
    if(bc) bc->applyAttribs();
}

void
Attributes::addObject(char *description, int change, int where)
{
    objectList->insertItem(description,where);
    if(objectList->isEnabled() == FALSE) objectList->setEnabled(TRUE);
	
    if(!bc  && change) changeToObject(0);
}

void
Attributes::changeToObject(int x)
{
    if(bc == BaseComponent::at(x) &&
       BaseComponent::queryAction() != A_OPEN) return;

    bc = BaseComponent::at(x);
    BaseComponent::at(x)->giveFocus(0);
    int y = BaseComponent::positionToBox(x);
    choseObject(y);
}

void
Attributes::choseObject(int obj)
{
    bc = BaseComponent::at(BaseComponent::boxToPosition(obj));
    if(bc)
    {
	clearAttributes();
	attributeList->setAutoUpdate(FALSE);
	bc->showAttribs();
	attributeList->setAutoUpdate(TRUE);
	if(!oldCapt.isEmpty())
	{
	    AttributeItem *te;
	    for(int z = 0; z < attributeList->count(); z++)
	    {
		te = (AttributeItem *)attributeList->zItem(z);
		if(te->queryCaption() == oldCapt)
		{
		    changeToAttribute(z);
		    attributeList->setCurrentItem(z);
		}
	    }
	}
	attributeList->repaint();
	objectList->setCurrentItem(
	    BaseComponent::positionToBox(bc->queryPosition()));
    }
}

void
Attributes::removeObject(int x)
{
    BaseComponent *par;
    par = (BaseComponent *) BaseComponent::at(x)->parentWidget();
    if(par &&
       par->queryPosition() == BaseComponent::queryCurrPar())
    {
	clearAttributes();
	objectList->removeItem(
	    BaseComponent::positionToBox(x));
    }
    if(!objectList->count()) objectList->setEnabled(FALSE);
}

void
Attributes::clearObjects()
{
    objectList->clear();
}

void 
Attributes::addAttribute(AttributeItem *newEntry)
{
    newEntry->setWritten(bc->isWritten(newEntry->queryCaption().data()));

    if(newEntry->queryCaption().contains('"') || 
       newEntry->queryValue().contains('"'))
    {
	QMessageBox p; 
	p.setText("Illegal quote in attribute, being stripped.");p.show();
	newEntry->queryCaption().replace(QRegExp("\""),""); 
	newEntry->queryValue().replace(QRegExp("\""),"");
    }
	    
    if(isSerial != -1 && 
       BaseComponent::at(isSerial)->isWritten(newEntry->queryCaption().data()))
    {
	CompIO::saveAttribute(newEntry);
	delete newEntry;
    }
    else if(isSerial == -1)
    {
	attribDict.insert(newEntry->queryCaption().data(),newEntry);
	attributeList->inSort(newEntry);
    }
}

void
Attributes::clearAttributes()
{
    attribDict.clear();
    attributeList->clear();
    textBox->hide();
    custCol->hide();
    men->hide();
    browser->hide();
    men->hide();
}

void
Attributes::dblClick(int chosen)
{
    changeToAttribute(chosen);

    switch(workingAttribute->type())
    {
    case W_TRUE:
    case W_LIST:
	if(men->currentItem() == (men->count()-1))
	{
	    men->setCurrentItem(0);
	    menuChangeAttribute(0);
	}
	else
	{
	    men->setCurrentItem(men->currentItem()+1);
	    menuChangeAttribute(men->currentItem()+1);
	}
	break;
    case W_COLOR:
	gotoColors();
	break;	
    case W_FILE:
	gotoFiles();
	break;
    case W_TEXT:
	textBox->selectAll();
	break;
    }
}			

void
Attributes::changeToAttribute(int chosen)
{
    int x;
    QString tempString;
	
    workingAttribute = (AttributeItem *)attributeList->zItem(chosen);
    oldCapt = workingAttribute->queryCaption();

    textBox->hide();
    custCol->hide();
    men->hide();
    browser->hide();
    men->hide();

    switch(workingAttribute->type())
    {
    case W_COLOR:
	custCol->show();
	break;
    case W_LIST:
	men->clear();
	for(x = 0; x < workingAttribute->count(); x++)
	    men->insertItem(workingAttribute->string(x));
	men->setCurrentItem(locateValue());
	men->setFocus();
	men->show();
	break;
    case W_TRUE:
	men->clear();
	for(x = 0; x < workingAttribute->count(); x++)
	    men->insertItem(workingAttribute->string(x));
	men->setCurrentItem(locateValue());
	men->setFocus();
	men->show();
	break;
    case W_FILE:
	setTextBox(workingAttribute->queryValue());
	browser->show();
	textBox->setFocus();
	textBox->show();
	break;
    case W_TEXT:
	setTextBox(workingAttribute->queryValue());
	textBox->setFocus();
	textBox->show();
	break;
    }
}

void
Attributes::menuChangeAttribute(int x)
{
    if(bc) emit madeChange(bc->queryPosition());
    if(workingAttribute->type() == W_LIST)
	workingAttribute->setValue(men->currentText());
    else if(workingAttribute->type() == W_TRUE)
	workingAttribute->setValue(men->currentText());
    attributeList->repaint();

    applyf();
}	

void
Attributes::textChangeAttribute(const char *p)
{
    if(validTextChange == FALSE) return;

    if(bc) emit madeChange(bc->queryPosition());
    workingAttribute->setValue((QString)p);
    attributeList->repaint();

    applyf();
}

void
Attributes::setTextBox(QString p)
{
    validTextChange = FALSE;
    textBox->setText(p);
    validTextChange = TRUE;
}
	
//internal usage, this is so the proper thing in the menu type of attribute
//is selected from the start
int
Attributes::locateValue()
{
    int y=0;
    for(int x = 0; x < workingAttribute->count(); x++)
    {
	if(
	    workingAttribute->string(x) == workingAttribute->queryValue())
	    y = x;
    }
    return y;
}

QString
Attributes::giveValue(int x)
{
    AttributeItem *one = (AttributeItem *)attributeList->zItem(x);
    return one->queryValue();
}

QString
Attributes::giveCapt(int x)
{
    AttributeItem *one = (AttributeItem *)attributeList->zItem(x);
    return one->queryCaption();
}

QColor 
Attributes::giveColor(char *p)
{
    AttributeItem *one = attribDict.find(p);
    if(!one)
    {
	QString i;
	QMessageBox pp;
	i.sprintf("There may be a problem with the attribute %s.\n",p);
	pp.setText(i.data());
	pp.show();
	return QColor(0,0,0);
    }
    return one->queryColor();
}


QString
Attributes::giveValue(char *p)
{
    AttributeItem *one = attribDict.find(p);

    if(!one)
    {
	QString i;
	QMessageBox pp;
	i.sprintf("There may be a problem with the attribute %s.\n",p);
	pp.setText(i.data());
	pp.show();
	return 0;
    }
    return one->queryValue();
}

int
Attributes::giveWritten(int x)
{
    AttributeItem *one = (AttributeItem *)attributeList->zItem(x);
    return one->queryWritten();
}

int
Attributes::giveWritten(char *p)
{

    int toReturn = 1;
    AttributeItem *one = attribDict.find(p);
    if(one) toReturn = one->queryWritten();

    return toReturn;
}

int
Attributes::giveIntValue(int x)
{
    AttributeItem *one = (AttributeItem *)attributeList->zItem(x);
    return one->queryValue().toInt();
}

int
Attributes::giveIntValue(char *p)
{
    AttributeItem *one = attribDict.find(p);
    if(!one)
    {
	QString i;
	QMessageBox pp;
	i.sprintf("There may be a problem with the attribute %s.\n",p);
	pp.setText(i.data());
	pp.show();
	return 0;
    }
    return one->queryValue().toInt();
}

void
Attributes::setValue(QString where, QString what)
{
    AttributeItem *one = attribDict.find(where.data());
    if(one) one->setValue(what);
}

void
Attributes::setValue(int x, QString p)
{
    AttributeItem *one = (AttributeItem *)attributeList->zItem(x);
    one->setValue(p);
}

bool
Attributes::giveBoolValue(char *p)
{
    AttributeItem *one = attribDict.find(p);
    if(!one)
    {
	QString i;
	QMessageBox pp;
	i.sprintf("There may be a problem with the attribute %s.\n",
		  p);
	pp.setText(i.data());
	pp.show();
	return 0;
    }

    if(one->queryValue() == "True")
	return TRUE;
    else
	return FALSE;
}

bool
Attributes::giveBoolValue(int x)
{
    AttributeItem *one;
    one = (AttributeItem *)attributeList->zItem(x);
    if(one->queryValue() == "True")
	return TRUE;
    else
	return FALSE;
}

void
Attributes::changeObject(char *p, int x)
{
    objectList->changeItem(p,x);
}

void
Attributes::gotoFiles()
{
    QString path;
    path = QFileDialog::getOpenFileName();
    if(!path.isNull())
	textBox->setText(path.data());

    applyf();
}

void
Attributes::gotoColors()
{
    ColorSelect p(0,"p",workingAttribute->queryColor());
    p.move(x(),y());
    if(p.exec())
    {
	if(bc) emit madeChange(bc->queryPosition());
	workingAttribute->setColor(p.queryColor()); 
    }
    attributeList->repaint();

    applyf();
}
	
//Call with -1 to turn of serial mode
void
Attributes::setSerialize(int obj)
{
    isSerial = obj;
    if(isSerial != -1) BaseComponent::at(obj)->fillAttributes();
}


