/*  $Id: MenuBar.cc,v 1.7 1998/07/28 18:49:17 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "MenuBar.h"
#include "YesNo.h"
#include "components/BaseComp.h"
#include "components/CompIO.h"
#include "components/zApplication.h"
#include "components/zDialog.h"
#include "components/LinkList.h"

#include <qlayout.h>
#include <qkeycode.h>
#include <qfile.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "pix/shortcuts/open.xpm"
#include "pix/shortcuts/save.xpm"
#include "pix/shortcuts/new_project.xpm"
#include "pix/shortcuts/new_dialog.xpm"
#include "pix/shortcuts/cut.xpm"
#include "pix/shortcuts/copy.xpm"
#include "pix/shortcuts/paste.xpm"
#include "pix/shortcuts/revert.xpm"
#include "pix/shortcuts/run.xpm"
#include "pix/shortcuts/stop.xpm"

#ifdef __USE_KDE
	#include <qdir.h>
	#include <kapp.h>
	#undef Unsorted
#else
	#include <qapp.h>
#endif

#include <qfiledlg.h>
#include "FancyButton.h"

MenuBar::MenuBar(QWidget *parent, char *name) : QWidget(parent,name)
{
        QPixmap p;

	childPID = 0;

	level = 0;
	currFile = "";
	workingComp = "";
	project = 0;
	ntStanE = 0;
	ntStanO = 0;

	compileWindow = new CompileWindow(0);
	compileWindow->resize(650,300);
	compileWindow->setCaption("Compile Progress");
	compileWindow->setReadOnly(TRUE);
	compileWindow->hide();

	xyLabel = new QLabel(this,"xyLabel");
	xyLabel->setText("X:0 Y:0");
	xyLabel->setFont(QFont("times",14,QFont::Normal,(int)0));
	xyLabel->setAlignment(AlignRight | AlignVCenter);
	xyLabel->setFrameStyle(QFrame::NoFrame | QFrame::NoFrame);
	xyLabel->setGeometry(460,25,70,30);
	ADDTOOLTIP(xyLabel,"X & Y coordinates over a toplevel widget.");

	chang = new QLabel(this,"chang");
        chang->setText("Not Modified");
        chang->setFont(QFont("times",14,QFont::Normal,(int)0));
        chang->setAlignment(AlignRight | AlignVCenter);
        chang->setFrameStyle(QFrame::NoFrame | QFrame::NoFrame);
        chang->setGeometry(530,25,89,30);
	ADDTOOLTIP(chang,"Displays whether the project has changed.");

	hintButt = new Hint("QtEZ:\nYou can get to any of the available editors\nfrom the windows menu. Shortcuts are as follows:\nRight click on component, alt+a, or window | attributes\n\ttakes you to attribute window.\nDouble-Click on component, alt+s, or window | source editor\n\ttakes you to the source window.\nMiddle click on a component or alt+b\n\twill send it to the back.\nalt+h will show you the globals editor\nalt+m will show you the menubar editor.\nalt+c will show you the signal/slot editor\nKeep in mind before droping a component from\nthe toolbar you should select the toplevel to drop onto.\nDo this by click on the parent you wish it to go on.",this,"hintButt");
	hintButt->move(width()-15,25);

	fileMenu = new QPopupMenu;
	fileMenu->insertItem("New Project",this,SLOT(gnuProject()),CTRL+Key_P);
        fileMenu->insertItem("New Dialog",this,SLOT(gnuWidget()),CTRL+Key_D);
	fileMenu->insertSeparator();
	fileMenu->insertItem("Open..",this,SLOT(openProject()),CTRL+Key_O);
	fileMenu->insertItem("Save",this,SLOT(save()),CTRL+Key_S);
	fileMenu->insertItem("Save As..",this,SLOT(saveAs()),CTRL+Key_A);
	fileMenu->insertItem("Revert",this,SLOT(revert()),CTRL+Key_R);
	fileMenu->insertSeparator();
	fileMenu->insertItem("Import..", this,SLOT(importDialog()),CTRL+Key_I);
	fileMenu->insertItem("Export Dialog..",this,SLOT(exportDialog()),CTRL+Key_E);
	fileMenu->insertSeparator();
        fileMenu->insertItem("&Save as source",this,SLOT(saveSource()),CTRL+Key_A);
        fileMenu->insertSeparator();
  	fileMenu->insertItem("&Quit",this,SLOT(end()),CTRL+Key_Q);

	editMenu = new QPopupMenu;
	editMenu->insertItem("C&ut",this,SLOT(cutComponent()),CTRL+Key_X);
	editMenu->insertItem("&Copy",this,SLOT(copyComponent()),CTRL+Key_C);
	editMenu->insertItem("&Paste",this,SLOT(pasteComponent()),CTRL+Key_V);
	editMenu->insertSeparator();
	editMenu->insertItem("&Raise",this,SLOT(raiseS()));
	editMenu->insertItem("&Lower",this,SLOT(lowerS()));
	editMenu->insertItem("Un/L&ock",this,SLOT(lockS()));

	runMenu = new QPopupMenu;
	runMenu->insertItem("&Make",this,SLOT(makeProgram()),CTRL+Key_M);
	runMenu->insertItem("Remake &All",this,SLOT(makeProgramAll()),CTRL+Key_R);
	runMenu->insertSeparator();
	runMenu->insertItem("&Run",this,SLOT(runProgram()),CTRL+Key_N);
	runMenu->insertItem("&Debug",this,SLOT(debugProgram()),CTRL+Key_D);
	runMenu->insertSeparator();
	runMenu->insertItem("&Stop",this,SLOT(stopProgram()),CTRL+Key_T);

	windowMenu = new QPopupMenu;
	configId = windowMenu->insertItem(
		"&Configuration",this,SLOT(configDialog()),ALT+Key_F);
	projectId = windowMenu->insertItem(
		"&Project Viewer",this,SLOT(showProject()),ALT+Key_V);
	windowMenu->insertSeparator();
	attribId = windowMenu->insertItem(
		"&Attributes",this,SLOT(propertiesDialog()),ALT+Key_A);
        glbEdId = windowMenu->insertItem(
                "&Globals Editor",this,SLOT(showGlobal()),ALT+Key_H);
        mnuEdId = windowMenu->insertItem(
                "&Menu Bar Editor",this,SLOT(showMenuBar()),ALT+Key_M);
	sigSlotId = windowMenu->insertItem(
		"&Signal/Slot Editor",this,SLOT(showSigSlot()),ALT+Key_C);
        srcEdId = windowMenu->insertItem(
                "&Source Editor",this,SLOT(showSource()),ALT+Key_S);
        toolBarId = windowMenu->insertItem(
                "&Tool Bar",this,SLOT(showToolBar()),ALT+Key_T);
	windowMenu->setCheckable(TRUE);

	helpMenu = new QPopupMenu;
	helpMenu->insertItem("&Help",this,SLOT(helpDialog()),Key_F1);
	helpMenu->insertSeparator();
	helpMenu->insertItem("&About",this,SLOT(aboutDialog()),CTRL+Key_Z);

	#ifdef __USE_KDE
		theMenu = new KMenuBar(this);
	#else
		theMenu = new QMenuBar(this);
	#endif
	theMenu->insertItem("&File",fileMenu);
	theMenu->insertItem("&Edit",editMenu);
	theMenu->insertItem("&Windows",windowMenu);
	theMenu->insertItem("&Run",runMenu);
	theMenu->insertSeparator();
	theMenu->insertItem("&Help",helpMenu);

        makeShortCuts();

	setCaption(__VERSION);
	resize(650,theMenu->height()+5+25);
	setMaximumSize(1000,height());
	setMinimumSize(600,height());
}       

void
MenuBar::makeShortCuts()
{
    FancyButton *push;
    int y = theMenu->height()+2;
    
    push = new FancyButton(this);
    push->setGeometry(0,y,25,25);
    push->setPixmap(QPixmap((const char**)new_project_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(gnuProject()));
    ADDTOOLTIP(push,"New Project");

    push = new FancyButton(this);
    push->setGeometry(25,y,25,25);
    push->setPixmap(QPixmap((const char**)new_dialog_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(gnuWidget()));
    ADDTOOLTIP(push,"New Dialog");


    push = new FancyButton(this);
    push->setGeometry(50,y,25,25);
    push->setPixmap(QPixmap((const char**)open_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(openProject()));
    ADDTOOLTIP(push,"Open");

    push = new FancyButton(this);
    push->setGeometry(75,y,25,25);
    push->setPixmap(QPixmap((const char**)save_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(save()));
    ADDTOOLTIP(push,"Save");

    push = new FancyButton(this);
    push->setGeometry(100,y,25,25);
    push->setPixmap(QPixmap((const char**)revert_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(revert()));
    ADDTOOLTIP(push,"Revert");

    push = new FancyButton(this);
    push->setGeometry(135,y,25,25);
    push->setPixmap(QPixmap((const char**)cut_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(cutComponent()));
    ADDTOOLTIP(push,"Cut");

    push = new FancyButton(this);
    push->setGeometry(160,y,25,25);
    push->setPixmap(QPixmap((const char**)copy_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(copyComponent()));
    ADDTOOLTIP(push,"Copy");

    push = new FancyButton(this);
    push->setGeometry(185,y,25,25);
    push->setPixmap(QPixmap((const char**)paste_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(pasteComponent()));
    ADDTOOLTIP(push,"Paste");

    push = new FancyButton(this);
    push->setGeometry(220,y,25,25);
    push->setPixmap(QPixmap((const char**)run_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(runProgram()));
    ADDTOOLTIP(push,"Run");

    push = new FancyButton(this);
    push->setGeometry(245,y,25,25);
    push->setPixmap(QPixmap((const char**)stop_xpm));
    QObject::connect(push,SIGNAL(clicked()),this,SLOT(stopProgram()));
    ADDTOOLTIP(push,"Stop");
}

void
MenuBar::end()
{
	if(!clearWidgets()) return;
	configs->saveYourself();

        QString i;
	i.sprintf("%s/QtEZ/Makefile",getenv("HOME"));
	unlink(i.data());
        i.sprintf("%s/QtEZ/main.cpp",getenv("HOME"));
        unlink(i.data());
        i.sprintf("%s/QtEZ/main.h",getenv("HOME"));
        unlink(i.data());
	i.sprintf("%s/QtEZ/main.o",getenv("HOME"));
	unlink(i.data());
	i.sprintf("%s/QtEZ/buffer.qtz",getenv("HOME"));
	unlink(i.data());
	i.sprintf("%s/QtEZ/tmpbk.qtz",getenv("HOME"));
	unlink(i.data());

	exit(0);
}

void
MenuBar::showToolBar()
{
	if(toolBar->isVisible())
	{
		toolBar->close();
	}
	else
	{
		toolBar->show();
		toolBar->raise();
	}
}

void 
MenuBar::gnuWidget()
{
	project = new zDialog(0);
	project->show();
	QObject::connect(project,SIGNAL(changeMade()),this,SLOT(changeMade()));	
	QObject::connect(project,SIGNAL(xyPos(int,int)),this,SLOT(xy(int,int)));
	changeMade();
}

void
MenuBar::gnuProject()
{
	if(clearWidgets())
	{
		setCaption(__VERSION);
		currFile = "";
		BaseComponent::prepApp();
		chang->setText("Not Modified");
	}
} 

int
MenuBar::clearWidgets()
{
    	int x;

 	if(!BaseComponent::count()) return 1;

	if((QString)chang->text() == "Modified")
	{
		YesNo yn(0,"yn");
		if(!yn.doit("All unsaved work will be lost.\nARE YOU SURE?"))
			return 0;
	}
	setCaption("QtEZ: Closing Project");
	
        BaseComponent::setAction(A_CLOSE);

	if(global->isVisible())
	{
		global->close();
	}
	if(menu->isVisible())
	{
		menu->close();
	}
	if(attrib->isVisible())
	{
		attrib->close();
	}
	source->hideDebug();
	if(source->isVisible())
	{
		source->close();
	}
	if(sigSlot->isVisible())
	{
		sigSlot->close();
	}
 
	compileWindow->hide();
	attrib->clearObjects();
	sigSlot->clearObjects();

	workingComp = "";
	global->clearHeader();
	global->clearFunction();
	global->clearComment();
	global->clearFreeHand();
	if(BaseComponent::count()) showLevel((int)(BaseComponent::count()));

	for(x = 0; x < BaseComponent::count(); x++)
	{
 	    if(!BaseComponent::at(x)->parentWidget()) 
		BaseComponent::at(x)->hide();
	}

	for(x = BaseComponent::count() -1; x >= 0; x--)
	{
 		stepLevel();
 		delete BaseComponent::at(x);
 		project = 0;
	}
        if(childPID)
        {
                kill(childPID,SIGKILL);
                childPID = 0;
                waitpid(0,0,0);
        }
	attrib->resetObj();
	source->resetObj();
	menu->resetObj();
	projView->clear();
	chang->setText("Not Modified");
	hideLevel();
        BaseComponent::setAction(A_NONE);
	return 1;
}

void
MenuBar::projectGone()
{
	project = 0;
}

void
MenuBar::configDialog()
{
        if(configs->isVisible())
        {
                configs->close();
        }
        else
        {
                configs->show();
                configs->raise();
        }
}

void
MenuBar::propertiesDialog()
{
	if(attrib->isVisible())
	{
		attrib->close();
	}
	else
	{
		attrib->show();
		attrib->raise();
		attrib->repaint();
	}
}

void
MenuBar::showSigSlot()
{
        if(sigSlot->isVisible())
        {
                sigSlot->close();
        }
        else
        {
                sigSlot->show();
                sigSlot->raise();
		sigSlot->repaint();
        }
}

void
MenuBar::showSource()
{
	if(source->isVisible())
	{
		source->close();
	}
	else
	{
		source->show();
		source->raise();
	}
}

void
MenuBar::showGlobal()
{
	global->setFonts(QFont(configs->queryFontN(), configs->queryFontS()));
	if(global->isVisible())
	{
		global->close();
	}
	else
	{
		global->show();
		global->raise();
	}
}

void
MenuBar::showProject()
{
	if(projView->isVisible())
	{
		projView->close();
	}
	else
	{
		projView->show();
		projView->raise();
		projView->repaint();
	}
}

void
MenuBar::showMenuBar()
{
        if(menu->isVisible())
        {
                menu->close();
        }
        else
        {
                menu->show();
                menu->raise();
        }
}

void
MenuBar::changeMade()
{
	chang->setText("Modified");
}

void 
MenuBar::saveSource()
{
	if(BaseComponent::at(0)) BaseComponent::at(0)->writeToFile(0);
}
        
void
MenuBar::openProject()
{
	if(BaseComponent::at(0)) 
	{
		workingComp = "";
		if(!clearWidgets()) return;
	}
        QString temp = QFileDialog::getOpenFileName(getenv("PWD"),"*.qtz");
        if(temp.isEmpty()) 
	  { BaseComponent::prepApp(); chang->setText("Not Modified"); return; }
	project = 0;

	currFile = temp;
	CompIO::load(temp,O_OPEN);

        QString i = temp.data();
        i = i.right(i.length()-(i.findRev('/')+1));
        i.insert(0,"QtEZ: ");
        setCaption(i.data());

	chang->setText("Not Modified");
}

void
MenuBar::revert()
{
        if(BaseComponent::at(0))
        {
                workingComp = "";
                if(!clearWidgets()) return;
        }
        if(currFile.isEmpty()) 
	  { BaseComponent::prepApp(); chang->setText("Not Modified"); return; }
        project = 0;

        CompIO::load(currFile,O_OPEN);

        chang->setText("Not Modified");
}

void
MenuBar::exportDialog() 
{ 
		if(!BaseComponent::at(0)) return;

                QString temp = QFileDialog::getSaveFileName(getenv("PWD"),
                        "*.qtz");
                if(temp.isEmpty()) return;

//Have to think
                CompIO::save(temp,
			BaseComponent::queryCurrPar(), 
			 BaseComponent::fromTopLev(
				BaseComponent::toTopLev(
				BaseComponent::queryCurrPar())+1));
}

void
MenuBar::importDialog()
{
        QString temp = QFileDialog::getOpenFileName(getenv("PWD"),"*.qtz");
        if(temp.isEmpty()) return;

        QString i = temp.data();
        i = i.right(i.length()-(i.findRev('/')+1));
        i.insert(0,"QtEZ: ");
        setCaption(i.data());

	CompIO::load(temp,O_IMPORT);
	chang->setText("Modified");
}

void
MenuBar::save()
{
        if(BaseComponent::at(0))
        {
                if(currFile.isEmpty()) 
		{
		    saveAs();
		    return;
		}

                CompIO::save(currFile);
                chang->setText("Not Modified");
        }
}

void
MenuBar::saveAs()
{
	if(BaseComponent::at(0))
	{
		QString temp = QFileDialog::getSaveFileName(getenv("PWD"),
			"*.qtz");
		if(temp.isEmpty()) return;
	
	        QString i = temp.data();
	        i = i.right(i.length()-(i.findRev('/')+1));
	        i.insert(0,"QtEZ: ");
	        setCaption(i.data());

		if(temp.right(4) != ".qtz") temp += ".qtz";
		currFile = temp.data();
		if(QFile::exists(temp.data()))
 	        {
		       YesNo yn(0,"yn");
		       if(!yn.doit("Over write existing file?"))
		       {
			      chang->setText("Not Modified");
			      return;
		       }
		}
		CompIO::save(temp);
		chang->setText("Not Modified");
	}
}

void 
MenuBar::helpDialog()
{
	QMessageBox p(this);
	p.setCaption("Help!");
	p.setIcon(QMessageBox::Information);
	p.move(x(),y()+height());
	p.setText("QtEZ:\nYou will find \"hint\" buttons on many of the dialogs that\nrequire a little assistance. These are little grey buttons\nwith a question mark on them. Clicking on them will bring\nup a box describing how to use the dialog you are looking\nat. You will also find there are \"tooltips\" they are \nyellow boxes with words in them describing the area of\nthe dialog you have your mouse over, using these QtEZ\nshould be clear enough to use. Should you\nrequire more assistance there are helpfiles that come\nwith the QtEZ distribution. These are normally in docs/\nthey are html format. They are also available on the\ninternet via http://qtez.zax.net/qtez/. I hope you find\nthis program usefull, yet simple. Should you require any\nmore help you can mail me at zachsman@wiw.org, I will\ntry to read mail concerning QtEZ problems first, please\ninclude the version number with report/question.");
	p.show();
}
        
void 
MenuBar::aboutDialog()
{
	AboutScreen a(0,"a");
	a.exec();
}

int
MenuBar::compileProgram(int runit, int debugit)
{
	source->clearDebug();

       	BaseComponent::at(0)->writeToFile(1);

	if(childPID)
	{
		kill(childPID,SIGKILL);
		childPID = 0;
		waitpid(0,0,0);
	}

	int stanE[2], stanO[2];
	if(pipe(stanE) || pipe(stanO))
	{
		QMessageBox p;
		p.setIcon(QMessageBox::Critical);
		p.setText("Failure creating pipes, bailing out.");
		p.show();
		return 0;
	}
      	compileWindow->clear();
	compileWindow->goDown("***Compilation***");
	compileWindow->setFont(QFont(configs->queryFontN(), 
				     configs->queryFontS()));
	if(configs->queryCompile()) compileWindow->show();

	#ifdef __USE_KDE
	  KApplication::flushX();
	#else
	  QApplication::flushX();
	#endif
	childPID = fork();

        if(!childPID)
        {
		::close(stanE[0]); ::close(stanO[0]);
		stanO[0] = open("/dev/null",O_RDONLY);	
		stanE[0] = open("/dev/null",O_RDONLY);	

		dup2(stanO[1],1); dup2(stanE[1],2);

		QString i;
		i = "cd ";
		i += BaseComponent::queryFinalPath().data();
		i += "; ";

		i += configs->queryMak();
		i += " all";

		system(i.data());

		if(runit)
		{
			QString s;
			if(debugit) 
			{
			   i = configs->queryTerm();   

			   s += BaseComponent::queryFinalPath().data();
			   s += BaseComponent::at(0)->queryExecutable().data();

			   QString whereD;
			   whereD.sprintf("-display %s",
				configs->queryWhereDisp());

			   execl(i.data(),whereD.data(), 
				"-e",configs->queryDebug(), s.data(),0);	
			}
			else
			{
	                   i = BaseComponent::queryFinalPath().data();
        	           i += BaseComponent::at(0)->queryExecutable().data();

                           QString whereD;
                           whereD.sprintf("-display %s",
                                configs->queryWhereDisp());

			   execl(i.data(),whereD.data(),0);

			}

		}
                exit(100);
        }
	::close(stanE[1]); ::close(stanO[1]);

	if(ntStanE) delete ntStanE;
	ntStanE = new QSocketNotifier(stanE[0],QSocketNotifier::Read);
	QObject::connect(ntStanE,SIGNAL(activated(int)),
                             this,SLOT(idE(int)));

        if(ntStanO) delete ntStanO;
        ntStanO = new QSocketNotifier(stanO[0],QSocketNotifier::Read);
        QObject::connect(ntStanO,SIGNAL(activated(int)),
                             this,SLOT(idO(int)));


        workingComp = BaseComponent::queryFinalPath().data();
        workingComp +=  BaseComponent::at(0)->queryExecutable().data();
	return 1;
}

void
MenuBar::idE(int s)
{
	static QString right;
	char *buff = new char[300];
	bzero(buff,300); read(s,buff,300);
	right += buff;
	while(right.contains('\n') > 1)
	{
 		source->addDebug(right.left(right.find('\n')).data());
		right= right.right((right.length()-right.find('\n'))-1);
	}
	compileWindow->goDown(buff);
	delete [] buff;
}

void
MenuBar::idO(int s)
{
    char *buff = new char[100];
    bzero(buff,100);
    int tmp = read(s,buff,100);

    if(!tmp) //Compilation complete
    {
	ntStanO->setEnabled(FALSE);
	ntStanE->setEnabled(FALSE);
	delete ntStanO; ntStanO = 0;
	delete ntStanE; ntStanE = 0;
	if(source->debuggy()) //failure AFTER STOPPING THE QSOCKETS
	{
	    QMessageBox p;
	    p.setIcon(QMessageBox::Warning);
	    p.setText(
"There were messages generated during compilation.\nCheck debug frame in Source Editor.");
	    p.show();
	}

    }	
    else
	compileWindow->goDown(buff);

    delete [] buff;
}

void
MenuBar::makeProgram()
{
	if(!BaseComponent::at(0)) return;

	//compile
	compileProgram(0,0);
}

void
MenuBar::makeProgramAll()
{
        if(!BaseComponent::at(0)) return;
        QString i;

	//make clean
	i = "cd ";
	i += BaseComponent::queryFinalPath().data(); 
	i += "; ";

	i += configs->queryMak();
	i += " clean";
	system(i.data());

	//compile
	compileProgram(0,0);
}

void
MenuBar::runProgram()
{
	if(!BaseComponent::at(1)) return;

	//compile & run it
	compileProgram(1,0);

}

void
MenuBar::debugProgram()
{
        if(!BaseComponent::at(1)) return;

        //compile & run it
        compileProgram(1,1);

}

void
MenuBar::stopProgram()
{
	compileWindow->hide();
	if(!childPID) return;

	kill(childPID,SIGKILL);
	childPID = 0;
	waitpid(0,0,0);
}

void
MenuBar::copyComponent()
{
	if(BaseComponent::at(0)) BaseComponent::at(0)->copyComp();
}

void
MenuBar::raiseS()
{
	int x = BaseComponent::queryCurrObj();
	if(BaseComponent::at(x)) BaseComponent::at(x)->raiseComp();
}

void
MenuBar::lowerS()
{
        int x = BaseComponent::queryCurrObj();
        if(BaseComponent::at(x)) BaseComponent::at(x)->lowerComp();
}

void
MenuBar::lockS()
{
        int x = BaseComponent::queryCurrObj();
        if(BaseComponent::at(x) && BaseComponent::at(x)->parentWidget()) 
		BaseComponent::at(x)->toggleLock();
}

void
MenuBar::cutComponent()
{
	if(BaseComponent::at(0)) BaseComponent::at(0)->cutComp();
}

void
MenuBar::pasteComponent()
{
        if(!BaseComponent::at(BaseComponent::queryCurrPar())) return;

	int oat;
	QString temp;
	project = BaseComponent::at(BaseComponent::queryCurrPar());
	
        temp.sprintf("%s/QtEZ/buffer.qtz",getenv("HOME"));

	oat = CompIO::load(temp,O_PASTE);

	//Update slots
	SignalConnect *s;
	LinkList *l = BaseComponent::at(oat)->membInfo();
	for(int x = 0; x < l->connectCount(); x++)
	{
		s = l->connect(x);
		if(BaseComponent::at(s->querySlotOwner())->parentWidget() &&
		   BaseComponent::at(s->querySlotOwner())->parentWidget() !=
		   BaseComponent::at(oat)->parentWidget())
		{
			l->removeConnection(x--);
		}
		else
		{   
			s->setSigOwner(oat);
		}
	}

	changeMade();
}

bool 
MenuBar::event (QEvent *e)
{
	if(e->type() == Event_Close)
	{
		if(!clearWidgets()) { return FALSE; }
		end();
	}
	else if(e->type() == Event_Resize)
	{
		hintButt->move(width()-25,30);
		theMenu->resize(width(),theMenu->height());
		chang->move(width()-120,30);
		xyLabel->move(width()-190,30);
	}
	return TRUE;
}

bool 
MenuBar::eventFilter(QObject *ob, QEvent *e)
{
	if(e->type() == Event_Paint)
	{
		if(ob == projView)
			windowMenu->setItemChecked(projectId,TRUE);
		if(ob == source)
			windowMenu->setItemChecked(srcEdId,TRUE);
		if(ob == toolBar)
			windowMenu->setItemChecked(toolBarId,TRUE);
		if(ob == global)
			windowMenu->setItemChecked(glbEdId,TRUE);
		if(ob == menu)
			windowMenu->setItemChecked(mnuEdId,TRUE);
		if(ob == attrib)
			windowMenu->setItemChecked(attribId,TRUE);
		if(ob == sigSlot)
			windowMenu->setItemChecked(sigSlotId,TRUE);
		if(ob == configs)
			windowMenu->setItemChecked(configId,TRUE);
	}
	if(e->type() == Event_Close)
	{
		if(ob == projView)
			windowMenu->setItemChecked(projectId,FALSE);
                if(ob == source)
                        windowMenu->setItemChecked(srcEdId,FALSE);
                if(ob == toolBar)
                        windowMenu->setItemChecked(toolBarId,FALSE);
                if(ob == global)
                        windowMenu->setItemChecked(glbEdId,FALSE);
                if(ob == menu)
                        windowMenu->setItemChecked(mnuEdId,FALSE);
                if(ob == attrib)
                        windowMenu->setItemChecked(attribId,FALSE);
                if(ob == sigSlot)
                        windowMenu->setItemChecked(sigSlotId,FALSE);
                if(ob == configs)
		{
                        windowMenu->setItemChecked(configId,FALSE);
			configs->saveYourself();
		}
	}

	return FALSE;
}

void
MenuBar::xy(int x, int y)
{
	QString temp;

	temp.sprintf("X:%d Y:%d",x,y);
	xyLabel->setText(temp.data());
}


void 
MenuBar::showLevel(int x) 
{ 
    if(!level)
    {
	level = new QProgressDialog("Wait..","Cancel",x,this);
	level->show();
    }
}

void 
MenuBar::stepLevel() 
{ 
   if(level) level->setProgress(level->progress()+1); 
}

int
MenuBar::wasLevelCanceled()
{
    if(level && level->wasCancelled())
    {
	delete level;
	level = 0;
	return 1;
    }
    return 0;
}
	
void 
MenuBar::hideLevel() 
{ 
    if(level)
    {
	level->setProgress(level->totalSteps());
	level->hide(); 
	delete level;
	level = 0;
    }
}
