/*  $Id: MnuEditor.cc,v 1.5 1998/07/27 16:35:00 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include <qstrlist.h>
#include "components/Deserialize.h"
#include "MnuEditor.h"

/* These account for the "hidden" 0 position taken by the Application object */
#define FROMTOPLEV(x) BaseComponent::fromTopLev(x+1)
#define TOTOPLEV(x) BaseComponent::toTopLev(x)-1

MnuEditor::MnuEditor(QWidget *parent, char *name) 
	: QWidget(parent,name,WStyle_Customize | WStyle_NormalBorder )
{
	currLevel = 0;
	bc = 0;
	passOn = 1;
	setCaption("Menubar Editor");
	resize(428,507);
	setMaximumSize(428,507);
	setMinimumSize(428,507);
	
	objList = new QComboBox(FALSE,this,"objList");
	objList->setFont(QFont("times",14,QFont::Normal,(int)0));
        objList->setGeometry(14,5,380,25);
	ADDTOOLTIP(objList,"Select object housing menu you wish to edit.");
	QObject::connect(objList,SIGNAL(activated(int)),
			  this,SLOT(choseTop(int)));

	hintButt = new Hint("Menubar Editor:\nTo use this dialog select the object which\nhouses the menubar you want to edit. There is\nalways at least one blank menu in the listbox\nbelow. Select anyone to edit (if you select the\nblank one, a new blank one will appear).\nFill out all the fields you wish to fill out, and then set\nthe level of the item with the < and > button at the\nbottom of the dialog. Items of the same level will\nbe on the same menu, and as you go to the right\nthe more menus in they will be. Then finally you can select\nthe object you wish to connect this item to\nin the left combo box, and the speicific slot in the right one. You \ncan insert and remove items with their respecitive button at \nthe bottom.",this,"hintButt");
	hintButt->move(394,5);

	label2 = new QLabel(this, "label2");
	label2->setText("Name:");
	label2->setFont(QFont("times",14,QFont::Normal,(int)0));
	label2->setAlignment(AlignLeft | AlignTop);
	label2->setFrameStyle(QFrame::Raised | QFrame::Box);
	label2->setGeometry(14,130,269,82);
	
	label1 = new QLabel(this, "label1");
	label1->setText("Caption");
	label1->setFont(QFont("times",14,QFont::Normal,(int)0));
	label1->setAlignment(AlignLeft | AlignTop);
	label1->setFrameStyle(QFrame::Raised | QFrame::Box);
	label1->setGeometry(9,46,277,82);
	
	addMnuItem = new QPushButton(this, "addMnuItem");
	addMnuItem->setText("&Insert");
        addMnuItem->setFont(QFont("times",14,QFont::Normal,(int)0));
        addMnuItem->setEnabled((bool)1);
        addMnuItem->setDefault((bool)0);
        addMnuItem->setAutoResize((bool)0);
        addMnuItem->setGeometry(112,428,95,30);
        ADDTOOLTIP(addMnuItem,"Click to insert new item at current one.");
        QObject::connect(addMnuItem,SIGNAL(clicked()),this,SLOT(addItm()));

	rmvMnuItem = new QPushButton(this, "rmvMnuItem");
	rmvMnuItem->setText("&Remove");
	rmvMnuItem->setFont(QFont("times",14,QFont::Normal,(int)0));
	rmvMnuItem->setEnabled((bool)1);
	rmvMnuItem->setDefault((bool)0);
	rmvMnuItem->setAutoResize((bool)0);
	rmvMnuItem->setGeometry(232,428,95,30);
	ADDTOOLTIP(rmvMnuItem,"Click to remove current menu item.");
	QObject::connect(rmvMnuItem,SIGNAL(clicked()),this,SLOT(rmvItm()));
	
	stpDown = new QPushButton(this, "stpDown");
	stpDown->setText("<");
	stpDown->setFont(QFont("times",14,QFont::Normal,(int)0));
	stpDown->setEnabled((bool)1);
	stpDown->setDefault((bool)0);
	stpDown->setAutoResize((bool)0);
	stpDown->setGeometry(13,428,55,30);
	ADDTOOLTIP(stpDown,"Click to descend current menu item one level.");
	QObject::connect(stpDown,SIGNAL(clicked()),this,SLOT(downLevel()));
	
	enab = new QCheckBox(this, "enab");
	enab->setText("Enabled");
	enab->setFont(QFont("times",14,QFont::Normal,(int)0));
	enab->setChecked((bool)1);
	enab->setGeometry(180,212,100,19);
	ADDTOOLTIP(enab,"Click if current menu item should be enabled.");
        QObject::connect(enab,SIGNAL(clicked()),this,SLOT(takeEffect()));
	
	chkd = new QCheckBox(this, "chkd");
	chkd->setText("Checked");
	chkd->setFont(QFont("times",14,QFont::Normal,(int)0));
	chkd->setChecked((bool)0);
	chkd->setGeometry(29,212,129,19);
	ADDTOOLTIP(chkd,"Click if current menu item should be checked.");
        QObject::connect(chkd,SIGNAL(clicked()),this,SLOT(takeEffect()));
	
	nam = new QLineEdit(this, "nam");
	nam->setFont(QFont("times",14,QFont::Normal,(int)0));
	nam->setFrame((bool)1);
	nam->setEchoMode(QLineEdit::Normal);
	nam->setGeometry(22,158,249,37);
	ADDTOOLTIP(nam,"Enter name if current menu item is a new level.");
	QObject::connect(nam,SIGNAL(textChanged(const char *)),
			 this,SLOT(takeEffect(const char *)));
	
	capt = new QLineEdit(this, "capt");
	capt->setFont(QFont("times",14,QFont::Normal,(int)0));
	capt->setFrame((bool)1);
	capt->setEchoMode(QLineEdit::Normal);
	capt->setGeometry(20,77,252,39);
	ADDTOOLTIP(capt,"Enter the caption for the curret menu item.");
        QObject::connect(capt,SIGNAL(textChanged(const char *)),
                         this,SLOT(takeEffect(const char *)));
	
	mnuList = new myListBox(this, "mnuList");
	mnuList->setAutoScrollBar((bool)1);
	mnuList->setAutoBottomScrollBar((bool)1);
	mnuList->setDragSelect((bool)1);
	mnuList->setBottomScrollBar((bool)0);
	mnuList->setScrollBar((bool)0);
	mnuList->setGeometry(13,235,405,192);
	mnuList->setFont(QFont("times",14,QFont::Normal,(int)0));
	ADDTOOLTIP(mnuList,"Select the menu item you wish to edit.");
        QObject::connect(mnuList,SIGNAL(highlighted(int)),
			 this,SLOT(selectMnu(int)));	
	
	stpUp = new QPushButton(this, "stpUp");
	stpUp->setText(">");
	stpUp->setFont(QFont("times",14,QFont::Normal,(int)0));
	stpUp->setEnabled((bool)1);
	stpUp->setDefault((bool)0);
	stpUp->setAutoResize((bool)0);
	stpUp->setGeometry(359,429,56,30);
	ADDTOOLTIP(stpUp,"Click to ascend current menu item one level.");
	QObject::connect(stpUp,SIGNAL(clicked()),this,SLOT(upLevel()));
	
	shtcutList = new QListBox(this, "shtcutList");
	shtcutList->setAutoScrollBar((bool)1);
	shtcutList->setAutoBottomScrollBar((bool)1);
	shtcutList->setDragSelect((bool)1);
	shtcutList->setBottomScrollBar((bool)0);
	shtcutList->setScrollBar((bool)0);
	shtcutList->setGeometry(294,46,125,120);
	shtcutList->setFont(QFont("times",14,QFont::Normal,(int)0));
	ADDTOOLTIP(shtcutList,"Select keystroke to be a shortcut.");
	QObject::connect(shtcutList,SIGNAL(highlighted(int)),
			 this,SLOT(takeEffect(int)));
	fillShrtCut();

	sftK = new QCheckBox("Shift",this,"sftK");
	sftK->setFont(QFont("times",14,QFont::Normal,(int)0));
	sftK->setGeometry(294,170,119,20);
	ADDTOOLTIP(sftK,"Check if shift key is a modifier to shortcut.");
	QObject::connect(sftK,SIGNAL(clicked()),
			this,SLOT(takeEffect()));

        altK = new QCheckBox("Alt",this,"altK");
        altK->setFont(QFont("times",14,QFont::Normal,(int)0));
        altK->setGeometry(294,192,119,20);
	ADDTOOLTIP(altK,"Check if alt key is a modifier to shortcut.");
        QObject::connect(altK,SIGNAL(clicked()),
			this,SLOT(takeEffect()));

        ctlK = new QCheckBox("Control",this,"ctlK");
        ctlK->setFont(QFont("times",14,QFont::Normal,(int)0));
        ctlK->setGeometry(294,214,119,20);
	ADDTOOLTIP(ctlK,"Check if control key is a modifier to shortcut.");
        QObject::connect(ctlK,SIGNAL(clicked()),
			this,SLOT(takeEffect()));

	slotList = new QComboBox(FALSE,this, "slotList");
	slotList->setFont(QFont("times",14,QFont::Normal,(int)0));
	slotList->setGeometry(225,468,191,30);
	ADDTOOLTIP(slotList,"Select slot to connect current menu item to.");
	QObject::connect(slotList,SIGNAL(activated(int)),
			 this,SLOT(takeEffect(int)));
	
	objSlotList = new QComboBox(FALSE,this, "objSlotList");
	objSlotList->setAutoResize((bool)0);
	objSlotList->setMaxCount(2147483647);
	objSlotList->setFont(QFont("times",14,QFont::Normal,(int)0));
	objSlotList->setGeometry(24,469,194,30);
	objSlotList->insertItem("None");
	ADDTOOLTIP(objSlotList,"Select object housing slot you with to connect to.");
	QObject::connect(objSlotList,SIGNAL(activated(int)),
                         this,SLOT(choseSlotObj(int)));

	
	mnuList->insertItem(new MnuItem(currLevel));
	mnuList->setCurrentItem(0);
}

void
MnuEditor::selectMnu(int)
{
	passOn = 0;
	int te;

	sftK->setChecked(0);
	altK->setChecked(0);
	ctlK->setChecked(0);
	shtcutList->setCurrentItem(0);
	shtcutList->setTopItem(0);

        MnuItem *m = (MnuItem *)mnuList->zItem(mnuList->currentItem());
	
	capt->setText(m->queryCapt());
	nam->setText(m->queryName());

	slotList->clear();
	te = m->queryConO();
	if(te == -1)
		objSlotList->setCurrentItem(0);
	else 
	 {
	     objSlotList->setCurrentItem(BaseComponent::positionToBox(te)+1);

  	     QStrList *p = BaseComponent::at(te)->querySlots();
	     slotList->insertStrList(p);
	     if(m->queryConS() < slotList->count())
		 slotList->setCurrentItem(m->queryConS());
	     delete p;
	}

	QString sht = m->queryShrtCut();
	if(!sht.isEmpty())
	{
		if(sht.find("ALT+") != -1) altK->setChecked(1);
		if(sht.find("CTRL+") != -1) ctlK->setChecked(1);
		if(sht.find("SHIFT+") != -1) sftK->setChecked(1);

		sht = sht.right(sht.length()-sht.findRev('+')-1);

		for(int y = 0; y < shtcutList->count(); y++)
			if(sht == shtcutList->text(y))
			{
				 shtcutList->setCurrentItem(y);
				 shtcutList->setTopItem(y);
			}
	}
	
	chkd->setChecked((bool)m->queryChk());
	enab->setChecked((bool)m->queryEna());
	currLevel = m->queryLev();
	passOn = 1;
}
		
void
MnuEditor::choseTop(int x)
{
	changeToObject(FROMTOPLEV(x));
}

void
MnuEditor::changeToObject(int xxx)
{
    BaseComponent *toChange = BaseComponent::at(xxx);
    if(toChange && toChange->parentWidget())
    {
	toChange = (BaseComponent *)toChange->parentWidget();
	xxx = toChange->queryPosition();
    }
    if((toChange == bc && !mnuListIsEmpty()) || 
       BaseComponent::queryAction() == A_OPEN) return;
        
    objList->setCurrentItem(TOTOPLEV(xxx));

    if((bc = BaseComponent::at(xxx)))
    {
	LinkList *tmp = BaseComponent::at(xxx)->membInfo();
	mnuList->setAutoUpdate(FALSE);
	mnuList->clear();
	for(int x = 0; x < tmp->menuCount(); x++)
		mnuList->insertItem(
		  new MnuItem(tmp->menu(x)->queryCapt(), tmp->menu(x)->queryName(),
		              tmp->menu(x)->queryConO(), tmp->menu(x)->queryConS(), 
		    	      tmp->menu(x)->queryShrtCut(), tmp->menu(x)->queryChk(),
			      tmp->menu(x)->queryEna(), tmp->menu(x)->queryLev()));
	mnuList->setAutoUpdate(TRUE);
	mnuList->repaint();
    }

    mnuList->insertItem(new MnuItem(0));
    mnuList->setCurrentItem(0);
}
	
int
MnuEditor::mnuListIsEmpty()
{
    QString ca("");
    if(mnuList->count() == 1)
    {
	MnuItem *ot = (MnuItem *)mnuList->zItem(0);
	ca = ot->queryCapt();
    }

    return (!mnuList->count() || ca.isEmpty());
}

void
MnuEditor::upLevel()
{
	MnuItem *ot = (MnuItem *)mnuList->zItem(mnuList->currentItem()-1);
	if(currLevel < 10 && mnuList->currentItem() > 0 && 
	   currLevel <= ot->queryLev()) 
	{
		currLevel++;
		takeEffect();
	}
}

void
MnuEditor::choseSlotObj(int te)
{
    slotList->clear();

    if(te)
    {
	QStrList *p = BaseComponent::at(
	    BaseComponent::boxToPosition(te-1))->querySlots();
	slotList->insertStrList(p);
	delete p;
    }
    takeEffect();
}

void
MnuEditor::downLevel()
{
	if(currLevel > 0) currLevel--;
	takeEffect();
}

void
MnuEditor::takeEffect()
{
	if(mnuList->currentItem() == -1 || !bc) return;
	MnuItem *m;

	if(mnuList->currentItem() == mnuList->count()-1)
        	mnuList->insertItem(new MnuItem(currLevel));

	QString sht = "";
	if(shtcutList->currentItem())
	{
		if(altK->isChecked()) sht += "ALT+";
		if(ctlK->isChecked()) sht += "CTRL+";
		if(sftK->isChecked()) sht += "SHIFT+";
		sht += shtcutList->text(shtcutList->currentItem());
	}

	QString nme = nam->text();
	if(mnuList->currentItem())
		m = (MnuItem *)mnuList->zItem(mnuList->currentItem()-1);
	else
                m = (MnuItem *)mnuList->zItem(mnuList->currentItem());

	if(nme.isEmpty() && (!currLevel || currLevel > m->queryLev()))
	{
		nme = "blank1";
		QString te;
		for(int x = 0,y=1; x < mnuList->count(); x++)
		{
			m = (MnuItem *)mnuList->zItem(x);
			te = m->queryName();
			if(nme == te)
			{
				x = 0;
				nme.sprintf("blank%d",++y);
			}
		}
	}
	nam->setText(nme.data());

	int Co;
	if(!objSlotList->currentItem())
		Co = -1;
	else
		Co = BaseComponent::boxToPosition(objSlotList->currentItem()-1);

	m = (MnuItem *)mnuList->zItem(mnuList->currentItem());
	m->setItem(capt->text(),nam->text(),
	Co, slotList->currentItem(),
	sht.data(),
	(int)chkd->isChecked(), (int)enab->isChecked(),currLevel);
	mnuList->repaint();
	if(bc)
	bc->modifyMenu(mnuList->currentItem(), capt->text(),nam->text(),
	        Co, slotList->currentItem(), sht.data(),
	        (int)chkd->isChecked(), (int)enab->isChecked(),currLevel);
}

void
MnuEditor::takeEffect(int)
{
	if(passOn) takeEffect();

}

void
MnuEditor::takeEffect(const char *)
{
	if(passOn) takeEffect();
}

void
MnuEditor::addItm()
{
        if(mnuList->currentItem() == -1 || !objList->count()) return;

	mnuList->insertItem(new MnuItem(0),mnuList->currentItem());
        if(bc) bc->insertMenu(mnuList->currentItem()-1);
}

void
MnuEditor::rmvItm()
{
	if(mnuList->currentItem() == -1 || !objList->count()) return;
	mnuList->removeItem(mnuList->currentItem());
	if(bc) bc->removeMenu(mnuList->currentItem()+1);
}

void
MnuEditor::addObject(char *desc,int where)
{
	objSlotList->insertItem(desc,where+1);
}
	
void
MnuEditor::addParent(char *desc)
{
	objList->insertItem(desc);
	if(objList->count() == 1) choseTop(0);
}

void
MnuEditor::removeObject(int x)
{
        BaseComponent *par;
        par = (BaseComponent *) BaseComponent::at(x)->parentWidget();
        if(par &&
           par->queryPosition() == BaseComponent::queryCurrPar())
        {
		mnuList->clear();
		shtcutList->setCurrentItem(0);	
		capt->setText("");
		nam->setText("");	
		altK->setChecked(0);
		sftK->setChecked(0);
		ctlK->setChecked(0);
		objList->setCurrentItem(0);
                slotList->clear();

                objSlotList->removeItem(
			BaseComponent::positionToBox(x));
        }

	if(!BaseComponent::at(x)->parentWidget())  
		objList->removeItem(TOTOPLEV(x));
}

void
MnuEditor::changeParent(char *desc, int x)
{
	if(TOTOPLEV(x) >= 0) objList->changeItem(desc,TOTOPLEV(x));
}

void
MnuEditor::changeObject(char *desc, int x)
{
	objSlotList->changeItem(desc,x+1);
}

int
MnuEditor::getTop(int)
{
 	return 0;
}

void
MnuEditor::fillShrtCut()
{
	shtcutList->insertItem("None");
	shtcutList->insertItem("Key_A");
        shtcutList->insertItem("Key_B");
        shtcutList->insertItem("Key_C");
        shtcutList->insertItem("Key_D");
        shtcutList->insertItem("Key_E");
        shtcutList->insertItem("Key_F");
        shtcutList->insertItem("Key_G");
        shtcutList->insertItem("Key_H");
        shtcutList->insertItem("Key_I");
        shtcutList->insertItem("Key_J");
        shtcutList->insertItem("Key_K");
        shtcutList->insertItem("Key_L");
        shtcutList->insertItem("Key_M");
        shtcutList->insertItem("Key_N");
        shtcutList->insertItem("Key_O");
        shtcutList->insertItem("Key_P");
        shtcutList->insertItem("Key_Q");
        shtcutList->insertItem("Key_R");
        shtcutList->insertItem("Key_S");
        shtcutList->insertItem("Key_T");
        shtcutList->insertItem("Key_U");
        shtcutList->insertItem("Key_V");
        shtcutList->insertItem("Key_W");
        shtcutList->insertItem("Key_X");
        shtcutList->insertItem("Key_Y");
        shtcutList->insertItem("Key_Z");
        shtcutList->insertItem("Key_0");
        shtcutList->insertItem("Key_1");
        shtcutList->insertItem("Key_2");
        shtcutList->insertItem("Key_3");
        shtcutList->insertItem("Key_4");
        shtcutList->insertItem("Key_5");
        shtcutList->insertItem("Key_6");
        shtcutList->insertItem("Key_7");
        shtcutList->insertItem("Key_8");
        shtcutList->insertItem("Key_9");
	shtcutList->insertItem("Key_Escape");
	shtcutList->insertItem("Key_Tab");	
	shtcutList->insertItem("Key_Backtab");
	shtcutList->insertItem("Key_Backspace");
	shtcutList->insertItem("Key_Return");
	shtcutList->insertItem("Key_Enter");
	shtcutList->insertItem("Key_Pause");
	shtcutList->insertItem("Key_Print");
	shtcutList->insertItem("Key_SysReq");
	shtcutList->insertItem("Key_Home");
	shtcutList->insertItem("Key_End");
	shtcutList->insertItem("Key_Left");
	shtcutList->insertItem("Key_Up");
	shtcutList->insertItem("Key_Down");
	shtcutList->insertItem("Key_Prior");
	shtcutList->insertItem("Key_Next");
	shtcutList->insertItem("Key_Shift");
	shtcutList->insertItem("Key_Meta");
	shtcutList->insertItem("Key_Alt");
	shtcutList->insertItem("Key_CapsLock");
	shtcutList->insertItem("Key_NumLock");
	shtcutList->insertItem("Key_ScrollLock");
        shtcutList->insertItem("Key_Space");
	shtcutList->insertItem("Key_F1");
        shtcutList->insertItem("Key_F2");
        shtcutList->insertItem("Key_F3");
        shtcutList->insertItem("Key_F4");
        shtcutList->insertItem("Key_F5");
        shtcutList->insertItem("Key_F6");
        shtcutList->insertItem("Key_F7");
        shtcutList->insertItem("Key_F8");
        shtcutList->insertItem("Key_F9");
        shtcutList->insertItem("Key_F10");
        shtcutList->insertItem("Key_F11");
        shtcutList->insertItem("Key_F12");
        shtcutList->insertItem("Key_F13");
        shtcutList->insertItem("Key_F14");
        shtcutList->insertItem("Key_F15");
        shtcutList->insertItem("Key_F16");
        shtcutList->insertItem("Key_F17");
        shtcutList->insertItem("Key_F18");
        shtcutList->insertItem("Key_F19");
        shtcutList->insertItem("Key_F20");
        shtcutList->insertItem("Key_F21");
        shtcutList->insertItem("Key_F22");
        shtcutList->insertItem("Key_F23");
        shtcutList->insertItem("Key_F24");
}

void
MnuEditor::clearObjects()
{
	objSlotList->clear();
	objSlotList->insertItem("None");
	slotList->clear();
}

void
MnuEditor::clearList()
{
	mnuList->clear();
	capt->setText("");
	nam->setText(""); 
	shtcutList->setCurrentItem(0);
	chkd->setChecked(FALSE);
	enab->setChecked(TRUE);
	altK->setChecked(FALSE);
	ctlK->setChecked(FALSE);
	sftK->setChecked(FALSE);
}

