/*  $Id: ProjectViewer.cc,v 1.5 1998/07/27 16:35:01 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "ProjectViewer.h"
#include "components/BaseComp.h"

/**********************************************************************/

int ProjectItem::wid = 0;

#include <qobjcoll.h>

void
ProjectItem::paint(QPainter *p)
{
        p->setFont(QFont(configs->queryFontN(), configs->queryFontS()));
        QFontMetrics fm = p->fontMetrics();
        int y = fm.ascent() + fm.leading() / 2;
	
	QString tempCap = cptn.data();
	if(hasChanged) tempCap += " (*)";
	p->drawText((level*PROJECT_SEP)+PROJECT_SEP,y,tempCap.data());

	/* Draw half the height a vertical line */
	if(level != 1)
		p->drawLine((level-1)*PROJECT_SEP,0,
		    (level-1)*PROJECT_SEP, configs->queryFontHeight() / 2);

	/* Draw the horizontal line */
	p->drawLine((level-1)*PROJECT_SEP, configs->queryFontHeight() / 2,
		    level*PROJECT_SEP, configs->queryFontHeight() / 2);
	
	if(isParent)
	{
	    /* Draw the "collapsable" box */
		int quart = configs->queryFontHeight() / 4;
		p->drawRect(level*PROJECT_SEP-quart,
		(configs->queryFontHeight()/2)-quart,quart*2,quart*2);
	}

	if(!isLast)
	{
	   if(isParent && childShow)
	   {
	       /* Draw final half vertical line */
		p->drawLine(level*PROJECT_SEP, configs->queryFontHeight() / 2,
		    level*PROJECT_SEP,configs->queryFontHeight()+3);
                    }

	   /* Draw all the full vertical lines leading up to my level */
	   for(int x = 1; x < level; x++)
	       p->drawLine(x*PROJECT_SEP,0,
		x*PROJECT_SEP,configs->queryFontHeight()+3);
	}
}

int 
ProjectItem::validX(int x)
{
    int quart = configs->queryFontHeight() / 4;
    return ( (x >  level*PROJECT_SEP-quart) && (x < level*PROJECT_SEP+quart) );
}

int 
ProjectItem::width(const QListBox *lb) const
{ 
	if(!wid) wid = lb->width(); 
	return wid-5; 
}

int
ProjectItem::height(const QListBox *) const
{
	return configs->queryFontHeight()+3;
}

/***********************************************************************/


ProjectViewer::ProjectViewer(QWidget *parent, char *name) : 
	QWidget(parent,name)
{
	setCaption("Project Viewer");
	resize(270,370);
	baseList.setAutoDelete(TRUE);

	projList = new myListBox(this,"projList");
	projList->setGeometry(0,0,width(),height());	
	projList->installEventFilter(this);
	QObject::connect(projList,SIGNAL(highlighted(int)),
			this,SLOT(highlighted(int)));

	hintButt = new Hint("Project Viewer:\nThis will allow you to glance at your project\nso the details don't get in the way and you\ncan clearly illustrate the layout. You can treat\neach item in the list as if they were\nthe real component they represent.\nSelecting, double clicking, and right clicking will\nperform similar actions as if they\nwere applied to the component. This list will\nalso include parent widgets that\nwere \"closed\", and double or right clicking\nwill redisplay them.",this,"hintButt");
	hintButt->move(width()-hintButt->width(),0);

}

void
ProjectViewer::changeToObject(int x)
{
    if(BaseComponent::queryAction() == A_OPEN) return;

    if(baseList.at(x)->isVis()) projList->setCurrentItem(baseList.at(x)->ref()); 
}

void
ProjectViewer::resizeEvent(QResizeEvent *)
{
	int more = 0;

	projList->setGeometry(0,0,width(),height());
	if(projList->scrollBar() == TRUE) more = 15;
	hintButt->move((width()-hintButt->width())-more,0);
}

void
ProjectViewer::buildTree()
{
	int *arr = new int[BaseComponent::count()];
	int x,lastParent=0;
	int isParent = 0, parent;

	baseList.clear();

	for(x = 0; x < BaseComponent::count(); x++)
	{
 	    arr[x] = 0;
            isParent = 0;

	    BaseComponent *tmp;
	    tmp = (BaseComponent *) BaseComponent::at(x)->parentWidget();
	    parent = tmp ? tmp->queryPosition() : 0;

            if(BaseComponent::at(x)->numChildren() || 
	       !BaseComponent::at(x)->parentWidget()) 
		isParent = 1; 

	    arr[x] = x ? arr[parent]+1 : 1;

	    baseList.append(new RefBaseComp(arr[x],isParent,1));
	    BaseComponent::at(x)->setLevel(arr[x]);

	}
	delete [] arr;

	showBuiltTree();
}

void
ProjectViewer::highlighted(int match)
{
    if(BaseComponent::queryAction() == A_OPEN) return;

    ProjectItem *hold = (ProjectItem *)projList->zItem(match);
    int y = hold->queryPosition();
    if(y == BaseComponent::queryCurrObj()) return;
    BaseComponent::at(y)->giveFocus();
}

void
ProjectViewer::showBuiltTree()
{
    RefBaseComp *temp;
    int invis = 0;

    projList->setAutoUpdate(FALSE);
    projList->clear();
    for(int x = 0, y = 0; x < baseList.count(); x++)
    {
	BaseComponent *tmpComp = BaseComponent::at(x);
	temp = baseList.at(x);
	    
	if(temp->lev() <= invis) invis = 0;
	temp->setVis(!invis);

	if(!invis && tmpComp)
	{
	    int hasChanged = tmpComp->hasChanged() && (tmpComp->numChildren() ||
			     !tmpComp->noFuncs());

	    QString i;
	    i = tmpComp->qualifiedName();
	    projList->insertItem(new ProjectItem(i.data(),
					      temp->lev(), temp->par(), x, 
				      tmpComp->showChildren(),
				      hasChanged));
	    temp->setRef(y++);
	}

	if(!invis && !tmpComp->showChildren()) invis = temp->lev();
    }
    if(projList->count())
    {
	ProjectItem *t = (ProjectItem *)projList->zItem(projList->count()-1);
	t->setLast(1);
    }

    int toBe = BaseComponent::queryCurrObj();
    if(toBe > 0 && baseList.at(toBe)->isVis())
    {
	for(int y = 0; y < toBe; y++){ if(!baseList.at(y)->isVis()) toBe--; }
	projList->setCurrentItem(toBe);
    }

    projList->setAutoUpdate(TRUE);
    projList->repaint();
}

void
ProjectViewer::moveObject(int was, int is)
{
	int hLevel = baseList.at(was)->lev(), hPar=baseList.at(was)->par();
	baseList.remove(was);

	baseList.insert(is,new RefBaseComp(hLevel,hPar,1));	
}

bool
ProjectViewer::eventFilter(QObject *, QEvent *e)
{
    if(projList->count() == 0) return FALSE;

    if(e->type() == Event_MouseButtonPress)
    {
	QMouseEvent *m = (QMouseEvent *)e;
	if(m->button() == RightButton)
	{
	    ProjectItem *hold = (ProjectItem *)projList->zItem(
		projList->currentItem());
	    int y = hold->queryPosition();

	    BaseComponent::at(y)->popShow(mapToGlobal(m->pos()));
	}
    }
    else if(e->type() == Event_MouseButtonDblClick)
    {
	QMouseEvent *m = (QMouseEvent *)e;
	ProjectItem *hold = (ProjectItem *)projList->zItem(
	    projList->currentItem());
	int y = hold->queryPosition();

	if(baseList.at(y)->par() && hold->validX(m->x()))
	{
	    BaseComponent::at(y)->toggleShowChildren();
	    showBuiltTree();	
	    return TRUE;
	}
	else
	{
	    if(!BaseComponent::at(y)->invisibleComp()) 
		BaseComponent::at(y)->show();
	    if(!BaseComponent::at(y)->parentWidget())
		BaseComponent::at(y)->raise();
	}
    }
    else if(e->type() == Event_Paint) resize(width(),height());
    return FALSE;
}


void
ProjectViewer::removeObject(int x)
{
	baseList.remove(x); 
	showBuiltTree();
}
