/*  $Id: SigSlotEdit.cc,v 1.5 1998/07/30 22:15:37 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "SigSlotEdit.h"
#include "components/BaseComp.h"
#include "components/LinkList.h"
#include "YesNo.h"

#include "SourceEdit.h"
extern SourceEdit *source;

#include "Attributes.h"
extern Attributes *attrib;

/******************************************************************************/
void
SigSlotEditItem::paint(QPainter *p)
{
	if(!ena)
		p->setBrush(QColor("grey"));
	else
		p->setBrush(QColor("white"));

	p->setFont(QFont(configs->queryFontN(), configs->queryFontS()));       
	QFontMetrics fm = p->fontMetrics();

	int y = fm.ascent() + fm.leading() / 2;
	int h = configs->queryFontHeight();

	p->drawRect(2,2,h,h);

	if(chk)
	{
		p->save();
		p->setPen(QColor("black"));
		p->drawLine(2,2,h,h);
		p->drawLine(2,h,h,2);
		p->restore();
	}

	p->drawText(h*2,y,capt->data());
}

int
SigSlotEditItem::height(const QListBox *) const
{
	return configs->queryFontHeight() + 3; 
}

int
SigSlotEditItem::width(const QListBox *e) const
{
	return e->width();
}

/***************************************************************************/


SigSlotEdit::SigSlotEdit(QWidget *parent, char *name) 
	: QWidget(parent,name,WStyle_Customize | WStyle_NormalBorder )
{
	bcSig = 0; bcSlot = 0;

	setCaption("Signal/Slot Editor");
	resize(550,400);
	setBackgroundColor(QColor(192,192,192));
	
	label2 = new QLabel(this, "label2");
	label2->setText("SIGNALS:");
	label2->setAlignment(AlignLeft | AlignTop);
	label2->setFrameStyle(QFrame::Raised | QFrame::Box);
	label2->setGeometry(5,4,197,351);
	
	sigList = new myListBox(this, "sigList");
	sigList->setGeometry(12,90,181,246);
	QObject::connect(sigList,SIGNAL(highlighted(int)),
			 this,SLOT(oki(int)));
	ADDTOOLTIP(sigList,"Select signal to connect to here.");
	
	label1 = new QLabel(this, "label1");
	label1->setText("SLOTS:");
	label1->setAlignment(AlignLeft | AlignTop);
	label1->setFrameStyle(QFrame::Raised | QFrame::Box);
	label1->setGeometry(209,4,197,349);
	
	hintButt = new Hint("Signal & Slot Editor:\nThis editor is to allow easy connection of signals to\nslots. To accomplish this, select the object in the\nleft combobox that has the signal you want to connect\nfrom. Then a list of available signals will fill the leftmost\nlistbox. Select the signal in that box that you were\nlooking for, then in the right combobox chose the object\nhousing the slot you wish to connect to. Doing this will\npull available slots into the rightmost listbox. Incompatible\nslots will have a greyed out check box beside them,\nhowever you can still connect them if you wish, to connect\nwill be an X in the box next to the slot already\nconnected to. Simply doubleclick on the little box to\nthe left of a slot you want to toggle. After that it will\ntake effect immediatly.",this,"hintButt");	
	hintButt->move((label1->x() + label1->width()) - hintButt->width(),
			label1->y());

	slotList = new myListBox(this, "slotList");
	slotList->setGeometry(222,90,175,247);
	slotList->installEventFilter(this);
	ADDTOOLTIP(slotList,
		"Double click on box to toggle connected, or not.");
		
	slotObjList = new QComboBox((bool)0,this,"slotObjList");
	slotObjList->setGeometry(218,39,181,30);
        QObject::connect(slotObjList,SIGNAL(activated(int)),
                                this,SLOT(CSlotObj(int)));
	ADDTOOLTIP(slotObjList,
	  "Select object housing slot you wish to connect to.");

	
	sigObjList = new QComboBox((bool)0,this,"sigObjList");
	sigObjList->setGeometry(12,40,181,29);
        QObject::connect(sigObjList,SIGNAL(activated(int)),
                                this,SLOT(choseSigObj(int)));
	ADDTOOLTIP(sigObjList,
		"Select object housing signal you wish to connect from.");

}

void
SigSlotEdit::addObject(char *description, int change, int where)
{
        sigObjList->insertItem(description,where);
	slotObjList->insertItem(description,where);

        if(sigObjList->isEnabled() == FALSE) sigObjList->setEnabled(TRUE);
	if(slotObjList->isEnabled() == FALSE)slotObjList->setEnabled(TRUE);	

        if(!bcSig && change) changeToObject(0);
}

void
SigSlotEdit::changeToObject(int x)
{
        if(bcSig == BaseComponent::at(x) &&
           BaseComponent::queryAction() != A_OPEN) return;

	bcSig = BaseComponent::at(x);
	BaseComponent::at(x)->giveFocus(0);
        int y = BaseComponent::positionToBox(x);
        choseSigObj(y);
}

void
SigSlotEdit::choseSigObj(int obj)
{
	bcSig = BaseComponent::at(BaseComponent::boxToPosition(obj));
	
        if(bcSig)
	{
           bcSig->fillSigTable();
	   choseSlotObj(slotObjList->currentItem());
	}
        if(bcSig) sigObjList->setCurrentItem(
		BaseComponent::positionToBox(bcSig->queryPosition()));
}

void
SigSlotEdit::choseSlotObj(int obj)
{
        bcSlot = BaseComponent::at(BaseComponent::boxToPosition(obj));

        if(bcSlot)
	{
		bcSlot->fillSlotTable();
		slotObjList->setCurrentItem(obj);
	}
	renderConnected();
}


void 
SigSlotEdit::CSlotObj(int x)
{
	source->choseSlotObject(x);
	choseSlotObj(x); 
}

void
SigSlotEdit::addSlot(QString p, int mode)
{
    QString i;
    if(mode == M_PUB_SLOT)
	i.sprintf("SLOT(%s)",p.data());
    else
	i.sprintf("SIGNAL(%s)",p.data());

    slotList->insertItem(new SigSlotEditItem(i.data()));
}

void
SigSlotEdit::addSignal(QString p)
{
	QString i; i.sprintf("SIGNAL(%s)",p.data());
	sigList->insertItem(i.data());
}

void
SigSlotEdit::render(int x)
{
	slotList->setAutoUpdate((bool)x);
	sigList->setAutoUpdate((bool)x);
	if(x)
	{
		slotList->repaint();
		sigList->repaint();
		renderConnected();
	}
}

void
SigSlotEdit::enterEvent(QEvent *)
{
	QString x = configs->queryFontN();
	if(x != sigList->font().family() || 
	   configs->queryFontS() != sigList->font().pointSize())
	{
		sigList->setFont(QFont(configs->queryFontN(), 
			configs->queryFontS()));
		slotList->setFont(QFont(configs->queryFontN(), 
			configs->queryFontS()));

		sigList->repaint();
		slotList->repaint();
	}
//	renderConnected();
}

void
SigSlotEdit::resizeEvent(QResizeEvent *)
{
	label2->setGeometry(5,5,(width()/2)-15,height()-10);
	sigList->setGeometry(10,90,(width()/2)-25,height()-100);
	sigObjList->setGeometry(10,40,(width()/2)-25,30);


	label1->setGeometry((width()/2)+10,5,(width()/2)-15,height()-10);
	slotList->setGeometry((width()/2)+15,90,(width()/2)-25,height()-100);
	slotObjList->setGeometry((width()/2)+15,40,(width()/2)-25,30);
        hintButt->move((label1->x() + label1->width()) - hintButt->width(),
			label1->y());
}

void
SigSlotEdit::renderConnected()
{
    	int sig = sigList->currentItem(),s;
	if(sig == -1 || !sigObjList->count()) return;

	SigSlotEditItem *i;
	LinkList *l=BaseComponent::at(BaseComponent::boxToPosition(
	    sigObjList->currentItem()))->membInfo();
	
	SignalConnect con;
       con.setSigOwner(BaseComponent::boxToPosition(sigObjList->currentItem()));
	con.setSig(l->signal(sig));
     con.setSlotOwner(BaseComponent::boxToPosition(slotObjList->currentItem()));
	
	//Make connections...
	for(s = 0; s < slotList->count(); s++)
	{
		con.setSlot(s);
	
		i = (SigSlotEditItem *) slotList->zItem(s);		
		i->setChk(l->connected(con));
	}

	//Grey out incompatibles...
	QString slot, signal = sigList->text(sig);
	signal = signal.right(signal.length() - signal.findRev('('));
	signal = signal.left (signal.find(')')+1);
	removeWhiteSpace(signal);
	for(s = 0; s < slotList->count(); s++)
	{
		i = (SigSlotEditItem *) slotList->zItem(s);
		slot = i->statusCapt().data();
        	slot = slot.right(slot.length() - slot.findRev('('));
	        slot = slot.left (slot.find(')')+1);
		removeWhiteSpace(slot);

		i->setEna((signal == slot));
	}
	slotList->repaint();
}

void
SigSlotEdit::removeWhiteSpace(QString &f)
{
	int test;

	for(int x = 0; x < f.length(); x++)
	{
		test = (int) f.at(x);
		if(test == 9 || test == 10 || test == 11 || test == 12 ||
		   test == 13 || test == 32)
			f.remove(x--,1);
	}
}

bool
SigSlotEdit::eventFilter(QObject *obj, QEvent *e)
{
	if(e->type() == Event_MouseButtonDblClick && obj == slotList &&
	    sigObjList->count() != 0 && sigList->currentItem() != -1)
	{
		QMouseEvent *p = (QMouseEvent *)e;
		if(p->x() < (configs->queryFontS() + 5))
		{
			int doit = 1;
			SigSlotEditItem *i;
			i = (SigSlotEditItem *)slotList->zItem(
				    slotList->currentItem());	
			if(!i->statusEna() && !i->statusChk())
			{
	                	YesNo yn(0,"yn",350);
                		if(!yn.doit(
			  "Selected members are not compatible.\nAre you sure you wish to connect anyway?"))
					doit = 0;
			}
			if(doit)
			{
			     i->toggle();
			     BaseComponent::at(BaseComponent::boxToPosition(
				 currSigObj()))->applySource(1);
			     slotList->repaint();
			     if(bcSig) emit madeChange(bcSig->queryPosition());
			}
		}
	}
	return FALSE;
}

void
SigSlotEdit::removeObject(int x)
{
        BaseComponent *par;
        par = (BaseComponent *) BaseComponent::at(x)->parentWidget();
        if(par &&
           par->queryPosition() == BaseComponent::queryCurrPar())
        {
                sigList->clear();
                slotList->clear();

		removeSlot(BaseComponent::positionToBox(x));
		removeSig (BaseComponent::positionToBox(x));
        }
}
