/*  $Id: SourceEdit.cc,v 1.5 1998/08/08 04:24:02 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "SourceEdit.h"
#include "components/BaseComp.h"
#include "components/LinkList.h"
#include "SigSlotEdit.h"

#include "SigSlo.h"
extern SigSlotEdit *sigSlot;

#include "Globals.h"
extern Globals *global;

#include "Attributes.h"
extern Attributes *attrib;

#include <qregexp.h>

#include <stdlib.h>

Implement::Implement(QWidget *parent, char *name) : QWidget(parent,name)
{
	holdLN = 0;holdWho = -1;holdWhere = 0;bc = 0;bcS = 0;

	fullBug.setAutoDelete(TRUE); parseBug.setAutoDelete(TRUE);

	findD = new FindDialog(0,"findD");
	QObject::connect(findD,SIGNAL(findText(const char *)),
			 this,SLOT(findText(const char *)));

	modif = new MemberModifiers(0,"test");
	QObject::connect(modif,SIGNAL(member(char *,char *,int, int, int, int)),
			  this,SLOT(memberMod(char *,char *,int,int,int,int)));

	objectList = new QComboBox(FALSE,this,"objectList");
	ADDTOOLTIP(objectList,"Select object you wish to edit.");
	objectList->setGeometry(0,30,width()/2,25);
	objectList->setFocusPolicy(QWidget::NoFocus);
	QObject::connect(objectList,SIGNAL(activated(int)),
				this,SLOT(choseObject(int)));

	membList = new QComboBox(FALSE,this,"membList");
	ADDTOOLTIP(membList,"Select member to be edited");
	membList->setGeometry(width()/2,25,(width()/2)-25,25);
	membList->setFocusPolicy(QWidget::NoFocus);	
	QObject::connect(membList,SIGNAL(activated(int)),
				this,SLOT(choseMemb(int)));

	hintButt = new Hint("Implementation:\nTo use this dialog select the object you wish to edit\nin the left combo box. Then select the member you wish\nto edit. If it's just a normal function\nyou can type in the source code in the text editer\nbelow. If it's a signal then the editor will change\na little to accomidate a little viewer so you can\nsee what you have connected to that signal.\nYou can add, modify, and remove members under the file\nmenu on top.",this,"hintButt");
	hintButt->move(width()-25,30);
	hintButt->setFocusPolicy(QWidget::NoFocus);	

        theSource = new SyntaxEdit(this,"theSource");
        theSource->setGeometry(0,membList->y()+membList->height()+5,
			       width(), height()-membList->y()+membList->height());
	QObject::connect(theSource,SIGNAL(madeChange()),
		  	       this,SLOT(sourceChange()));
	theSource->setFocus();

	debugShowing = 0;
	debugInfo = new myListBox(this,"debugInfo");
	debugInfo->setGeometry(0,height()-60,width(),60);
	debugInfo->setFocusPolicy(QWidget::NoFocus);
	debugInfo->hide();
	QObject::connect(debugInfo,SIGNAL(highlighted(int)),
		this,SLOT(choseDebug(int)));

	objectListSlot = new QComboBox(FALSE,this,"objectListSlot");
	ADDTOOLTIP(objectListSlot,
		"Select object which houses slot to connect to.");
	objectListSlot->setGeometry(0,55,width(),25);
	objectListSlot->hide();
	objectListSlot->setFocusPolicy(QWidget::NoFocus);
	QObject::connect(objectListSlot, SIGNAL(activated(int)),
		                   this, SLOT(CSlotObj(int)));

	slotList = new myListBox(this,"slotList");
	slotList->setGeometry(0,80, width(),height()-80);
	slotList->setFocusPolicy(QWidget::NoFocus);
	slotList->hide();

        fileMenu = new QPopupMenu;
        fileMenu->insertItem("&New Member",this,SLOT(addMemb()));
        fileMenu->insertItem("&Delete Member",this,SLOT(delMemb()));
        fileMenu->insertItem("&Modify Member",this,SLOT(modMemb()));

	editMenu = new QPopupMenu;
        editMenu->insertItem("&Cut",theSource,SLOT(cut()));
        editMenu->insertItem("&Copy",theSource,SLOT(copyText()));
        editMenu->insertItem("&Paste",theSource,SLOT(paste()));
	editMenu->insertSeparator();
	editMenu->insertItem("&Find Text..",this,SLOT(findDialog()));
	editMenu->insertItem("&Replace..");

	#ifdef __USE_KDE
	        theMenu = new KMenuBar(this);
		theMenu->setGeometry(0,0,fileMenu->width(),theMenu->height());
	#else
	        theMenu = new QMenuBar(this);
	#endif
        theMenu->insertItem("&File",fileMenu);
        theMenu->insertItem("&Edit",editMenu);

}

void
Implement::choseSlot(int x)
{
	SigSlo *p = (SigSlo *)slotList->zItem(x);
	p->setOn();
	slotList->repaint();

	if(bcS) emit madeChange(bcS->queryPosition());
}

void
Implement::sourceChange()
{
	QString p = membList->currentText();
	if(!theSource->text().isEmpty())
		p.replace(0,1,"*");
	else
		p.replace(0,1," ");
	membList->changeItem(p.data(),membList->currentItem());

	if(bc) emit madeChange(bc->queryPosition());
}

void
Implement::clearSlots()
{
	slotList->clear();
}

int
Implement::queryCurrentMemb()
{
	return currentMemb;
}

void
Implement::choseSlotObject(int x)
{
        if(bc) bc->applySource(0);

	bcS = BaseComponent::at(BaseComponent::boxToPosition(x));
        if(bcS) bcS->fillSlotTable();
	objectListSlot->setCurrentItem(x);
}

void
Implement::renderSlots(int shown)
{
	if(!shown)
		slotList->setAutoUpdate(FALSE);
	else
	{
		slotList->setAutoUpdate(TRUE);
		slotList->repaint();
	}
}
	
void
Implement::addSlot(QString s, int y)
{
	slotList->insertItem(new SigSlo(s,y));
}

void
Implement::addMemb()
{
	modif->move(x(),y());
	modif->getNew();
}

void
Implement::modMemb()
{
	if(readoly) return;

	QString i(membList->currentText());
	i = i.right(i.length() - 1);
	modif->move(x(),y());
	modif->getModify(i.data(),baseIn.data(),virt,mode,defi);
}

void
Implement::delMemb()
{
	if(readoly) return;

	if(bc) bc->delMemb(membList->currentItem());
}

void
Implement::resizeEvent(QResizeEvent *)
{
    objectList->resize(width() / 2, objectList->height());
    membList->setGeometry(width() / 2,objectList->y(),
			  (width()/2)-hintButt->width(),membList->height());
    hintButt->move(width()-hintButt->width(),objectList->y());

    objectListSlot->resize(width(),objectListSlot->height());
    slotList->resize(width(),height()-(objectListSlot->x()+objectListSlot->height()));

    theSource->resize(width(),height()-(membList->y()+membList->height()));

    if(debugShowing)
    {
	/* Give several lines of the space over to the debugger */
	debugInfo->setGeometry(0,height()-(configs->queryFontHeight()*6),
			       width(),configs->queryFontHeight()*6);
	theSource->resize(width(),theSource->height()-(configs->queryFontHeight()*6));
    }

    theSource->setFocus();
}	

//Guaranteed the in will be buffered to next carriage return (by MenuBar::idE)
void
Implement::addDebug(char *in)
{
    QString inC(in), right(in);
    if(inC.isEmpty()) return;

    QString i, dirTo; dirTo.sprintf("^%s/QtEZ/",getenv("HOME"));
    inC.replace(QRegExp(dirTo.data()),"");

    static QString command, file;  static int line; 

    QString actualErr = inC.right(inC.length()-inC.find(' '));
    if(!inC.find(QRegExp("^[^:]*:[^:]*:[0-9]*: ")))
    {
	//command:file:line:
	command = right.left(right.find(':')); 
	right=right.right(right.length()-(right.find(':')+1));

	file = right.left(right.find(':'));
	right=right.right(right.length()-(right.find(':')+1));

	line = right.left(right.find(':')).toInt();
    }
    else if(!inC.find(QRegExp("^[^:]*:[0-9]*: ")))
    {
	//file:line:
	file = right.left(right.find(':'));
	right=right.right(right.length()-(right.find(':')+1));

	line = right.left(right.find(':')).toInt();
    }
    else if(!inC.find(QRegExp("^[^:]*:[^:]*: ")))
    {
	//command:file:
	command = right.left(right.find(':'));
	right=right.right(right.length()-(right.find(':')+1));

	file = right.left(right.find(':'));

    }
    else if(!inC.find(QRegExp("^[^:]*: ")))
    {
	//file:
	file = right.left(right.find(':'));
    }

    debugInfo->insertItem(new DebugItem(actualErr.data(), command.data(), 
					file.data(), line));

    if(!debuggy()) showDebug();
}

void
Implement::choseDebug(int x)
{
	DebugItem *p = (DebugItem *)debugInfo->zItem(x);
	
	//Save who ever is there now
	if(bc) bc->applySource(0);

	if(!p->queryWho())
	{
	    QMessageBox p; p.setText("Error occured in globals editor section."); p.show();
	    global->show();
	    global->raise();
	}
	else
	{
	    //Show proper Object
	    if(bc != p->queryWho())    changeToObject(p->queryWho()->queryPosition());

	    //Show proper Member
	    if(currentMemb != p->queryFunction())
	    {
		choseMemb(p->queryFunction());	
		changeToMemb(p->queryFunction());
	    }

	    //Hightlight proper Line
	    if(p->queryLine())
	    {
		if(p->queryLine() > 0)
		{
		    theSource->toBegin(TRUE);
		    theSource->setCursorPosition(p->queryLine(),0);
		    theSource->toEnd(TRUE);
		}
	    }
	}
}

void
Implement::showDebug()
{
	debugInfo->show();
	debugShowing = 1;

	resize(width(),height());
}

void
Implement::hideDebug()
{
	debugInfo->hide();
	debugShowing = 0;

	resize(width(),height());
}

void
Implement::addObject(char *desc, int change, int where)
{
	objectList->insertItem(desc,where);
	objectListSlot->insertItem(desc,where);
	if(!bc && change)
	{
		bc = BaseComponent::at(0);
		choseObject(0);
	}
	if(!bcS && change)
	{
	    bcS = BaseComponent::at(0);
	    choseSlotObject(0);
	}
}

void
Implement::changeObject(char *newDesc, int x)
{
	objectList->changeItem(newDesc, x);
	objectListSlot->changeItem(newDesc,x);
}

void
Implement::changeToObject(int x)
{
        if(bc == BaseComponent::at(x) && 
	   BaseComponent::queryAction() != A_OPEN) return;

//	This is the source of a problematic race condition.
//	bc = BaseComponent::at(x);
	BaseComponent::at(x)->giveFocus(0);
	choseObject(BaseComponent::positionToBox(x));
}

void
Implement::choseObject(int obj)
{
        if(bc) bc->applySource(0); 

	bc = BaseComponent::at(BaseComponent::boxToPosition(obj));

        if(bc) bc->showMembs();
        if(bcS) bcS->fillSlotTable();

        if(bc) objectList->setCurrentItem(
	    BaseComponent::positionToBox(bc->queryPosition()));
}

void 
Implement::addMemb(char *p,int x)
{
	QString i(p);
	if(x == 1)
		i.insert(0,"*");
	else if(x == 2)
		i.insert(0,"-");
	else if(x == 3)
		i.insert(0,"+");
	else if(x == 4)
		i.insert(0,"^");
	else
		i.insert(0," ");

	membList->insertItem(i.data());
}

void
Implement::clearMembs()
{
	membList->clear();
}

void
Implement::removeObject(int x)
{
        BaseComponent *par;
        par = (BaseComponent *) BaseComponent::at(x)->parentWidget();
        if(par &&
           par->queryPosition() == BaseComponent::queryCurrPar())
        {
		if(BaseComponent::positionToBox(x) == objectList->currentItem())
	        {
	            theSource->setText("");
	            slotList->clear();
		    if(bc == BaseComponent::at(x)) bc = 0;
		}

                objectList->removeItem(BaseComponent::positionToBox(x));
                objectListSlot->removeItem(BaseComponent::positionToBox(x));
        }
}

void
Implement::choseMemb(int memb)
{
	if(bc) bc->applySource(0);

	currentMemb = memb;
	if(!bcS) 
	{
		bcS = BaseComponent::at(0);
		objectListSlot->setCurrentItem(0);
	}

	if(bc) bc->showSource(memb);
	if(bcS) bcS->fillSlotTable();
}

void
Implement::setMembInfo(char *p, char *pas, int vir, int mod, int def,int ro)
{
	QString t(membList->currentText());
	theSource->clear();
	theSource->setAutoUpdate(FALSE);
	theSource->setText(p);
	theSource->setAutoUpdate(TRUE);
	theSource->repaint();
	virt = vir;
	
	baseIn = pas;
	defi = def;
	readoly = ro;
	if(defi == 0 || t.find('(') == -1 || ro)
		theSource->setReadOnly(TRUE);
	else
		theSource->setReadOnly(FALSE);

        mode = mod;
        if(mode == 3)
        {
                objectListSlot->show();
                slotList->show();
        }
        else
        {
                objectListSlot->hide();
                slotList->hide();
        }

}

QString
Implement::querySource()
{
       return theSource->text();
}

void
Implement::changeToMemb(int x)
{
	membList->setCurrentItem(x);
}

void
Implement::memberMod(char *proto, char *pass, int vir, int mod, int def, int n)
{
	if(n)  //new
	{
		if(bc) bc->newMemb(proto,pass,vir,mod,def);
	}
	else   //modify
	{
		if(bc) bc->modMemb(proto, pass,vir, mod, 
				   def, membList->currentItem());
	}
}

void
Implement::enterEvent(QEvent *)
{
	QString x = configs->queryFontN();
	if(x != theSource->font().family() ||
	   configs->queryFontS() != theSource->font().pointSize())
	{
		theSource->setFont(QFont(configs->queryFontN(), 
		configs->queryFontS()));
                debugInfo->setFont(QFont(configs->queryFontN(),
                configs->queryFontS()));
	}
        if(bcS) bcS->fillSlotTable();
}

void
Implement::leaveEvent(QEvent *)
{
	if(bc) bc->applySource(0);
}	

SignalConnect 
Implement::queryKeyString(int s)
{
        return SignalConnect(bc ? bc->queryPosition() : -1, currentMemb,
			     bcS ?bcS->queryPosition(): -1, s);
}

void 
Implement::CSlotObj(int x)
{
	sigSlot->choseSlotObj(x);
	choseSlotObject(x);
}

void
Implement::findText(const char *find)
{
	if(!bc) return;
	int found, x, startAt = currentMemb+1;

	for(x = bc->queryPosition(); x < BaseComponent::count(); x++)
	{
		found = BaseComponent::at(x)->funcFind(find,startAt);
		if(found != -1) break;
		startAt = 0;
	}
	if(found == -1)
	{
	        for(x = 0; x <= bc->queryPosition(); x++)
	        {
        	        found = BaseComponent::at(x)->funcFind(find,0);
                	if(found != -1) break;
	        }
	}

	if(found == -1) 
	{
		QMessageBox p;
		p.setText("Text not found.");
		p.show();
	}
	else
	{
		changeToObject(x);
		changeToMemb(found); choseMemb(found);
	}
}
