/*  $Id: SourceEdit.h,v 1.3 1998/07/27 16:35:03 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#ifndef SOURCEEDIT_H
#define SOURCEEDIT_H

#include "main.h"
#include "MembMod.h"
#include "myListBox.h"
#include "Hint.h"
#include "components/BaseComp.h"
#include "components/Deserialize.h"
#include "SyntaxEdit.h"
#include "myListBox.h"
#include "DebugItem.h"
#include "FindDialog.h"

#ifdef __USE_KDE
        #include <kmenubar.h>
#else
	#include <qmenubar.h>
#endif

#include <qtabdlg.h>
#include <qpopmenu.h>
#include <qcombo.h>
#include <qlist.h>
#include <qpushbt.h>

class
Implement : public QWidget
{
	Q_OBJECT
	friend class BaseComponent;
	BaseComponent *bc, *bcS;
	MemberModifiers *modif;
	SyntaxEdit *theSource;
	myListBox *debugInfo;
	QComboBox *objectList, *membList;
	QString baseIn;
	Hint *hintButt;
	int virt, mode, defi, readoly;
	QComboBox *objectListSlot;
	myListBox *slotList;
	int currentMemb, debugShowing, holdWho, holdWhere, holdLN, debugStyle;
	QPopupMenu *editMenu, *fileMenu;
        QList<DebugItem> fullBug, parseBug;
	FindDialog *findD;

	#ifdef __USE_KDE
		KMenuBar *theMenu;
	#else
		QMenuBar *theMenu;
	#endif

    public:
	Implement(QWidget *parent=0, char *name=0);
	~Implement() { fullBug.clear(); parseBug.clear();}

	void addSlot(QString,int);
	SignalConnect queryKeyString(int x);
	void clearObjects()
	{ objectListSlot->clear(); objectList->clear(); bcS = 0; }
	void resetObj() 
	{ currentMemb = -1; bc = 0; bcS = 0;
	  membList->clear(); slotList->clear(); objectListSlot->clear(); 
	  objectList->clear();}

	BaseComponent *currComp() { return bc; }
	void renderSlots(int);
	int queryCurrentMemb();

	void addObject(char *description, int change=0,int where=-1);
  public slots:
	void addDebug(char *);
        void clearDebug(){fullBug.clear();parseBug.clear();debugInfo->clear(); 
	hideDebug(); resize(width(),height()); }
        void showDebug();
        void hideDebug();
	int debuggy() { return debugShowing; }

	void findText(const char *);
	void findDialog() { findD->show(); }

	void delMemb();
	void changeToObject(int x);
	void removeObject(int x);
	void changeObject(char *newDesc, int x);
	void choseObject(int x);

	void choseDebug(int x);

	void choseMemb(int x);
	void addMemb(char *,int);
	void clearMembs();
	void changeToMemb(int);

	void sourceChange();

	void choseSlotObject(int);
	void CSlotObj(int);
	void choseSlot(int);
	void clearSlots();

	void setMembInfo(char *,char *, int vir, int mod, int def,int ro);
	QString querySource();
	
	void memberMod(char *proto, char *pass, int vir, int mod, int d, int n);

	void addMemb();
	void modMemb();
	
    protected:
	virtual void resizeEvent(QResizeEvent *);
	virtual void enterEvent(QEvent *);
	virtual void leaveEvent(QEvent *);

    signals:
	void madeChange(int);
};

class
SourceEdit : public QTabDialog
{
    Q_OBJECT
    Implement *implement;
    SyntaxEdit *header;

public:
   SourceEdit(QWidget *parent, char *name) : QTabDialog(parent,name)
   {
	resize(650,400);
	setCaption("Source Editor");
	setOkButton(0);

	implement = new Implement(this,"implement");
	QObject::connect(implement,SIGNAL(madeChange(int)),
		this,SIGNAL(madeChange(int)));
	addTab(implement,"Implementation");

	header = new SyntaxEdit(this,"header","Source Editor:\nThis box will give you the opportunity to include anything\nor define anything you want this class to have. This will go in\nthe header file of this class and will be placed before the\nclass declaration.");
	QObject::connect(header,SIGNAL(madeChange()),
		this,SLOT(headerChange()));
	addTab(header,"Class Header");	
    }
    ~SourceEdit() { delete header; delete implement; }

    /* Implement public interface here for Implement */
    void hideDebug() { implement->hideDebug(); }
    void clearDebug() { implement->clearDebug(); }
    void addDebug(char *p) { implement->addDebug(p); }
    int debuggy() { return implement->debuggy(); }
    void resetObj() { implement->resetObj(); header->clear(); }
    void choseSlotObject(int x) { implement->choseSlotObject(x); }    
    void addObject(char *description, int change=0,int where=-1)
	{ implement->addObject(description,change,where); }
    void removeObject(int x) { implement->removeObject(x); }
    void clearObjects() { implement->clearObjects(); }
    void changeToMemb(int x) { implement->changeToMemb(x); }
    void renderSlots(int x) { implement->renderSlots(x); }
    void clearSlots() { implement->clearSlots(); }
    void clearMembs() { implement->clearMembs(); }
    void addMemb() { implement->addMemb(); }    
    void setMembInfo(char *h,char *a,char *b, int vir, int mod, int def,int ro)
 	{ implement->setMembInfo(a,b,vir,mod,def,ro); header->setText(h); }
    QString querySource() { return implement->querySource(); }
    void changeObject(char *newDesc, int x) 
	{ implement->changeObject(newDesc,x); }
    int queryCurrentMemb() { return implement->queryCurrentMemb(); }
    void addSlot(QString p,int d) { implement->addSlot(p,d); }
    void addMemb(char *p,int d) { implement->addMemb(p,d); }

public slots:
    void changeToObject(int x) 
	{ implement->changeToObject(x); }
    void headerChange() 
        { if(implement->currComp()) 
	  {
		implement->currComp()->setHeader(header->text().data()); 
	        emit madeChange(implement->currComp()->queryPosition()); 
          }
        }
signals:
    void madeChange(int);
};

#endif
		
