/*  $Id: ToolBar.cc,v 1.10 1998/07/27 16:35:03 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "ToolBar.h"
#include "components/BaseComp.h"
#include "components/KnownComp.h"

#include <qpixmap.h>

#include <stdlib.h>
#include <strings.h>
#include <string.h>

#if defined(FORCE_DYNLIB) || defined(__ELF__) || defined(AOUT)
#include <dlfcn.h>
#endif

ToolBar::ToolBar() : QWidget(0,"toolbar",WStyle_Customize | WStyle_DialogBorder)   
{
    	currPressed = 0;
	numPotent = 1;
	currObj = 0;

	dynList.setAutoDelete(TRUE);

	func = new QPopupMenu(0);
	func->insertItem("New Parent",this,SLOT(resetParent()));
	func->insertItem("Reload",this,SLOT(reload()));

	#ifdef __USE_KDE
		theMenu = new KMenuBar(this);
		theMenu->setGeometry(0,0,62,theMenu->height());
	#else
		theMenu = new QMenuBar(this);
	#endif
	theMenu->insertItem("Tool",func);

	push = new QPushButton(this,"QLineEdit");
	push->setGeometry(2,29*(numPotent),29,29);
	push->setToggleButton(TRUE);
	push->setPixmap(QPixmap((const char**)zTextBox_xpm));
	ADDTOOLTIP(push,"Text Box");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QMultiLineEdit");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zMTextBox_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"Multi-Line Text Box");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QPushButton");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zPushButton_xpm));
        push->setGeometry(2,29*(numPotent),29,29);
	ADDTOOLTIP(push,"Push Button");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QRadioButton");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zRadioButton_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"Radio Button");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QCheckBox");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zCheckBox_xpm));
        push->setGeometry(2,29*(numPotent),29,29);
	ADDTOOLTIP(push,"Check Box");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QListBox");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zListBox_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"List Box");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QComboBox");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zComboBox_xpm));
        push->setGeometry(2,29*(numPotent),29,29);
	ADDTOOLTIP(push,"Combo Box");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QScrollBar");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zScrollBar_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"Scroll Bar");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QSlider");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zSlider_xpm));
        push->setGeometry(2,29*(numPotent),29,29);
	ADDTOOLTIP(push,"Slider");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QButtonGroup");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zButtonGroup_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"Button Group");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QLabel");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zLabel_xpm));
        push->setGeometry(2,29*(numPotent),29,29);
	ADDTOOLTIP(push,"Label");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QLCDNumber");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zLCDNumber_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"LCD Number");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"QTimer");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zTimer_xpm));
        push->setGeometry(2,29*(numPotent),29,29);
	ADDTOOLTIP(push,"Timer");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

        push = new QPushButton(this,"Custom");
	push->setToggleButton(TRUE);
        push->setPixmap(QPixmap((const char**)zCustom_xpm));
        push->setGeometry(31,29*(numPotent++),29,29);
	ADDTOOLTIP(push,"Custom");
	QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));

	buildLdTable();

	setCaption("ToolBar");
	resize(62,29*numPotent);
	setFixedSize(size());
}

void
ToolBar::reload()
{
    numPotent -= (int)((dynList.count()+1) / 2);
    dynList.clear();
    buildLdTable();
    resize(62,29*numPotent);
    setFixedSize(size());
    if(currPressed) resetPressed();
    repaint();
}

void
ToolBar::addComp(bool on)
{
    if(on)
    {
	QPushButton *oldPressed = currPressed;
	currPressed = (QPushButton *)sender(); 

	if(oldPressed) oldPressed->setOn(FALSE);
    }
    else if(sender() && currPressed == sender())
    {
	BaseComponent *newbie = BaseComponent::ofType(getTopLevel(),
						      currPressed->name());
        if(newbie)
	{
	    if(newbie->parentWidget()) newbie->move(0,0);
	    newbie->show();
	    currPressed = 0;
	}
    }
}

char 
*ToolBar::currentPressed()
{ 
    return currPressed ? (char *)currPressed->name() : 0;
}

void 
ToolBar::resetPressed()
{ 
    QPushButton *oldPressed = currPressed;
    currPressed = 0; 

    if(oldPressed) oldPressed->setOn(FALSE);
}

void
ToolBar::buildLdTable()
{
    int left = 1, fi;
    char *Buff = new char[150];
    QString tmpDir,i;

    tmpDir.sprintf("%s/QtEZ/.QtEZLd",getenv("HOME"));
    ifstream p(tmpDir.data());

    while(p.getline(Buff,150,'\n'))
    {
	i = Buff;
	if((fi = i.find(';')) != -1 && i.at(0) != '#')
	{
	    QString lib = i.right((i.length()-fi)-1);
	    if(lib.find(".") != -1) lib = lib.left(lib.findRev("."));
	    QString desc = i.left(fi);

	    QString xpm = lib.data(); xpm += ".img";	    
	    push = new QPushButton(this,desc.data());
	    dynList.append(push);
	    ifstream test(xpm.data());
	    if(test.fail()) 
		push->setPixmap((const char**)zCustom_xpm);
	    else
	    {
		test.close();
		push->setPixmap((const char *)xpm.data());
	    }
	    if(left)
		push->setGeometry(2,29*(numPotent),29,29);
	    else
		push->setGeometry(31,29*(numPotent++),29,29);
	    left = !left;
	    ADDTOOLTIP(push,i.left(fi));
	    QObject::connect(push,SIGNAL(toggled(bool)),this,SLOT(addComp(bool)));
	    push->setToggleButton(TRUE);
	    push->show();
	}
    }
    if(!left) numPotent++;
    delete [] Buff;
    p.close();
}

BaseComponent
*ToolBar::getTopLevel()
{
    return (currObj && !currObj->canHaveChildren()) ? 0 : currObj;
}

void
ToolBar::changeObj(int x)
{
	currObj = BaseComponent::at(x);
}
