/*  $Id: BaseComp.h,v 1.9 1998/07/27 16:35:05 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#ifndef BASECOMP_H
#define BASECOMP_H

//This is also defined in AttributeItem.h (if changed)
#ifndef ATTRIBUTEITEM_H
enum ATTRIB_TYPE { W_LIST, W_FILE, W_TEXT, W_TRUE, W_COLOR};
#endif

#define ADDATTRIB(args...) attrib->addAttribute(new AttributeItem(## args))
#define WRITEATTRIB(x,y) if(isWritten(x)) writeInParent(y)
#define strBool(x) x ? "TRUE" : "FALSE"


enum VAR_TYPE { PAR_NULL, PAR_POINTER, PAR_NORM };

enum M_MODE { M_PRIVATE, M_PROTECTED, M_PUBLIC, M_SIGNAL, M_PUB_SLOT };

enum COMP_ACTION { A_NONE, A_RESIZE_E, A_RESIZE_W, A_RESIZE_N, A_RESIZE_S,
                   A_MOVE, A_OPEN, A_RESIZE_SE, A_RESIZE_SW, A_RESIZE_NE,
                   A_RESIZE_NW, A_SWITCH, A_CLOSE, A_SAVE, A_DRAW_SQUARE };

#include <qstrlist.h>
#include <qpopmenu.h>
#include <fstream.h>
#include <iostream.h>

class LinkList;
class zMenuBar;
class LineF;

class
BaseComponent : public QWidget
{
    Q_OBJECT

    static ofstream workingFile;
    static QString whichApp, MakeFile, finalPath;
    static COMP_ACTION action;
    static int genCount, mult_sel, doRaise, isTemp, currPos, currObj, currPar; 
    static QPoint click, perm;
    static VAR_TYPE toParent;
    static QList<BaseComponent> sel_list;
    static QList<int> boxesList;

    int flag,consCount,paintee, locked, lockId, _level, _showChildren, _vis;
    int managed;
    QStrList defList;
    QString *changed, *libLocate, *myHeader, *specClass; 
    M_MODE _mode;
    void itSized(); 
    int numElements(char *); 
    QString nextName(char *,int);

public: 
    BaseComponent(QWidget *parent, char *name);
    ~BaseComponent();

    void setGoingVis(int x) { _vis = x; }
    int goingVis() { return _vis; }

    LineF functionWith(int);
    
    int isWritten(char *);
    void addWritten(char *); 
    void delWritten(char *);

    static BaseComponent *getLibComponent(BaseComponent *, char *, void * =0);
    static BaseComponent *findDynComponent(BaseComponent *, QString);
    static BaseComponent *ofType(BaseComponent *, QString ,int =-2, int =-2); 
    static BaseComponent *at(int x); 
    static int boxToPosition(int x); 
    static int positionToBox(int x);
    static int toTopLev(int); 
    static int fromTopLev(int); 
    static void setAction(COMP_ACTION x) { action = x; } 
    static int queryAction() { return action; } 
    static QString queryApp() { return whichApp; } 
    static void setApp(char *p) { whichApp = p; } 
    static BaseComponent *prepApp();
    static int genMakMain() { return gMM; }
    static int queryCurrObj() { return currObj; } 
    static int nextParent(int);
    static int prevParent(int);
    static BaseComponent *queryCurrBaseComp() { return at(currObj); }
    static QString queryFinalPath() { return finalPath; }

    int isManaged() { return managed; }
    int queryConsCount() { return consCount; }
    char *qualifiedName(char *newName=0);	
    void resetType() { *myType = queryQType(); } 
    const QWidget *queryLook() { return myLook; }

    int funcFind(const char *,int);
    char *queryHeaderPath() { return myHeader->data(); }
    char *queryLibLocate() 
	{ return libLocate->isEmpty() ? 0 : libLocate->data(); } 
    void setLibLocate(char *p) { *libLocate = p; }

    void giveFocus(int=1);	

    void setLevel(int x) { _level = x; }
    int queryLevel() { return _level; }

    virtual int invisibleComp() { return 0; }
    virtual int canHaveChildren() { return 1;}
    virtual int canResize() { return 1; }
    virtual int externalLinkage() { return 0;}
    virtual int instantiate()    { return 1; } 
    virtual char *specialInit() { return 0; } 
    virtual char *specialInher() { return 0; } 
    virtual int copyAble()       { return 1; }
    virtual QString parseInitSpecial(QString p) { return p; } 
    virtual void fillAttributes() {} 
    virtual void interpretAttributes() {} 
    virtual char *queryQType() { return "QWidget"; } 
    virtual void fillMembInfo() {} 
    virtual char *aboutComp() 
	{ return "By Sam Magnuson:\nPart of core development for QtEZ."; }

    void setCurrPar(int x); static int queryCurrPar() { return currPar; }
    void fillBoxes(); void insertMenu(int where); 

    void menuApplyAttribs();
    void modifyMenu(int where, QString cap, QString na, int conO, int conS,
		    QString scut, int chk, int ena, int lev); 
    void removeMenu(int where); 
    void makeMenu();

    void beingOpened() { sel_list.clear(); mult_sel = 0; action = A_OPEN; }
    void doneOpening() { action = A_NONE; }

    void setHeader(char *p); 
    void appendHeader(char *p);
    QString header();

    QStrList *querySlots();
    void fillMembs();
    void fillSource();
    void saveSource();
    void saveSigSlo();
    char *getClassName(char *);
    void writeYourself(QString *, QString *,QString *);

    QString queryExecutable();

    M_MODE mode() { return _mode; }
    void setMode(M_MODE x) { _mode = x; }

    bool noFuncs();
    int isMyChild(BaseComponent *);
    int numChildren(bool =FALSE);

    QString queryStyle() { return *uiStyle; }

    void setActive(int =0);

    char *staticClass(){ return specClass->isEmpty() ? 0 : specClass->data(); } 
    bool hasChanged()
    {
	if(changed->isEmpty())
	    return TRUE;
	else
	    return FALSE;
    }

    LinkList *membInfo();
	
    void setLook(QWidget *p);
    void setLook(QPixmap p);
    void setLook();	

    void shiftPositions(int);
    bool loneWidget();
    void writeToFile(int temp=0);

    static int count() { return currPos; }
    void setPosition(int x);
    int queryPosition() { return myPosition; }
    char *queryYType() { return myType->data(); }
    void writeInParent(QString);
    void writeLine(QString, int =1);
    QString openFile(QString="",QString="",int=0);

    zMenuBar *queryMenu() { return myMenu; }
    QList<QPopupMenu> *queryMenuContent() { return menuContent; }
    void setMenu(zMenuBar *x);
	
protected:
    LinkList *memberInfo;
    zMenuBar *myMenu;
    QList<QPopupMenu> *menuContent;
    QPopupMenu *popMenu;
    QWidget *myLook; 
    void writePalette(char * =0);
    int currentMemb, stretch, myPosition;
    static int snapTo, gMM, mainWidget;
    QString *tooltipData, *uiStyle, *myType, *iconPix;
    static QString target,includes, CFlags, LFlags, mainAux;
    static QStrList *fontList;
    void makeStyle(GUIStyle);
    void makeStyle(QString *);
    void newXY(int,int);
    void changeName(char *,int=1);
    void setSnapTo(int);
    void doDeclaration(M_MODE,QString,int doHead=1);
    void addMember(char *proto, char *bi="", char *source="", int virt=1,
                int mode=1, int definedH=1);
    void addHead(char *);

    virtual void writeInits() {}
    virtual void postWriteInits() {}
    virtual void mousePressEvent(QMouseEvent*);
    virtual void mouseMoveEvent (QMouseEvent*);
    virtual void mouseReleaseEvent (QMouseEvent *);	
    virtual void keyReleaseEvent (QKeyEvent *);
    virtual void resizeEvent(QResizeEvent *);
    virtual void paintEvent(QPaintEvent *);
    virtual bool eventFilter(QObject *, QEvent *);
    
    void genericFillAttrib();
    void genericInterpAttrib();    
    void genericFillMembInfo();

    char *queryStaticClass() { return specClass->data(); }
    void setStaticClass(char *p) { *specClass = p; }

    void registerLFlag(char *what)
    {
	QString i; i.sprintf("%s ",what);
	if(LFlags.find(i.data()) == -1) LFlags.append(i.data());
    }
    void registerCFlag(char *what)
    {
	QString i; i.sprintf("%s ",what);
	if(CFlags.find(i.data()) == -1) CFlags.append(i.data());
    }
		
public slots:
	
    int showChildren() { return _showChildren; }
    void setShowChildren(int x) { _showChildren = x; }
    void toggleShowChildren() { _showChildren = !_showChildren; }

    void showAttribs();
    void applyAttribs();

    void showMembs();
    void showSource(int memb);
    void applySource(int = 0);
    void newMemb(char *,char *,int, int,int);
    void modMemb(char *,char *,int, int, int, int);
    void fillSlotTable();
    void fillSigTable();
    void delMemb(int);
    void makeChange(int x);

    int queryLocked() { return locked; }
    void setLock(int x);
    void toggleLock();

    void delShow();
    void attShow(); 
    void souShow(); 
    void sigShow();
    void menShow();
    void aboShow();
    void popShow(QPoint);
    void raiseComp();
    void lowerComp();
    void hRaise() { raise(); if(myLook) myLook->raise(); }
    void hLower() { lower(); if(myLook) myLook->lower(); }
    void pasteComp();
    void cutComp();
    void copyComp();
signals:
    void xyPos(int,int);
    void changeMade();
    void currentW(int);

};


#include <qframe.h>

class
BaseFrame : public BaseComponent
{
    private:
	QString *fShape, *fStyle;
    public:
	BaseFrame(QWidget *, char *);
	~BaseFrame();

    protected:
	void prepFrame(QFrame *);
	void showFrameAttrib(QFrame *);
	void interpFrameAttrib(QFrame *);
	void writeFrame();
};

#endif
	
