/*  $Id: CompIO.cc,v 1.10 1998/07/28 18:51:11 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "CompIO.h"
#include "Deserialize.h"
#define NUM_INDENT 2
#define LAST_LEV componentStack.last()->queryLevel()

#include <qregexp.h>
#include <stdio.h>

int yylex();
extern FILE *yyin;

#include "../MenuBar.h"
extern MenuBar *menubar;

ofstream CompIO::outgoing;
open_type CompIO::open_kind;

QList<BaseComponent> componentStack;
QString baseDir = "./";
float fileVersion=0;

CompIO::CompIO()
{
}

/*
You'll notice I only write things that aren't at their default state. 
This is to speed up loading afterwards, the sideeffect of this is - if I 
change the default behaviour I'm going to either abandon this 
stuff (and potentially mess up others' projects) or keep these default 
values.
*/
void
CompIO::save(QString p, int start, int stop)
{
    if(!BaseComponent::at(start)) return;

    BaseComponent::setAction(A_SAVE);
    componentStack.clear();

    int end = stop,poe=0,y,x, lessStack = 0;
    if(stop == -1) end = BaseComponent::count();

    int currHold = attrib->currentObject(), levelIn=0;
    QString right, indent; 

    outgoing.open(p);

    //output the version of this file
    outgoing << (float) _VERSION_NO << AREA_SEP1;
    //output how many components there are stored in this file
    outgoing << (int) BaseComponent::count() << AREA_SEP1 << endl;

    menubar->showLevel((end-start)/100);

    for(x = start; x < end; x++) saveComponent(x);
    while(componentStack.count())
    {
        indent.fill(' ',LAST_LEV*NUM_INDENT);
        outgoing << indent.left(indent.length()-NUM_INDENT);
        outgoing << "}; component" << endl;
        componentStack.removeLast();
    }

    if(stop == -1)
    {
	if(!global->isEmpty())
	{
	    outgoing << endl << "globals_editor {" << endl;	

	    //dump contents of the header editor
	    if(!global->queryHeader().isEmpty())
	    {
		right = "  header { ";
		right += global->queryHeader().data();
		right.replace("\n"," };\n  header { "); right += " };";
		outgoing << right << endl;
	    }

	    //dump contents of main function editor
	    if(!global->queryFunction().isEmpty())
	    {
		right = "  main_func { ";
		right += global->queryFunction().data();
		right.replace("\n"," };\n  main_func { "); right += " };";
		outgoing << right << endl;
	    }
	    
	    //dump the signature and comments section of globals editor
	    if(!global->queryComment().isEmpty())
	    {
		right = "  signature { ";
		right += global->queryComment().data();
		right.replace("\n"," };\n  signature { "); right += " };";
		outgoing << right << endl;
	    }	

	    //dump the raw source area of globals editor
	    if(!global->queryFreeHand().isEmpty())
	    {
		right = "  raw { ";
		right += global->queryFreeHand().data();
		right.replace("\n"," };\n  raw { "); right += " };";
		outgoing << right << endl;
	    }

	    outgoing << "}; globals_editor" << endl;
	}
    }
    //turn serialization mode back off in attribute table
    attrib->setSerialize(-1);
    outgoing.close();
    menubar->hideLevel();
    BaseComponent::setAction(A_NONE);
}

void
CompIO::saveComponent(int where)
{
    BaseComponent *temp = BaseComponent::at(where);
    if(componentStack.count() &&
	temp->queryLevel() <= LAST_LEV && temp != componentStack.last())
    {
        while(componentStack.count() && temp->queryLevel() <= LAST_LEV)
        {
		QString indent;indent.fill(' ',(LAST_LEV*NUM_INDENT)*2);
                outgoing << indent << "}; component" << endl;
                componentStack.removeLast();
        }
    }
    componentStack.append(temp);

    QString indent;indent.fill(' ',LAST_LEV*NUM_INDENT);
    QString compIndent = indent.left(indent.length()-NUM_INDENT);

    int y;
    QString right("");
    
    temp->membInfo()->fixConnections();

    //output my class type
    outgoing << compIndent;
    outgoing << endl << "component " << temp->queryQType(); 
    if(!temp->parentWidget()) outgoing << " : 0";
    outgoing << " { " << endl;

    //am i locked?
    if(temp->queryLocked())
    {
	outgoing << indent;
	outgoing << "locked { " << temp->queryLocked() << " };" << endl;
    }

    //am i showing my children (in project viewer) ?
    if(!temp->showChildren())
    {
	outgoing << indent;
	outgoing <<  "showChildren { " << temp->showChildren();
	outgoing << " };" << endl;
    }

    //showing myself?
    if(!temp->isVisible())
    {
	outgoing << indent;
	outgoing << "visable { " << temp->isVisible();
	outgoing << " };" << endl;
    }

    //dump contents of my headers
    if(!temp->header().isEmpty())
    {
	right = "  header { ";
	right += temp->header().data();
	right.replace("\n"," };\n  header { "); right += " };";
	outgoing << right << endl;
    }

    //dump my menubar info
    for(y = 0; y < temp->membInfo()->menuCount(); y++)
    {
	MenubarElement *t = temp->membInfo()->menu(y);
	outgoing << indent <<  "menubar {" << endl;

	outgoing << indent + indent;
	outgoing <<  "name { " << t->queryName() << " };" << endl;

	outgoing << indent + indent;
	outgoing << "caption { " << t->queryCapt() << " };" << endl;

	if(t->queryConO() != -1)
	{
	    outgoing << indent + indent;
	    outgoing <<  "connect { " << t->queryConO();
	    outgoing << ":" << t->queryConS() << " };" << endl;
	}

	if(t->queryChk())
	{
	    outgoing << indent + indent;
	    outgoing <<  "checked { " << t->queryChk() << " };" << endl;
	}

	if(!t->queryEna())
	{
	    outgoing << indent + indent;
	    outgoing <<  "enabled { " << t->queryEna() << " };" << endl;
	}

	if(t->queryLev())
	{
	    outgoing << indent + indent;
	    outgoing <<  "level { " <<   t->queryLev() << " };" << endl; 
	}

	QString tmp = t->queryShrtCut();
	if(!tmp.isEmpty())
	{
	    outgoing << indent + indent;
	    outgoing <<  "shortcut { " << t->queryShrtCut() << " };" << endl;
	}

	outgoing << indent <<  "}; menubar"<< endl;
    }

    //dump my attributes
    attrib->setSerialize(where);

    //dump my members
    for(y = 0; y < temp->membInfo()->count(); y++)
    {
	if(!temp->membInfo()->isReadOnly(y))
	{
	    outgoing << indent <<  "function { " << endl;

	    outgoing << indent + indent <<  "prototype { ";
	    outgoing << temp->membInfo()->memb(y)->data();
	    outgoing << " }; " << endl;
	    outgoing << indent + indent <<  "baseinit { ";
	    outgoing << temp->membInfo()->baseInit(y)->data();
	    outgoing << " }; " << endl;
		
	    if(!temp->membInfo()->source(y)->isEmpty())
	    {
		right = indent + indent + "code { ";
		QString spec; spec.sprintf(" };\n%s",right.data());
		right += temp->membInfo()->source(y)->data();
		right.replace("\n",spec.data()); right += " };";
		outgoing << right << endl;		
	    }

	    outgoing << indent + indent <<  "prot_mode { ";
	    outgoing << temp->membInfo()->mode(y);
	    outgoing << " }; " << endl;

	    if(!temp->membInfo()->virt(y))
	    {
		outgoing << indent + indent <<  "virtual { ";
		outgoing << temp->membInfo()->virt(y);
		outgoing << " }; " << endl;
	    }

	    if(!temp->membInfo()->define(y))
	    {
		outgoing << indent + indent <<  "defined { ";	
		outgoing << temp->membInfo()->define(y);
		outgoing << " }; " << endl;
	    }

	    outgoing << indent <<  "}; function" << endl;
	}
    }

    //dump my signals/slots connections
    for(y = 0; y < temp->membInfo()->connectCount(); y++)
    {
	outgoing << indent <<  "sigslot { " << *temp->membInfo()->connect(y);
	outgoing << " };" << endl;
    }
    
    if(!temp->numChildren() || (temp->queryLevel() <= LAST_LEV &&
	temp != componentStack.last()))
    {
	while(componentStack.count() && temp->queryLevel() <= LAST_LEV)
	{
		outgoing << compIndent << "}; component" << endl;
		componentStack.removeLast();
	}
    }	
}

void
CompIO::saveAttribute(AttributeItem *one)
{
    QString indent; indent.fill(' ',LAST_LEV*NUM_INDENT);

     outgoing << indent;
     outgoing << "attrib { \"" << one->queryCaption().data() << "\" \"";

    QString tmp;

    if(one->type() == W_COLOR)
    {
	tmp.sprintf("%d:%d:%d:",
		    one->queryColor().red(),
		    one->queryColor().green(),
		    one->queryColor().blue());
	outgoing  << tmp.data();
    }
    else
	outgoing << one->queryValue().data();
	
    outgoing << "\" }; " << endl;
}
	
#include <time.h>

int
CompIO::load(QString where, open_type otype)
{
    time_t now = time(0);

    baseDir = where.left(where.findRev('/')+1).data();
    open_kind = otype;

    QString temp;
    char p;
    int numToGet=0;

    componentStack.clear();
    BaseComponent *project=0;
    if(open_kind == O_PASTE) 
	project = BaseComponent::at(BaseComponent::queryCurrObj());
    componentStack.append(project);

    ifstream incoming(where.data());
    if(incoming.fail()) 
    {
	QMessageBox p;
	p.setText("Bad serialization.");
	p.show();
	return 0;
    }
    temp = "";

    while(chkByte(incoming,p,AREA_SEP1) && !incoming.eof()) temp += p;
    fileVersion = temp.toFloat();

    if((float)_VERSION_NO < (float)fileVersion)
    {
	QMessageBox i;

	i.setIcon(QMessageBox::Warning);
	i.setText("This version of QtEZ is a out of date,\n\
I will try to open the file, but if it fails grab\n\
a new copy from http://qtez.zax.net/qtez/\n");
	i.show();
    }

    //How many widgets will I be opening in this run?
    if(fileVersion >= (float) 0.33)
    {
	temp = "";
	while(chkByte(incoming,p,AREA_SEP1) && 
	      !incoming.eof()) temp += p;
	numToGet = temp.toInt();
	if(numToGet >= 0) menubar->showLevel(numToGet);
    }	

    //Prior to 0.48 I used a dumb serilization - hopefully will 
    //not need to go back to it but, all old files should still 
    //load just fine (hope)
    if((float)fileVersion < (float)0.48) return oldLoadVersion(incoming);
    incoming.close();
    
    BaseComponent::setAction(A_OPEN);

    yyin = fopen(where.data(),"r");
    int toRet = yylex();
    fclose(yyin);

    for(int x = BaseComponent::count()-1; x >= 0; x--)
        BaseComponent::at(x)->raise();

    if(project) project->show();
    BaseComponent::setAction(A_NONE);

    projView->buildTree();

    attrib->resetObj();
    if(BaseComponent::at(0)) BaseComponent::at(0)->giveFocus();
    menubar->hideLevel();

    //This is to allow for signals connected to signals
    if(fileVersion <= (float) 0.52) convertSlots();

    cout << "took " << time(0) - now << endl;
    return toRet;
}


int
CompIO::oldLoadVersion(ifstream &incoming)
{
    QString temp;
    char p;
    BaseComponent *comp, *project=componentStack.last();
    int upC, toRet = -1, counter=0, addToConn=0;


	//This magic is so the introduction of a TOP most component
	//(Application) will not fail to initiate with one, as well as
	//uping the signal/slot connections
    if((float)fileVersion < (float)0.45)
    {
	BaseComponent::prepApp();
	addToConn++;
    }
    BaseComponent::setAction(A_OPEN);

    QString type;
    int prnt;
    while(chkByte(incoming,p,AREA_SEP2,1) && !incoming.eof())
    {
	temp = "";

	while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	type = temp;
	if(fileVersion >= (float)0.32)
	{
	    //Does this coming widget have a parent?
	    prnt = type.right(type.length()-
			      (type.find(':')+1)).toInt();

	    //Of what type is this coming widget?
	    type = type.left(type.find(':'));
	}
	else
	{
	    if(type == "QWidget") 
		prnt = 0;
	    else
		prnt = 1;
	}

	int fixer=0;
	if((float)fileVersion < (float)0.45 && prnt!=-1) { prnt++;fixer=1;}
	comp = BaseComponent::ofType(project, type, prnt, 
		     ((fileVersion >= (float)0.43)&&open_kind!=O_PASTE));
	if(fixer) prnt--;

	if(fileVersion >= (float)0.43)
	{
	    if(prnt == -1) prnt=0;
	    else prnt=1;
	}
	if(!prnt)
	{
	    if(project) project->show();
	    project = comp;
	    comp->hide();
	}	
	menubar->stepLevel();
 
	toRet = comp->queryPosition();

	//This assures signals and slot objects will go up ass
	//necesary.
	upC = open_kind==O_IMPORT ? project->queryPosition() : 0;
	upC += addToConn;

	//This block checks for locked components
	if(fileVersion >= (float)0.40)
	{
	    temp = "";
	    while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	    comp->setLock(temp.toInt());
	}

	//This makes the project viewer look as it was when saved
	//collapsed, etc.
	if(fileVersion >= (float)0.42)
	{
	    temp = "";
	    while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	    comp->setShowChildren(temp.toInt());
	}
	
	//This block loads a menubar stored.
	int count = 0;
	if(fileVersion >= (float) 0.099)
	{
	    QString capt, nam, shtc;
	    int co=0,cs=0,c=0,e=0,l=0;
	    while(chkByte(incoming,p,AREA_SEP1,1))
	    {
		while(chkByte(incoming,p,FIELD_SEP))
		{
		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    capt = temp.data();
		    incoming.get(p);

		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    nam = temp.data();
		    incoming.get(p);

		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    co = temp.toInt();
		    if(co != -1) co += upC;
		    incoming.get(p);
 
		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    cs = temp.toInt();
		    incoming.get(p);

		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    c = temp.toInt();
		    incoming.get(p);

		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    e = temp.toInt();
		    incoming.get(p);

		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    l = temp.toInt();
		    incoming.get(p);

		    temp = "";
		    while(p != ':') { temp += p;
		    incoming.get(p); }
		    shtc = temp.data();
		}
		project->modifyMenu(count++,capt,nam,co,cs,
				    shtc,c,e,l);
	    }
	    //Simply to advance pointer
	    chkByte(incoming,p,0);	
	}
	attrib->clearAttributes();
	comp->fillAttributes();


	//This block restores saved attrib table (serialization of how 
	//the widget looks).
	int multi = 0;
	QString capt;
	while(chkByte(incoming,p,AREA_SEP1,1))
	{
	    multi = 0;
	    temp = "";

	    while(chkByte(incoming,p,FIELD_SEP)) temp += p;
 
	    capt = temp;
	    temp = "";

	    while(chkByte(incoming,p,FIELD_SEP)) temp += p;

	    attrib->setValue(capt,temp);
	}
	//Simply to advance pointer
	chkByte(incoming,p,0);

	comp->applyAttribs();
 
	if(project != comp) comp->show();
	QString load(comp->name());
	load.insert(0,"Loading: ");
	menubar->setCaption(load.data());

	//This block loads stored members and puts back into widget.
	QString proto, sour, bi;
	int mod, vir, def;
	while(chkByte(incoming,p,AREA_SEP1,1))
	{
	    temp = "";
	    while(chkByte(incoming,p,FIELD_SEP)) { temp += p; }
	    proto = temp;
	    temp = "";
	    while(chkByte(incoming,p,FIELD_SEP)) { temp += p; }
	    bi = temp;
	    temp = "";
	    while(chkByte(incoming,p,FIELD_SEP)) { temp += p; }
	    sour = temp;
	    temp = "";
	    while(chkByte(incoming,p,FIELD_SEP)) { temp += p; }
	    mod = temp.toInt();
	    temp = "";                
	    while(chkByte(incoming,p,FIELD_SEP)) { temp += p; }
	    vir = temp.toInt();
	    temp = "";
	    while(chkByte(incoming,p,FIELD_SEP)) { temp += p; }
	    def = temp.toInt();
	    comp->membInfo()->add(
		proto.data(), bi.data(),sour.data(), vir, mod, def);
	}
	//Simply to advance pointer
	chkByte(incoming,p,0);

	//This block loads stored signal/slots connections.
	int si[4];
	while(chkByte(incoming,p,AREA_SEP1,1))
	{
	    count = 0;
	    while(chkByte(incoming,p,FIELD_SEP))
	    {			
		temp = "";
		while(p != ':') {temp += p;incoming.get(p);}
		si[count++] = temp.toInt();						}
	    si[0] += upC; si[2] += upC;
	    comp->membInfo()->addConnection(
		SignalConnect(si[0],si[1],si[2],si[3]));
	}
	//Simply to advance pointer
	chkByte(incoming,p,0);

	QObject::connect(project,SIGNAL(changeMade()),
			 menubar,SLOT(changeMade()));
	QObject::connect(project,SIGNAL(xyPos(int,int)),
			 menubar,SLOT(xy(int,int)));

	if(project->queryMenu()) project->makeMenu();

	QObject::connect(comp,SIGNAL(currentW(int)),
                         attrib,SLOT(changeToObject(int)));
	QObject::connect(comp,SIGNAL(currentW(int)),
                         source,SLOT(changeToObject(int)));
	QObject::connect(comp,SIGNAL(currentW(int)),
                         sigSlot,SLOT(changeToObject(int)));
	QObject::connect(comp,SIGNAL(currentW(int)),
                         projView,SLOT(changeToObject(int)));
	QObject::connect(comp,SIGNAL(currentW(int)),
                         menu,SLOT(changeToObject(int)));

	counter++;
    }
    //Simply to advance pointer
    chkByte(incoming,p,0);

    temp = "";

	//These blocks insert information back into the globals editor.
    if(fileVersion < (float) 0.22)
    {
	while(chkByte(incoming,p,EOF)) temp += p;
 
	global->appendHeader(temp);
    }
    else 
    {
	while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	global->appendHeader(temp);

	temp = "";
	while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	global->appendFunction(temp);

	temp = "";
	while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	global->appendComment(temp);

	if(fileVersion >= (float)0.41)
	{
	    temp = "";
	    while(chkByte(incoming,p,AREA_SEP1)) temp += p;

	    global->appendFreeHand(temp);
	}
    }
    //This final step sets all the widgets so they look EXACTLY
    //as they were saved (in layered order)
    for(int x = BaseComponent::count()-1; x >= 0; x--)
	BaseComponent::at(x)->raise();

    if(project) project->show(); 
    BaseComponent::setAction(A_NONE);
    incoming.close();

    projView->buildTree();

    attrib->resetObj();
    if(BaseComponent::at(0)) BaseComponent::at(0)->giveFocus();
    menubar->hideLevel();
    //This is to allow for signals connected to signals
    if(fileVersion <= (float) 0.52) convertSlots();
    return toRet;
}	

//return 1 until I feel comp has been matched with no comment out
int
CompIO::chkByte(ifstream &from, char &where, char comp, int pee)
{
	char hold;
	int toRet = 1;

	if(pee)
	   where = from.peek();
	else
	   from.get(where);

	if(where == comp) toRet = 0;

	if(where == '\\')
	{
		if(pee)
		{
			from.get(where);
			hold = from.peek();
			from.putback(where);
		}
		else
			hold = from.peek();

		if(hold == FIELD_SEP ||
		   hold == AREA_SEP1 ||
		   hold == AREA_SEP2)
		{
			from.get(where);
			toRet = 1;
		}
	}	

	return toRet;
}

void
CompIO::convertSlots()
{
	for(int bc = 0; bc < BaseComponent::count(); bc++)
	{
		SignalConnectList *p;
		p = BaseComponent::at(bc)->membInfo()->connections();

		for(int conn = 0; conn < p->count(); conn++)
		{
			LinkList *temp = BaseComponent::at(
				p->at(conn)->querySlotOwner())->membInfo();

			int tS = temp->slot(p->at(conn)->querySlot());
			int tCount = p->at(conn)->querySlot();
			for(int func = 0; func < tS; func++)
			{
			      if(temp->mode(func) == M_SIGNAL) tCount++;
			}
			p->at(conn)->setSlot(tCount);
		}
	}
}
