/*  $Id: Deserialize.l,v 1.13 1998/07/27 16:35:06 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

%{
   #include "CompIO.h"
   #include "Deserialize.h"
   #include "BaseComp.h"
   #include "../main.h"

   //For debugging
   #if 0
   #include <time.h>
   #define LXDEBUG(x) printf("-%d- %d, %s (%s)\n",(int)time(0),lineNumber,x,yytext);
   #else
   #define LXDEBUG(x)   
   #endif

   #include "../MenuBar.h"
   extern MenuBar *menubar;

   #include "../ToolBar.h"
   extern ToolBar *toolBar;

   #include <iostream.h>
   #include <qlist.h>
   #include <stdio.h>
   #include <qstring.h>

   BaseComponent *tmpComponent=0;
   int lineNumber = 0, charNumber =0, toRet = -1, connectionStepup=0; 

   MenubarElement oneMenubar;
   FunctionElement oneFunction;

   extern "C" { char *parseValue(); int cleanUp(); }

   extern QList<BaseComponent> componentStack;
   extern QString baseDir;

   /* Include-fu stolen a bit from docs */
   YY_BUFFER_STATE include_stack[MAX_INCL_DEPTH];
   StackCount stacker[MAX_INCL_DEPTH];
   int include_count = 0;
   /* *********************** */

%}

DIGIT -?[0-9]*
STRING [:alnum:]*
WHITE [ \t]*

%option stack

%x COMPONENT
INCOMPONENT locked|showChildren|attrib|sigslot|visable|header

%x FUNCTION
INFUNCTION prototype|baseinit|code|prot_mode|virtual|defined|readonly

%x GLOBALS
INGLOBALS header|signature|raw|main_func

%x MENUBAR
INMENUBAR name|caption|connect|checked|enabled|level|shortcut

ININITIAL start_dir
%%

<*>{WHITE} /* Ignored whitespace */
<*>"/*" {
           LXDEBUG("Into a /* comment")
	
	/* Ignored comment in slash asteric style */
	int c;

	while((c = yyinput()) != 0)
	{
		if(c == '\n')
		lineNumber++;

		else if(c == '*')
		{
			if((c = yyinput()) == '/')
				break;
			else
				unput(c);
		}
	}

           LXDEBUG("Out of /* comment")
}
<*>"//" {
           LXDEBUG("Into a // comment")

        /* Ignored comment in c++ slash-slash style */
	int c;
	while( (c = yyinput()) != '\n' && c != EOF );
	unput(c);

           LXDEBUG("Out of // comment")

}
<*>\n { lineNumber++; charNumber = 0; }
<*>.  {
	charNumber++;
	if(lineNumber > (HEADER_LENGTH-1)) 
	{
	  cerr << "Unrecognized symbol '" << yytext << "' at:" << endl;
	  cerr << "\tline: " << lineNumber << endl;
	  cerr << "\tchar: " << charNumber <<  endl;
	}
}
<*>^{WHITE}@incl\ [^\n]* {
           LXDEBUG("Into a Include")

	QString inc(yytext); inc=inc.simplifyWhiteSpace(); 
	inc=inc.right( inc.length() - (inc.find(" ")+1) );
	/* if not starting with a / then insert the start_dir directive */
	if(inc.at(0) != '/') 
	{
		QString tmp(stacker[include_count].dir());
		inc.insert(0,tmp.data());
		/* if still no / at begining then insert the dir that this */
		/* file was opened from */
		if(inc.at(0) != '/') inc.insert(0,baseDir.data());
	}

	if ( include_count >= MAX_INCL_DEPTH )
	{
		QString buff; 
		buff.sprintf("@incl nested too deeply: %d", MAX_INCL_DEPTH);
		QMessageBox p; p.setText(buff.data());p.show();

		return cleanUp();
	}

	FILE *test = fopen(inc.data(),"r");
	if(test)
	{
		stacker[include_count].setLine(lineNumber);
		include_stack[include_count++] = YY_CURRENT_BUFFER;
		if(BaseComponent::count()) 
		{
		    BaseComponent *t = BaseComponent::at(
				BaseComponent::count()-1);
	   	    stacker[include_count].setDepth(t->queryPosition());
		}

		yyin = test;
		lineNumber = 0;
		yy_switch_to_buffer(yy_create_buffer(yyin,YY_BUF_SIZE));
	}
	else
	{
		QString buff;buff.sprintf("Bad @incl location: %s.",inc.data());
		QMessageBox p; p.setText(buff.data()); p.show();
	}
}
<*><<EOF>> {
	if ( --include_count < 0 )
		return cleanUp();
	else
	{
	    lineNumber = stacker[include_count].line();
            yy_switch_to_buffer(include_stack[include_count]);
	    if(BaseComponent::count())
	    {
	        BaseComponent *t = BaseComponent::at(BaseComponent::count()-1);
	        stacker[include_count].setDepth(t->queryPosition());
	    }
	}

           LXDEBUG("Into a Include")
}

<INITIAL>{ININITIAL} {
           LXDEBUG("Into a {Initial}")
	QString v = parseValue();

	if(!strcmp(yytext,"start_dir")) stacker[include_count].setDir(v.data());

           LXDEBUG("Out of {Initial}")
}

<INITIAL>globals_editor{WHITE}\{ {
	BEGIN(GLOBALS);
}
<GLOBALS>{INGLOBALS} {
           LXDEBUG("Into a {Inglobals}")

    QString v = parseValue();


    if(!strcmp(yytext,"header")) global->appendHeader(v.data());
    else if(!strcmp(yytext,"signature")) global->appendComment(v.data());
    else if(!strcmp(yytext,"raw")) global->appendFreeHand(v.data());
    else if(!strcmp(yytext,"main_func")) global->appendFunction(v.data());

           LXDEBUG("Out of {Inglobals}")

}
<GLOBALS>\}\;{WHITE}globals_editor {
	BEGIN(INITIAL);
}

<INITIAL,COMPONENT>^{WHITE}component {

   LXDEBUG("Into a {start-component}")


   /* Retreive type of this component */
    int c; QString concat(""), type; 
    while((c = yyinput()) != '{' && c != EOF)	concat += (char)c;
    type = concat.simplifyWhiteSpace(); 

    // if there is a colon there, then we can assume there is an explicitly
    // defined parent (normally lack of one, 0)
    BaseComponent *parent = componentStack.last();
    int colon;
    if((colon=concat.find(':')) != -1)
    {
	if(!concat.right(concat.length()-(colon+1)).toInt())
              parent=0;    

	type = concat.left(colon).simplifyWhiteSpace();
     }

     if(tmpComponent) tmpComponent->applyAttribs();

     //There must only be one application, so if another one in an included
     //file then ignore it.	
     if(type == "Application" && BaseComponent::count())
	tmpComponent = BaseComponent::at(0);
     else
	tmpComponent = BaseComponent::ofType(parent, type);

     connectionStepup = stacker[include_count].depth();
     if(CompIO::openPurpose() == O_IMPORT && componentStack.last())
           connectionStepup += componentStack.last()->queryPosition();

    tmpComponent->hide();
    attrib->clearAttributes(); 
    tmpComponent->fillAttributes();
    componentStack.append(tmpComponent);
    toRet = tmpComponent->queryPosition();
	
    menubar->stepLevel();
    QString load(tmpComponent->name());
    load.insert(0,"Loading: ");
    menubar->setCaption(load.data());

    yy_push_state(COMPONENT);

           LXDEBUG("Out of {start-component}")
}
<COMPONENT>{INCOMPONENT} {
           LXDEBUG("Into a {Incomponent}")
    QString v = parseValue();


    if(!strcmp(yytext,"locked"))
           tmpComponent->setLock(v.toInt());
    else if(!strcmp(yytext,"showChildren"))
           tmpComponent->setShowChildren(v.toInt());
    else if(!strcmp(yytext,"attrib"))
    {
           QString capt(""), value(""); char c; char *dat = v.data();
	   while((c = *dat++) != '"' && c != '\0');
	   while((c = *dat++) != '"' && c != '\0') capt += (char)c;
	   while((c = *dat++) != '"' && c != '\0');
	   while((c = *dat++) != '"' && c != '\0') value += (char)c;

	   attrib->setValue(capt,value);
    }
    else if(!strcmp(yytext,"header"))
	tmpComponent->appendHeader(v.data());
    else if(!strcmp(yytext,"visable"))
	    tmpComponent->setGoingVis(v.toInt());
    else if(!strcmp(yytext,"sigslot") && CompIO::openPurpose() != O_PASTE) 
	tmpComponent->membInfo()->addConnection(SignalConnect(v.data(),
						connectionStepup));

           LXDEBUG("Out of {Incomponent}")
}
<COMPONENT>function{WHITE}\{ {
	oneFunction.clear();

	BEGIN(FUNCTION);
}
<FUNCTION>{INFUNCTION} {
           LXDEBUG("Into a {Infunction}")
    QString v = parseValue(); v.stripWhiteSpace();

    
    if(!strcmp(yytext,"prototype")) oneFunction.setPrototype(v.data()); 
    else if(!strcmp(yytext,"baseinit"))  oneFunction.setBaseinit(v.data());
    else if(!strcmp(yytext,"code"))  
    {
	if(!oneFunction.source()->isEmpty()) oneFunction.appendSource("\n");
	oneFunction.appendSource(v.data());
    }
    else if(!strcmp(yytext,"prot_mode"))  oneFunction.setProtection(v.toInt());
    else if(!strcmp(yytext,"virtual"))  oneFunction.setVirt(v.toInt());
    else if(!strcmp(yytext,"defined"))  oneFunction.setDefined(v.toInt());
    else if(!strcmp(yytext,"readonly")) oneFunction.setReadonly(v.toInt());

	LXDEBUG("Out of {function}")

}
<FUNCTION>\}\;{WHITE}function {
	tmpComponent->membInfo()->add(oneFunction);
	
	BEGIN(COMPONENT);
}
<COMPONENT>menubar{WHITE}\{ {
	oneMenubar.clear();

	BEGIN(MENUBAR);
}
<MENUBAR>{INMENUBAR} {
           LXDEBUG("Into a {Inmenubar}")
    QString v = parseValue();

    if(!strcmp(yytext,"name")) oneMenubar.setNam(v.data());
    else if(!strcmp(yytext,"caption")) oneMenubar.setCapt(v.data());
    else if(!strcmp(yytext,"shortcut")) oneMenubar.setStC(v.data());
    else if(!strcmp(yytext,"checked")) oneMenubar.setChk(v.toInt());
    else if(!strcmp(yytext,"enabled")) oneMenubar.setEna(v.toInt());
    else if(!strcmp(yytext,"level")) oneMenubar.setLev(v.toInt());
    else if(!strcmp(yytext,"connect"))
    {
	int connectObj, connectSlot;
	v = v.simplifyWhiteSpace();
	connectObj = v.left(v.find(':')).toInt();
	connectSlot = v.right(v.length()-(v.find(':')+1)).toInt();

	//Add the step up to it if it's no -1 and it's not to the constant
	//0 (Application)
	if(connectObj > 0) connectObj+=connectionStepup;

	oneMenubar.setConO(connectObj);
	oneMenubar.setConS(connectSlot);	
    }

           LXDEBUG("Out of {Inmenubar}")

}
<MENUBAR>\}\;{WHITE}menubar {
	tmpComponent->membInfo()->appendMenu(oneMenubar);

	BEGIN(COMPONENT);
}
<COMPONENT>\}\;{WHITE}component {
           LXDEBUG("Into a {end-component}")

        if(tmpComponent)
        {
                tmpComponent->applyAttribs();
		if(tmpComponent->queryMenu()) tmpComponent->makeMenu();
		if(tmpComponent->goingVis()) tmpComponent->show();
		tmpComponent->fillMembInfo();
        }

        QObject::connect(tmpComponent,SIGNAL(currentW(int)),
                         attrib,SLOT(changeToObject(int)));
        QObject::connect(tmpComponent,SIGNAL(currentW(int)),
                         source,SLOT(changeToObject(int)));
        QObject::connect(tmpComponent,SIGNAL(currentW(int)),
                         sigSlot,SLOT(changeToObject(int)));
        QObject::connect(tmpComponent,SIGNAL(currentW(int)),
                         projView,SLOT(changeToObject(int)));
        QObject::connect(tmpComponent,SIGNAL(currentW(int)),
                         menu,SLOT(changeToObject(int)));
	QObject::connect(tmpComponent,SIGNAL(currentW(int)),
			toolBar,SLOT(changeObj(int)));

        componentStack.removeLast();
        tmpComponent = 0;
        //just leave it null if the last on stack, is also the first (project from called function)
        if(componentStack.count() != 1) tmpComponent = componentStack.last();
        if(tmpComponent)
	{
		attrib->clearAttributes(); 
		tmpComponent->fillAttributes(); 
	}
        yy_pop_state();

           LXDEBUG("Out of a {end-component}")
}

%%

char
*parseValue()
{
    int c, flag=0; static QString concat; concat = "";
    while((c = yyinput()) != '{' && c != EOF);
    while((c = yyinput()) != EOF && !flag)
    {
	if(c == '}')
	{
	    if( (c = yyinput()) == ';')
		flag = 1;
	    else
		unput(c);
	    c = '}';
	}
	if(!flag) concat += (char)c;
    }

    /* Only take off first and last space if necesary.. */
    if(concat.at(0) == ' ') concat.remove(0,1);
    if(concat.length() && concat.at(concat.length()-1) == ' ')
	concat.remove(concat.length()-1,1);

    return concat.data();
}

int
cleanUp()
{
	include_count = 0;
        lineNumber = 0;
        for(int x = 0; x < MAX_INCL_DEPTH; x++) stacker[x].clear();
        return toRet;
}
