/*  $Id: LinkList.cc,v 1.7 1998/07/27 16:35:06 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "LinkList.h"
#include "BaseComp.h"
#include "../main.h"

#include <qregexp.h>
#include <qobjcoll.h>

LinkList::LinkList()
{
        flag = 0;
	headers = "";
        functionList.setAutoDelete(TRUE);
        mnuInfo.setAutoDelete(TRUE);
        connectList.setAutoDelete(TRUE);
}

//Returns the full prototype of the member at x in the list
QString
*LinkList::memb(int x)
{
	if(x > count()-1) return 0;
	return functionList.at(x)->prototype();	
}

//returns the prototype of the member sans variable names and the
// return type.
QString
LinkList::membName(int x)
{
	QString p = membRight(x).data();

	if(p.find("()") != -1) return p;

	p.replace(QRegExp("\\s[a-zA-Z0-9^,^(]*[^=],"),",");
	p.replace(QRegExp("\\*[a-zA-Z0-9^,^(]*[^=],"),"*,");
	p.replace(QRegExp("  &[a-zA-Z0-9^,^(]*[^=],"),"&,");

	p.replace(QRegExp("\\s[a-zA-Z0-9^,^(]*[^=])"),")");
	p.replace(QRegExp("\\*[a-zA-Z0-9^,^(]*[^=])"),"*)");
	p.replace(QRegExp(  "&[a-zA-Z0-9^,^(]*[^=])"),"&)");

	p.replace(QRegExp("\\s[a-zA-Z0-9^,^(]*="),"=");
	p.replace(QRegExp("\\*[a-zA-Z0-9^,^(]*="),"*=");
	p.replace(QRegExp(  "&[a-zA-Z0-9^,^(]*="),"&=");

	return p;
}

//remove default values from the function
QString
LinkList::membNoDefs(int x)
{
	QString p = membRight(x).data();

	if(p.find("()") != -1) return p;

	p.replace(QRegExp("=.*,"),",");
	p.replace(QRegExp("=.*)"),")");

	return p;
}

//returns the member definition except the return value
QString
LinkList::membRight(int x)
{
    QString p = memb(x)->data();

    if(p.find('(') != -1)
    {
	QString farRight = p.right(p.length()-p.find('('));
	p = p.left(p.find('(')).stripWhiteSpace();
	p = p.right(p.length()-p.findRev(' ')).stripWhiteSpace();p += farRight;
    }
    return p;
}

//returns the return value
QString
LinkList::retType(int x)
{
    QString p = memb(x)->data(); p.stripWhiteSpace();
    if(p.find('(') != -1) p = p.left(p.find('(')).stripWhiteSpace();
    if(p.find(' ') != -1)
	p = p.left(p.findRev(' '));
    else
	p = "";
    return p;
}


//queryies sig/slot connection at x
SignalConnect
*LinkList::connect(int x)
{
	return connectList.at(x);
}


//checks if it finds a match for the connection given
//connections are stored in the format x:y:z (x being the signal's respective 
//object's position, y being the signal's position itself, and z being the
//slot it's connected to's position. This is all stored in the slot's 
//respective position's memberInfo.
int
LinkList::connected(SignalConnect p)
{
	if(connectList.find(&p) != -1)
		return 1;
	else
		return 0;
}

//returns the base initializer of the member at x
QString
*LinkList::baseInit(int x)
{
	return functionList.at(x)->baseinit();
}

//returns the source code of the member at x
QString
*LinkList::source(int x)
{
	return functionList.at(x)->source();
}

int
LinkList::isConstructor(int x)
{
	QString i(memb(x)->data());
	return (i.left(i.find('(')).stripWhiteSpace() == "constructor");
}

int
LinkList::isDestructor(int x)
{
        QString i(memb(x)->data());
        return (i.left(i.find('(')).stripWhiteSpace() == "destructor");
}

int
LinkList::startLine(int x)
{
    return functionList.at(x)->startLine();
}

//returns whether or not the member is defined/declared within this
//class in question
int
LinkList::define(int x)
{
	return functionList.at(x)->defined();
}

int
LinkList::isReadOnly(int x)
{
	return functionList.at(x)->readonly();
}

//returns whether the member is virtual
int
LinkList::virt(int x)
{
	return functionList.at(x)->virt();
}

//returns the mode, values are listed in the top of the accomponing 
//header file

int
LinkList::mode(int x)
{
	return functionList.at(x)->protection();
}

MenubarElement
*LinkList::menu(int where)
{
	return mnuInfo.at(where);
}

//adds a member with definition memb, source code of source, and says
//whether is virtual, it's mode (protect, private, etc), and whether it's
//defined locally or not
int
LinkList::add(char *memb, char *bi, char *source, int virt, int mode, 
	int d)
{
    return add(FunctionElement(memb,bi,source,virt,mode,d));
}

int
LinkList::add(FunctionElement p)
{
    int matchAt;
    if((matchAt = functionList.find(&p)) != -1)  return 0;

    QString b(p.prototype()->data());
    b = b.left(b.find('(')); b = b.stripWhiteSpace();
    if(b == "constructor" || b == "destructor")
	functionList.insert(0,new FunctionElement(p));
    else
	functionList.append(new FunctionElement(p));
    return 1;
}

//inserts a connection at place with the string of memb (described above)
void
LinkList::insertConnect(SignalConnect memb, int place)
{
	connectList.insert(place, new SignalConnect(memb));
}

void
LinkList::fixConnections()
{
        LinkList *temp;
        int bye, num = BaseComponent::count();
        for(int x = 0; x < num; x++)
        {
                temp = BaseComponent::at(x)->membInfo();
                for(int y = 0; y < temp->connectList.count(); y++)
                {
                        bye = 0;
                        if(!BaseComponent::at(
                             temp->connect(y)->querySigOwner()) ||
                          !BaseComponent::at(
                             temp->connect(y)->querySlotOwner()))
                        {
                                bye = 1;
                        }
                        else if(!BaseComponent::at(
                             temp->connect(y)->querySigOwner())->
                                membInfo()->memb(
                             temp->connect(y)->querySig()) ||
                          !BaseComponent::at(
                             temp->connect(y)->querySlotOwner())->
                                membInfo()->memb(
                             BaseComponent::at(
                             temp->connect(y)->querySlotOwner())->
                                membInfo()->signalSlot(temp->connect(y)->
                                 querySlot())))
                        {
                                bye = 1;
                        }
                        if(bye)
                        {
                                temp->removeConnection(y);
                                if(y < count()-1) y--;
                        }
                }
        }
}

//This will allow a child widget to be inserted before the next parent widget
int
LinkList::insertObject(BaseComponent *par)
{
	LinkList *temp;

	int st = par->nextParent(par->queryPosition());
	if(st == -1) st = BaseComponent::count();

	for(int x = BaseComponent::count()-1; x >= st; x--)
	{
		temp = BaseComponent::at(x)->membInfo();
		for(int y = 0; y < temp->connectList.count(); y++)
		{
			if(temp->connect(y)->querySlotOwner() >= st)
                        {
                        	temp->connect(y)->setSlotOwner(
                                temp->connect(y)->querySlotOwner()+1);
                         }
                         if(temp->connect(y)->querySigOwner() >= st)
                         {
				temp->connect(y)->setSigOwner(
                                temp->connect(y)->querySigOwner()+1);
                         }
		}
		for(int y = 0; y < temp->menuCount(); y++)
		{
			if (temp->menu(y)->queryConO() >= st)
			{
				temp->menu(y)->setConO(
				temp->menu(y)->queryConO()+1);
			}
		}		
		BaseComponent::at(x)->setPosition(x+1);
	}
	return st;
}

//this is used to remove a member, the reason for the extra code is
//signals/slots will need to be shifted due to the member's position
//changing. If this is called with a -1 place it means the object itself
//is being deletted and the member must take care of remving ALL connections
//that deal with this object
void
LinkList::remove(int place, BaseComponent *p)
{
	int numSlots = 0;
	int y,x,z;
	LinkList *temp;

	if(place == -1)
	{
		for(x = 0; x < p->count(); x++)
		{
			temp = p->at(x)->membInfo();
                        for(y = 0; y < temp->connectList.count(); y++)
                        {
                                if(temp->connect(y)->querySlotOwner() ==
                                        p->queryPosition())
                                {
                                        temp->removeConnection(y);
                                        y--;
                                }
				else if(temp->connect(y)->querySigOwner() ==
					p->queryPosition())
				{
					temp->removeConnection(y);
					y--;
				}
                        }
			for(y = 0; y < temp->connectList.count(); y++)
			{
				if(temp->connect(y)->querySlotOwner() > 
					p->queryPosition())
				{
			              	temp->connect(y)->setSlotOwner(
					 temp->connect(y)->querySlotOwner()-1);
				}
				if(temp->connect(y)->querySigOwner() >
					p->queryPosition())
				{
					temp->connect(y)->setSigOwner(
					 temp->connect(y)->querySigOwner()-1);
				}
			}

	                for(y = 0; y < temp->menuCount(); y++)
        	        {
				if (temp->menu(y)->queryConO() == 
					p->queryPosition())
				{
					temp->menu(y)->setConO(-1);
				}	
                	        if (temp->menu(y)->queryConO() > 
					p->queryPosition())
	                        {
	                                temp->menu(y)->setConO(
	                                temp->menu(y)->queryConO()-1);
        	                }
	                }
		}
	}
	else if(mode(place) == M_SIGNAL)
        {
		for(y = 0; y < connectList.count(); y++)
		{
			if(connect(y)->querySig() == place)
			{
				removeConnection(y);
				y--;	
			}
		}				
                for(y = 0; y < connectList.count(); y++)
                {
			if(connect(y)->querySig() > place)
			{
				connect(y)->setSig(
				connect(y)->querySig()-1);
			}
		}
	}
	
	else if(mode(place) == M_PUB_SLOT)
	{
		for(z = 0; z < place; z++)
		   if(mode(z) == M_PUB_SLOT) numSlots++;

                for(x = 0; x < p->count(); x++)
                {
                        temp = p->at(x)->membInfo();

			for(y = 0; y < temp->connectList.count(); y++)
			{
				if(temp->connect(y)->querySlot() == numSlots &&
                	           temp->connect(y)->querySlotOwner() ==
                        	        p->queryPosition())
	                        {
	                                temp->removeConnection(y);
        	                	y--;
				}
	                }
			for(y = 0; y < temp->connectList.count(); y++)
			{
				if(temp->connect(y)->querySlot() > numSlots &&
				   temp->connect(y)->querySlotOwner() ==
					p->queryPosition())
				{
					temp->connect(y)->setSlot(
					 temp->connect(y)->querySlot()-1);
				}
			}
                        for(y = 0; y < temp->menuCount(); y++)
                        {
                                if (temp->menu(y)->queryConS() == numSlots &&
                                        temp->menu(y)->queryConO() ==
                                                p->queryPosition())
                                {
                                        temp->menu(y)->setConO(-1);
                                }
                        }
	                for(y = 0; y < temp->menuCount(); y++)
        	        {
                	        if (temp->menu(y)->queryConS() > numSlots &&
					temp->menu(y)->queryConO() == 
						p->queryPosition())
	                        {
	                                temp->menu(y)->setConO(
	                                temp->menu(y)->queryConO()-1);
	                        }
	                }
		}
	}
	if(place != -1)
	    functionList.remove(place);
}

//simply adds a connection
void 
LinkList::addConnection(SignalConnect p)
{
	connectList.append(new SignalConnect(p));
}

//removing a connection
void
LinkList::removeConnection(int x)
{
	connectList.remove(x);
}

//this is the count of members within memory for thisi instance
int
LinkList::count()
{
	return functionList.count();
}

//this is the count of connections stored in this instance
int
LinkList::connectCount()
{
	return connectList.count();
}

//changes the source at x to p
void
LinkList::setSource(int x, QString p)
{
                functionList.at(x)->setSource(p.data());
}

//change the base initilizer at x to p
void
LinkList::setBaseInit(int x, QString p)
{
                functionList.at(x)->setBaseinit(p.data());
}

//changes the member definition at x to p
void
LinkList::setMemb(int x, QString p)
{
       functionList.at(x)->setPrototype(p.data());
}

void
LinkList::setStartLine(int x, int y)
{
    functionList.at(x)->setStartLine(y);
}

void
LinkList::setReadOnly(int x, int y)
{
    functionList.at(x)->setReadonly(y);
}

//this function (based on whether on is on or not) will toggle whether the
//connection with look p will cease to exist or not
void
LinkList::setConnect(SignalConnect p, int on)
{
	int x;
	if(on)
	{
                if((x = connectList.find(&p)) == -1)
                        addConnection(p);
                else
                {
                        connectList.remove(x);
                        addConnection(p);
                }
	}
	else
	{
                if((x = connectList.find(&p)) != -1)
                        connectList.remove(x);
	}
}

//sets member x to virtual based on val
void
LinkList::setVirt(int x, int val)
{
    functionList.at(x)->setVirt(val);
}

//sets whether member x is defined locally based on val
void
LinkList::setDef(int x, int val)
{
    functionList.at(x)->setDefined(val);
}
	
//sets the protection mode of the member at x based on val
void
LinkList::setMode(int x, int val)
{
    functionList.at(x)->setProtection(val);
}

//this is used when deciding whether a member passes inspection or if
//it needs to be written to file, this is normally based on two 
//contingonsies, a) if the member is defined locally and b) if the 
//source is not empty, however in the case of a signal we declare them
//with empty source, hence the cb flag
int
LinkList::passInspection(int x, int mo, int cb)
{
	QString t(memb(x)->data());
        t = t.left(t.find('(')); t = t.stripWhiteSpace();
	if((t == "constructor" || t == "destructor") && mo == 2) return 1;

	if(cb == 1)   /* Will not concern with the source value */
	{
		if(mode(x) == mo)
		{
			if(define(x) || memb(x)->find('(') == -1)
				return 1;
		}
	}
	else 
	{
		if(mode(x) == mo)
		{
			if((!source(x)->isEmpty() && define(x)) || 
				memb(x)->find('(') == -1)
                	    return 1;
		}
	}
	return 0;
}

//this returns the slot position of x, for example memb 13 maybe be
//the fifth slot in memory for this instance, so 13 is returned from 5
//being passed in, this is used in the connection string (third element)
int
LinkList::slot(int x)
{
	int retVal = 0;
	for(int y = 0; y < count(); y ++)
	{
		if(mode(y) == M_PUB_SLOT)
		{
			if(retVal++ == x) return y;
		}
	}
	return count();
}

//like above for signals
int
LinkList::signal(int x)
{
        int retVal = 0;
        for(int y = 0; y < count(); y ++)
        {
                if(mode(y) == M_SIGNAL)
                {
                        if(retVal++ == x) return y;
                }
        }
        return count();
}

char
*LinkList::signalSlotStr(int where)
{
	static QString i; int y = signalSlot(where);
	if(mode(y) == M_SIGNAL)
		i.sprintf("SIGNAL(%s)",membName(y).data());
	else
		i.sprintf("SLOT(%s)",membName(y).data());
        return i.data();
}

int
LinkList::signalSlot(int where)
{
        int retVal = 0;
        for(int y = 0; y < count(); y++)
        {
                if(mode(y) == M_SIGNAL || mode(y) == M_PUB_SLOT)
                {
                        if(retVal++ == where) return y;
                }
        }
        return count();
}

void
LinkList::removeMenu(int where)
{
	mnuInfo.remove(where);
}

void
LinkList::insertMenu(int where)
{
	mnuInfo.insert(where,new MenubarElement());
}

void 
LinkList::clearMenus()
{
	mnuInfo.clear();
}

void
LinkList::appendMenu(MenubarElement e)
{
	mnuInfo.append(new MenubarElement(e));
}

void 
LinkList::modifyMenu(int where, QString cap, QString na, int conO, int conS,
                QString scut, int chk, int ena, int lev)
{
	if((menuCount() - 1) < where)
		appendMenu(MenubarElement(cap,na,conO,conS,scut,chk,ena,lev));
	else
	{
		mnuInfo.remove(where);
		mnuInfo.insert(where,
			 new MenubarElement(cap,na,conO,conS,scut,chk,ena,lev));
	}
}

int
LinkList::menuCount()
{
	return mnuInfo.count();
}

void
LinkList::raise(int where)
{
	BaseComponent *pa;
	pa = (BaseComponent *)BaseComponent::at(where)->parentWidget();
	if(!pa) return;

	int y;

        LinkList *temp;

	//Shift general signal/slots	
        int newWhere = pa->queryPosition();
	for(y =  0; y < BaseComponent::count(); y++)
	{
		temp = BaseComponent::at(y)->membInfo();
		for(int z = 0; z < temp->connectList.count(); z++)
		{
			//Temporarily set to -1 to flag ones that need to
			//Be set to the new top.
                        if(temp->connect(z)->querySlotOwner() == where)
                        {
                                temp->connect(z)->setSlotOwner(-1);
                        }
                        if(temp->connect(z)->querySigOwner() == where)
                        {
                                temp->connect(z)->setSigOwner(-1);
                        }
		}
                for(int z = 0; z < temp->connectList.count(); z++)
                {
                        if(temp->connect(z)->querySlotOwner() < where &&
                                temp->connect(z)->querySlotOwner() > newWhere)
                        {
                                temp->connect(z)->setSlotOwner(
                                        temp->connect(z)->querySlotOwner()+1);
                        }
                        if(temp->connect(z)->querySigOwner() < where &&
                                temp->connect(z)->querySigOwner() > newWhere)
                        {
                                temp->connect(z)->setSigOwner(
                                        temp->connect(z)->querySigOwner()+1);
                        }
                }
                for(int z = 0; z < temp->connectList.count(); z++)
                {
			//Set flagged (above) appropriatly.
                        if(temp->connect(z)->querySlotOwner() == -1)
                                temp->connect(z)->setSlotOwner(newWhere+1);
                        if(temp->connect(z)->querySigOwner() == -1)
                                temp->connect(z)->setSigOwner(newWhere+1);
		}

		//Shift menus connections
		for(int z = 0; z < temp->menuCount(); z++)
                {
                        if (temp->menu(z)->queryConO() == where)
                        {
				//-2 because -1 is used as none :)
                                temp->menu(z)->setConO(-2);
                        }
		}
		for(int z = 0; z < temp->menuCount(); z++)
		{
			if(temp->menu(z)->queryConO() < where &&
				temp->menu(z)->queryConO() > newWhere)
			{
				temp->menu(z)->setConO(
					temp->menu(z)->queryConO()+1);
			}
		}
		for(int z = 0; z < temp->menuCount(); z++)
                {
			//Set flagged (above) appropriatly
                        if (temp->menu(z)->queryConO() == -2)
                        {
                                temp->menu(z)->setConO(newWhere+1);
                        }
		}
	}

       //Now shift down all the elements.
        BaseComponent *hold = BaseComponent::at(where);
        for(int x = where-1; x > newWhere; x--)
        {
                 BaseComponent::at(x)->setPosition(x+1);
        }

	newWhere++;
        hold->setPosition(newWhere);
        hold->hRaise();
        BaseComponent::at(0)->makeChange(pa->queryPosition());

	//Update lists to account for new order
	if(pa->queryPosition() == BaseComponent::queryCurrPar())
	{
		::attrib->removeObject(where);
		::source->removeObject(where);
		::menu->removeObject(where);
		::sigSlot->removeObject(where);

		QString i;
		i.sprintf("%s *%s",hold->queryQType(),(char *)hold->name());
	
		::attrib->addObject(i.data(),0,1);
		::source->addObject(i.data(),0,1);
		::menu->addObject(i.data(),1);
		::sigSlot->addObject(i.data(),0,1);

	}
	projView->moveObject(where,newWhere);

	if(BaseComponent::at(newWhere)->numChildren())
	{
	    flag = 1;

	    QObjectList *p=BaseComponent::at(newWhere)->queryList(
			"BaseComponent",0,TRUE,FALSE);	
	    for(int toRaise = 0; toRaise < p->count(); toRaise++)
	    {
		BaseComponent *temp = (BaseComponent *)p->at(toRaise);
		if(temp != BaseComponent::at(newWhere))
			raise(temp->queryPosition());
	    }
	    delete p;

	    flag = 0;
	}
	if(!flag) projView->showBuiltTree();
}		

//I could have obviously simply added another parameter the the raise(int) 
//func but I think this will improve readibility in the days to come - if
//this source code is adopted, because it could get a little ugly up there,
//however I may still choose to combine the efforts for the sake of lessening
//the size ;)
void
LinkList::lower(int where)
{
	int x;
        BaseComponent *pa;
        pa = (BaseComponent *)BaseComponent::at(where)->parentWidget();
        if(!pa) return;

        int y;

        LinkList *temp;

        //Shift general signal/slots
        int newWhere = pa->queryPosition();
        for(x = pa->queryPosition()+1; x < BaseComponent::count();x++)
        { 
	    if(BaseComponent::at(x)->queryLevel() <= pa->queryLevel()) break;
  	    newWhere++; 
	}

        for(y =  0; y < BaseComponent::count(); y++)
        {
                temp = BaseComponent::at(y)->membInfo();
                for(int z = 0; z < temp->connectList.count(); z++)
                {
                        //Temporarily set to -1 to flag ones that need to
                        //Be set to the new bottom.
                        if(temp->connect(z)->querySlotOwner() == where)
                        {
                                temp->connect(z)->setSlotOwner(-1);
                        }
                        if(temp->connect(z)->querySigOwner() == where)
                        {
                                temp->connect(z)->setSigOwner(-1);
                        }
                }
                for(int z = 0; z < temp->connectList.count(); z++)
                {
                        if(temp->connect(z)->querySlotOwner() > where &&
                                temp->connect(z)->querySlotOwner() <= newWhere)
                        {
                                temp->connect(z)->setSlotOwner(
                                        temp->connect(z)->querySlotOwner()-1);
                        }
                        if(temp->connect(z)->querySigOwner() > where &&
                                temp->connect(z)->querySigOwner() <= newWhere)
                        {
                                temp->connect(z)->setSigOwner(
                                        temp->connect(z)->querySigOwner()-1);
                        }
                }
                for(int z = 0; z < temp->connectList.count(); z++)
                {
                        //Set flagged (above) appropriatly.
                        if(temp->connect(z)->querySlotOwner() == -1)
                                temp->connect(z)->setSlotOwner(newWhere);
                        if(temp->connect(z)->querySigOwner() == -1)
                                temp->connect(z)->setSigOwner(newWhere);
                }

                //Shift menus connections
                for(int z = 0; z < temp->menuCount(); z++)
                {
                        if (temp->menu(z)->queryConO() == where)
                        {
                                //-2 because -1 is used as none :)
                                temp->menu(z)->setConO(-2);
                        }
                }
                for(int z = 0; z < temp->menuCount(); z++)
                {
                        if(temp->menu(z)->queryConO() > where &&
                                temp->menu(z)->queryConO() <= newWhere)
                        {
                                temp->menu(z)->setConO(
                                        temp->menu(z)->queryConO()-1);
                        }
                }
                for(int z = 0; z < temp->menuCount(); z++)
                {
                        //Set flagged (above) appropriatly
                        if (temp->menu(z)->queryConO() == -2)
                        {
                                temp->menu(z)->setConO(newWhere);
                        }
                }
        }

       //Now shift down all the elements.
        BaseComponent *hold = BaseComponent::at(where);
        for(x = where+1; x <= newWhere; x++)
        {
                 BaseComponent::at(x)->setPosition(x-1);
        }
        hold->setPosition(newWhere);
        hold->hLower();
        BaseComponent::at(0)->makeChange(pa->queryPosition());

        //Update lists to account for new order
        if(pa->queryPosition() == BaseComponent::queryCurrPar())
        {
                ::attrib->removeObject(where);
                ::source->removeObject(where);
                ::menu->removeObject(where);
                ::sigSlot->removeObject(where);

                QString i;
                i.sprintf("%s *%s",hold->queryQType(),(char *)hold->name());

                ::attrib->addObject(i.data(),0,
			BaseComponent::positionToBox(newWhere));
                ::source->addObject(i.data(),0,
			BaseComponent::positionToBox(newWhere));
                ::menu->addObject(i.data(),
			BaseComponent::positionToBox(newWhere));
                ::sigSlot->addObject(i.data(),0,
			BaseComponent::positionToBox(newWhere));

        }
	projView->moveObject(where,newWhere);

        if(BaseComponent::at(newWhere)->numChildren())
        {
	    flag = 1;

            QObjectList *p=BaseComponent::at(newWhere)->queryList(
                        "BaseComponent",0,TRUE,FALSE);
            for(int toRaise = p->count()-1; toRaise >= 0; toRaise--)
            {
                BaseComponent *temp = (BaseComponent *)p->at(toRaise);
                if(temp != hold) lower(temp->queryPosition());
            }
            delete p;

            flag = 0;

	    //Make sure the children aren't lower then hold's myLook
	    hold->hLower();
        }

	if(!flag) projView->showBuiltTree();
}

