/*  $Id: zButtonGroup.cc,v 1.4 1998/07/27 16:35:06 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "zButtonGroup.h"
#include <qobjcoll.h>

zButtonGroup::zButtonGroup(QWidget *parent)
					: BaseFrame(parent,"buttonGroup")
{
        look = new QButtonGroup(this,"look");
	look->setTitle("");

	look->setAlignment(AlignLeft);
	align = new QString("Left");

	prepFrame(look);
	setLook(look);
}

zButtonGroup::~zButtonGroup()
{
	//Delete your stuff, but don't delete look!
	delete align;
}

void 
zButtonGroup::fillAttributes()
{
        ADDATTRIB("Title",(char *)look->title(),W_TEXT);

        ADDATTRIB("Title Alignment",align->data(),W_LIST,3,
        "Left", "Center", "Right");

        ADDATTRIB("Background Colour",look->backgroundColor(),W_COLOR);

	ADDATTRIB("Exclusive",look->isExclusive(),W_TRUE);
	
        ADDATTRIB("Font",(char *)myLook->font().family(),W_LIST,fontList);

        ADDATTRIB("Font Size",myLook->font().pointSize(),W_LIST,24,
              "10", "12", "14", "16", "18", "20", "22", "24", "26", "28",
              "30", "32", "34", "36", "38", "40", "44", "48", "52", "56",
              "60", "64", "68", "72");

        ADDATTRIB("Bold",myLook->font().bold(),W_TRUE);

        ADDATTRIB("Italic",myLook->font().italic(),W_TRUE);

        ADDATTRIB("Tool Tip",tooltipData->data(),W_TEXT);

	genericFillAttrib();

	showFrameAttrib(look);
}

void 
zButtonGroup::interpretAttributes()
{
	int x;
	QString i;

	look->setTitle(attrib->giveValue("Title"));

	*align = attrib->giveValue("Title Alignment");
	if(*align == "Left")
		look->setAlignment(AlignLeft);
	else if(*align == "Right")
		look->setAlignment(AlignRight);
	else if(*align == "Center")
		look->setAlignment(AlignCenter);

	look->setBackgroundColor(attrib->giveColor("Background Colour"));

	look->setExclusive(attrib->giveBoolValue("Exclusive"));

	int bo = attrib->giveBoolValue("Bold") ? QFont::Bold : QFont::Normal;
	myLook->setFont(QFont(attrib->giveValue("Font").data(),
                          attrib->giveIntValue("Font Size"),
                          bo,attrib->giveBoolValue("Italic")));

	*tooltipData = attrib->giveValue("Tool Tip");


        genericInterpAttrib();

	interpFrameAttrib(look);
}

void
zButtonGroup::writeInits()
{
	QString i;

	i.sprintf("setTitle(\"%s\");\n",(char *)look->title());
        WRITEATTRIB("Title", i);

        QString xtc;
        if(look->font().bold())
                xtc = "QFont::Bold";
        else
                xtc = "QFont::Normal";
        i.sprintf("setFont(QFont(\"%s\",%d,%s,(int)%d));\n",
        (char *)look->font().family(),look->font().pointSize(),
        xtc.data(),look->font().italic());
       if(isWritten("Font Size") || isWritten("Font") || 
	  isWritten("Bold") || isWritten("Italic"))
	   writeInParent(i);

        i.sprintf(
        "setAlignment(Align%s);\n"
                ,align->data());
        WRITEATTRIB("Title Alignment",i);

        i.sprintf(
        "setGeometry(%d,%d,%d,%d);\n"
                ,x(),y(),width(),height());
       writeInParent(i);


       if(isWritten("Maximum Width") || isWritten("Maximum Height"))
       {
	   i.sprintf("setMaximumSize(%d,%d);\n",
		     look->maximumSize().width(),look->maximumSize().height());
	   writeInParent(i);
       }
       if(isWritten("Minimum Width") || isWritten("Minimum Height"))
       {
	   i.sprintf("setMinimumSize(%d,%d);\n",
		     look->minimumSize().width(),look->minimumSize().height());
	   writeInParent(i);
       }

        if(!tooltipData->isEmpty())
        {
                QString p;
                if(noFuncs() == TRUE)
                        p = name();
                else
                        p = "this";

                i.sprintf(
                "QToolTip::add(%s,\"%s\");\n"
                        ,p.data(),tooltipData->data());
                writeLine(i);
        }

        writePalette();

        if(*uiStyle == "Motif")
               i.sprintf("setStyle(MotifStyle);\n");
        else if(*uiStyle == "Windows")
                i.sprintf("setStyle(WindowsStyle);\n");
        else
                i = "";
        if(!i.isEmpty()) writeInParent(i);

	i.sprintf("setExclusive(%s);\n",strBool(look->isExclusive()));
	WRITEATTRIB("Exclusive",i);

	writeFrame();
}

void
zButtonGroup::postWriteInits()
{
	QString i;
        QObjectList *ol = queryList("BaseComponent",0,TRUE,FALSE);
	for(int x = 0; x < ol->count(); x++)
	{
		BaseComponent *temp = (BaseComponent *)ol->at(x);
		if(temp->queryLook()->inherits("QButton"))
		{
			i.sprintf("insert(%s,%d);\n",ol->at(x)->name(),x);
			writeInParent(i);
		}	
	}
        delete ol;
}

void
zButtonGroup::fillMembInfo()
{
        //fills generic virtual protected stuffs
        genericFillMembInfo();

        //empty destructor
        addMember("destructor()","","",0,2,1);

        //very generic constructor
        addMember("constructor(QWidget *parent=0, char *name=0)",
                        "init(parent,name)","",0,2,1);

        addMember("void clicked(int id)","","",0,3,0);
        addMember("void pressed(int id)","","",0,3,0);
        addMember("void released(int id)","","",0,3,0);
}

/******************************************************************************
 Uncomment if you intend to link this in dynamically, otherwise not necesary
******************************************************************************/

