/*  $Id: zLabel.cc,v 1.4 1998/07/27 16:35:07 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "zLabel.h"

zLabel::zLabel(QWidget *parent) 
					: BaseFrame(parent,"label")
{
	pixmap = new QString("");
	look = new QLabel(this,"look");

	look->setAlignment(AlignTop | AlignLeft);
	vAlign = new QString[2];
	vAlign[0] = "Top"; vAlign[1] = "AlignTop";
	
	hAlign = new QString[2]; 
	hAlign[0] = "Left"; hAlign[1] = "AlignLeft";

	prepFrame(look);
	setLook(look);
}

zLabel::~zLabel()
{
	//Delete your stuff, but don't delete look!
	delete [] vAlign;
	delete [] hAlign;
}
void 
zLabel::fillAttributes()
{
	ADDATTRIB("Text",(char *)look->text(),W_TEXT);

	ADDATTRIB("Pixmap",(char *)pixmap->data(),W_FILE);

	ADDATTRIB("Vertical Alignment",vAlign[0].data(),W_LIST,3,
	"Top", "Middle", "Bottom");

	ADDATTRIB("Horizontal Alignment",hAlign[0].data(),W_LIST,3,
	"Left", "Center", "Right");

        ADDATTRIB("Background Colour",look->backgroundColor(),W_COLOR);

        ADDATTRIB("Text Colour",look->colorGroup().text(),W_COLOR);

        ADDATTRIB("Font",(char *)myLook->font().family(),W_LIST,fontList);

        ADDATTRIB("Font Size",myLook->font().pointSize(),W_LIST,24,
              "10", "12", "14", "16", "18", "20", "22", "24", "26", "28",
              "30", "32", "34", "36", "38", "40", "44", "48", "52", "56",
              "60", "64", "68", "72");


        ADDATTRIB("Bold",myLook->font().bold(),W_TRUE);

        ADDATTRIB("Italic",myLook->font().italic(),W_TRUE);

        ADDATTRIB("Tool Tip",tooltipData->data(),W_TEXT);

	genericFillAttrib();

	showFrameAttrib(look);
}

void 
zLabel::interpretAttributes()
{
	int x=0;

	if(!attrib->giveValue("Pixmap").isEmpty())
	{
		*pixmap = attrib->giveValue("Pixmap");
		look->setText("");
		look->setPixmap(QPixmap(pixmap->data()));
	}
	else if(!attrib->giveValue("Text").isEmpty())
	{
		*pixmap = "";
		look->setPixmap(QPixmap());
		look->setText(attrib->giveValue("Text").data());
	}
	
	if(attrib->giveValue("Vertical Alignment") == "Top")
	{
		vAlign[0] = "Top";
		vAlign[1] = "AlignTop";
		x = AlignTop;
	}
	else if(attrib->giveValue("Vertical Alignment") == "Middle")
	{
		vAlign[0] = "Middle";
		vAlign[1] = "AlignVCenter";
		x = AlignVCenter;
	}
	else if(attrib->giveValue("Vertical Alignment") == "Bottom")
	{
		vAlign[0] = "Bottom";
		vAlign[1] = "AlignBottom";
		x = AlignBottom;
	}

	if(attrib->giveValue("Horizontal Alignment") == "Left")
	{
		hAlign[0] = "Left";
		hAlign[1] = "AlignLeft";
		x = x | AlignLeft;
	}
	else if(attrib->giveValue("Horizontal Alignment") == "Center")
	{
		hAlign[0] = "Center";
		hAlign[1] = "AlignHCenter";
		x = x | AlignHCenter;
	}
	else if(attrib->giveValue("Horizontal Alignment") == "Right")
	{
		hAlign[0] = "Right";
		hAlign[1] = "AlignRight";
		x = x | AlignRight;
	}
	look->setAlignment(x);

	QColorGroup p(look->foregroundColor(),
			 attrib->giveColor("Background Colour"),
			 QColor(255,255,255),QColor(96,96,96),
			 QColor(160,160,160), attrib->giveColor("Text Colour"),
			 QColor(255,255,255));

	look->setPalette(QPalette(p,
		look->palette().disabled(),look->palette().active()));

        int bo = attrib->giveBoolValue("Bold") ? QFont::Bold : QFont::Normal;
        myLook->setFont(QFont(attrib->giveValue("Font").data(),
                          attrib->giveIntValue("Font Size"),
                          bo,attrib->giveBoolValue("Italic")));

        *tooltipData = attrib->giveValue("Tool Tip");

	genericInterpAttrib();

	interpFrameAttrib(look);
}

void
zLabel::writeInits()
{
	QString i(look->text());
	
	if(!pixmap->isEmpty())
	{
		i.sprintf("setPixmap(QPixmap(\"%s\"));\n",pixmap->data());
		WRITEATTRIB("Pixmap",i);	
	}
	else if(!i.isEmpty())
	{
		i.sprintf("setText(\"%s\");\n",(char *)look->text());
		WRITEATTRIB("Text",i);
	}	

	QString xtc;
	if(look->font().bold())
		xtc = "QFont::Bold";
 	else	
		xtc = "QFont::Normal";
	i.sprintf("setFont(QFont(\"%s\",%d,%s,(int)%d));\n",
	(char *)look->font().family(),look->font().pointSize(),
	xtc.data(),look->font().italic());
	if(isWritten("Font Size") || isWritten("Font") || isWritten("Bold") || isWritten("Italic"))
	   writeInParent(i);


	i.sprintf(
	"setAlignment(%s | %s);\n"
		,hAlign[1].data(),vAlign[1].data());
	writeInParent(i);

	i.sprintf(
	"setGeometry(%d,%d,%d,%d);\n"
		,x(),y(),width(),height());
	writeInParent(i);

        if(!tooltipData->isEmpty())
        {
                QString p;
                if(noFuncs() == TRUE)
                        p = name();
                else
                        p = "this";

                i.sprintf(
                "QToolTip::add(%s,\"%s\");\n"
                        ,p.data(),tooltipData->data());
                writeLine(i);
	}

        writePalette();

       if(isWritten("Maximum Width") || isWritten("Maximum Height"))
       {
	   i.sprintf("setMaximumSize(%d,%d);\n",
		     look->maximumSize().width(),look->maximumSize().height());
	   writeInParent(i);
       }
       if(isWritten("Minimum Width") || isWritten("Minimum Height"))
       {
	   i.sprintf("setMinimumSize(%d,%d);\n",
		     look->minimumSize().width(),look->minimumSize().height());
	   writeInParent(i);
       }

        if(*uiStyle == "Motif")
               i.sprintf("setStyle(MotifStyle);\n");
        else if(*uiStyle == "Windows")
                i.sprintf("setStyle(WindowsStyle);\n");
        else
                i = "";
        if(!i.isEmpty()) writeInParent(i);

	writeFrame();
}

void
zLabel::fillMembInfo()
{
        //fills generic virtual protected stuffs
        genericFillMembInfo();

        addMember("destructor()","","",0,2,1);

       addMember("constructor(QWidget *parent=0, char *name=0)",
                        "init(parent,name)","",0,2,1);
}			
