/*  $Id: zListBox.cc,v 1.4 1998/07/27 16:35:08 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "zListBox.h"

zListBox::zListBox(QWidget *parent) 
					: BaseComponent(parent,"list")
{
	resize(200,110);
	look = new QListBox(this,"foo");
	setLook(look);
}

zListBox::~zListBox()
{
	//Delete your stuff, but don't delete look!
}

void 
zListBox::fillAttributes()
{
	ADDATTRIB("Auto-Scrollbar (right)",look->autoScrollBar(),W_TRUE);

	ADDATTRIB("Auto-Scrollbar (bottom)",
		look->autoBottomScrollBar(),W_TRUE);

	ADDATTRIB("Multiple Selection",look->isMultiSelection(),W_TRUE);
	
	ADDATTRIB("Drag Selection",look->dragSelect(),W_TRUE);

	ADDATTRIB("Bottom ScrollBar",look->bottomScrollBar(),W_TRUE);

	ADDATTRIB("Right ScrollBar",look->scrollBar(),W_TRUE);

        ADDATTRIB("Text Colour",look->colorGroup().text(),W_COLOR);

        ADDATTRIB("Font",(char *)myLook->font().family(),W_LIST,fontList);

        ADDATTRIB("Font Size",myLook->font().pointSize(),W_LIST,24,
              "10", "12", "14", "16", "18", "20", "22", "24", "26", "28",
              "30", "32", "34", "36", "38", "40", "44", "48", "52", "56",
              "60", "64", "68", "72");


        ADDATTRIB("Bold",myLook->font().bold(),W_TRUE);

        ADDATTRIB("Italic",myLook->font().italic(),W_TRUE);

        ADDATTRIB("Tool Tip",tooltipData->data(),W_TEXT);

        genericFillAttrib();
}

void 
zListBox::interpretAttributes()
{
	look->setAutoScrollBar(attrib->giveBoolValue("Auto-Scrollbar (right)"));
	
	look->setAutoBottomScrollBar(
		attrib->giveBoolValue("Auto-Scrollbar (bottom)"));

	look->setDragSelect(attrib->giveBoolValue("Drag Selection"));

	look->setBottomScrollBar(attrib->giveBoolValue("Bottom ScrollBar"));

	look->setScrollBar(attrib->giveBoolValue("Right ScrollBar"));

	look->setMultiSelection(attrib->giveBoolValue("Multiple Selection"));
	
        QColorGroup p(look->foregroundColor(),
                         look->backgroundColor(),
                         QColor(255,255,255),QColor(96,96,96),
                         QColor(160,160,160), attrib->giveColor("Text Colour"),
                         QColor(255,255,255));

        look->setPalette(QPalette(p,
                look->palette().disabled(),look->palette().active()));

        int bo = attrib->giveBoolValue("Bold") ? QFont::Bold : QFont::Normal;
        myLook->setFont(QFont(attrib->giveValue("Font").data(),
                          attrib->giveIntValue("Font Size"),
                          bo,attrib->giveBoolValue("Italic")));

        *tooltipData = attrib->giveValue("Tool Tip");

        genericInterpAttrib();
}

void
zListBox::writeInits()
{
	QString i;

	i.sprintf("setAutoScrollBar(%s);\n",strBool(look->autoScrollBar()));
	WRITEATTRIB("Auto-Scrollbar (right)",i);

	i.sprintf("setMultiSelection(%s);\n",
		strBool(look->isMultiSelection()));
	WRITEATTRIB("Multiple Selection",i);
	
	i.sprintf("setAutoBottomScrollBar(%s);\n",
		strBool(look->autoBottomScrollBar()));
	WRITEATTRIB("Auto-Scrollbar (bottom)",i);

	i.sprintf("setDragSelect(%s);\n",strBool(look->dragSelect()));
	WRITEATTRIB("Drag Selection",i);

	i.sprintf("setBottomScrollBar(%s);\n",strBool(look->bottomScrollBar()));
	WRITEATTRIB("Bottom ScrollBar",i);

	i.sprintf("setScrollBar(%s);\n",strBool(look->scrollBar()));
	WRITEATTRIB("Right ScrollBar",i);

        i.sprintf("setGeometry(%d,%d,%d,%d);\n",x(),y(), width(),height());
        writeInParent(i);

        if(!tooltipData->isEmpty())
        {
                QString p;
                if(noFuncs() == TRUE)
                        p = name();
                else
                        p = "this";

                i.sprintf(
                "QToolTip::add(%s,\"%s\");\n"
                        ,p.data(),tooltipData->data());
                writeLine(i);
        }

        QString xtc;
        if(look->font().bold())
		xtc = "QFont::Bold";
	else
		xtc = "QFont::Normal";
	i.sprintf("setFont(QFont(\"%s\",%d,%s,(int)%d));\n",
	(char *)look->font().family(),look->font().pointSize(),
	xtc.data(),look->font().italic());
       if(isWritten("Font Size") || isWritten("Font") || isWritten("Bold") || isWritten("Italic"))
	   writeInParent(i);

       if(isWritten("Maximum Width") || isWritten("Maximum Height"))
       {
	   i.sprintf("setMaximumSize(%d,%d);\n",
		     look->maximumSize().width(),look->maximumSize().height());
	   writeInParent(i);
       }
       if(isWritten("Minimum Width") || isWritten("Minimum Height"))
       {
	   i.sprintf("setMinimumSize(%d,%d);\n",
		     look->minimumSize().width(),look->minimumSize().height());
	   writeInParent(i);
       }

        if(*uiStyle == "Motif")
               i.sprintf("setStyle(MotifStyle);\n");
        else if(*uiStyle == "Windows")
                i.sprintf("setStyle(WindowsStyle);\n");
        else 
		i = "";
        if(!i.isEmpty()) writeInParent(i);

        writePalette();

}


void
zListBox::fillMembInfo()
{
        //fills generic virtual protected stuffs
        genericFillMembInfo();

        addMember("destructor()","","",0,2,1);

       addMember("constructor(QWidget *parent=0, char *name=0)",
                        "init(parent,name)","",0,2,1);

	addMember("void highlighted(int ind)","","",0,3,0);
	addMember("void selected(int ind)","","",0,3,0);
}
