/*  $Id: zMTextBox.cc,v 1.4 1998/07/27 16:35:08 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "zMTextBox.h"

zMTextBox::zMTextBox(QWidget *parent) 
					: BaseComponent(parent,"mlined")
{
	resize(200,110);
	look = new QMultiLineEdit(this,"foo");
	setLook(look);
}

zMTextBox::~zMTextBox()
{
	//Delete your stuff, but don't delete look!
}

void 
zMTextBox::fillAttributes()
{
	ADDATTRIB("Read Only",look->isReadOnly(),W_TRUE);

	ADDATTRIB("Text",look->text().data(),W_TEXT);	

        ADDATTRIB("Background Colour",look->backgroundColor(),W_COLOR);

        ADDATTRIB("Text Colour",look->colorGroup().text(),W_COLOR);

        ADDATTRIB("Font",(char *)myLook->font().family(),W_LIST,fontList);

        ADDATTRIB("Font Size",myLook->font().pointSize(),W_LIST,24,
              "10", "12", "14", "16", "18", "20", "22", "24", "26", "28",
              "30", "32", "34", "36", "38", "40", "44", "48", "52", "56",
              "60", "64", "68", "72");


        ADDATTRIB("Bold",myLook->font().bold(),W_TRUE);

        ADDATTRIB("Italic",myLook->font().italic(),W_TRUE);

        ADDATTRIB("Tool Tip",tooltipData->data(),W_TEXT);

        genericFillAttrib();

}

void 
zMTextBox::interpretAttributes()
{
	QString i;
	look->setText(attrib->giveValue("Text"));

        QColorGroup p(look->foregroundColor(),
          attrib->giveColor("Background Colour"),
          QColor(255,255,255),QColor(96,96,96),
          QColor(160,160,160), attrib->giveColor("Text Colour"),
          QColor(255,255,255));

        look->setPalette(QPalette(p,
                look->palette().disabled(),look->palette().active()));

	look->setReadOnly(attrib->giveBoolValue("Read Only"));

        int bo = attrib->giveBoolValue("Bold") ? QFont::Bold : QFont::Normal;
        myLook->setFont(QFont(attrib->giveValue("Font").data(),
                          attrib->giveIntValue("Font Size"),
                          bo,attrib->giveBoolValue("Italic")));

        *tooltipData = attrib->giveValue("Tool Tip");

        genericInterpAttrib();
}

void
zMTextBox::writeInits()
{
	QString i;
	if(!look->text().isNull() && !look->text().isEmpty())
	{
		i.sprintf("setText(\"%s\");\n",look->text().data());
		WRITEATTRIB("Text",i);
	}
	
        QString xtc;
        if(look->font().bold())
		xtc = "QFont::Bold";
	else
		xtc = "QFont::Normal";
	i.sprintf("setFont(QFont(\"%s\",%d,%s,(int)%d));\n",
	(char *)look->font().family(),look->font().pointSize(),
	xtc.data(),look->font().italic());
       if(isWritten("Font Size") || isWritten("Font") || isWritten("Bold") || isWritten("Italic"))
	   writeInParent(i);

	
        i.sprintf("setGeometry(%d,%d,%d,%d);\n",x(),y(), width(),height());
        writeInParent(i);

        if(!tooltipData->isEmpty())
        {
                QString p;
                if(noFuncs() == TRUE)
                        p = name();
                else
                        p = "this";

                i.sprintf(
                "QToolTip::add(%s,\"%s\");\n"
                        ,p.data(),tooltipData->data());
                writeLine(i);
        }

	i.sprintf("setReadOnly(%s);\n",strBool(look->isReadOnly()));	
	WRITEATTRIB("Read Only",i);

       if(isWritten("Maximum Width") || isWritten("Maximum Height"))
       {
	   i.sprintf("setMaximumSize(%d,%d);\n",
		     look->maximumSize().width(),look->maximumSize().height());
	   writeInParent(i);
       }
       if(isWritten("Minimum Width") || isWritten("Minimum Height"))
       {
	   i.sprintf("setMinimumSize(%d,%d);\n",
		     look->minimumSize().width(),look->minimumSize().height());
	   writeInParent(i);
       }

        if(*uiStyle == "Motif")
               i.sprintf("setStyle(MotifStyle);\n");
        else if(*uiStyle == "Windows")
                i.sprintf("setStyle(WindowsStyle);\n");
        else
                i = "";
        if(!i.isEmpty()) writeInParent(i);

        writePalette();

}

void
zMTextBox::fillMembInfo()
{
	//fills generic virtual protected stuffs
        genericFillMembInfo();

	//empty destructor
        addMember("destructor()","","",0,2,1);

        //very generic constructor
       addMember("constructor(QWidget *parent=0, char *name=0)",
                        "init(parent,name)","",0,2,1);

	addMember("void setText (const char *val)","","",0,4,0);

	addMember("void append (const char *val)","","",0,4,0);

	addMember("void copyText()","","",0,4,0);
        addMember("void paste ()","","",0,4,0);
	addMember("void cut ()","","",0,4,0);
	
	addMember("void textChanged()","","",0,3,0);
	addMember("void returnPressed()","","",0,3,0);
}
