/*  $Id: zSlider.cc,v 1.4 1998/07/27 16:35:09 zachsman Exp $
 *
 *   PART OF QtEZ BY SAM MAGNUSON (http://qtez.zax.net/qtez/):
 *   QtEZ is bound by the licensing agreement of Troll Tech AS
 *      free software edition
 */

#include "zSlider.h"
#undef Above
#undef Below

zSlider::zSlider(QWidget *parent)
					: BaseComponent(parent,"slide")
{
	resize(33,330);
	look = new QSlider(this,"foo");

	tickM = new QString;
     	if(look->tickmarks() == QSlider::NoMarks)
		*tickM = "NoMarks";
        else if(look->tickmarks() == QSlider::Above)
		*tickM = "Above";
        else if(look->tickmarks() == QSlider::Below)
		*tickM = "Below";
        else if(look->tickmarks() == QSlider::Left)
		*tickM = "Left";
        else if(look->tickmarks() == QSlider::Right)
		*tickM = "Left";

	orient = new QString;
        if(look->orientation() == QSlider::Vertical)
		*orient = "Vertical";
        else
                *orient = "Horizontal";

	setLook(look);
}

zSlider::~zSlider()
{
	delete tickM;
	delete orient;
	//Delete your stuff, but don't delete look!
}

void 
zSlider::fillAttributes()
{
	ADDATTRIB("Starting Value", look->value(),W_TEXT);

	ADDATTRIB("Minimum Value", look->minValue(),W_TEXT);

	ADDATTRIB("Maximum Value", look->maxValue(),W_TEXT);

	ADDATTRIB("Line Incriment",look->lineStep(),W_TEXT);

	ADDATTRIB("Page Incriment",look->pageStep(),W_TEXT);

	ADDATTRIB("Direction",orient->data(),W_LIST, 2,
	"Vertical","Horizontal");

	ADDATTRIB("Tick Marks",tickM->data(),W_LIST,5,
	"NoMarks","Above","Below","Left","Right");

        ADDATTRIB("Tool Tip",tooltipData->data(),W_TEXT);

        ADDATTRIB("Background Colour",look->colorGroup().background(),W_COLOR);

	genericFillAttrib();
}

void 
zSlider::interpretAttributes()
{
	look->setValue(attrib->giveIntValue("Starting Value"));

	look->setRange(attrib->giveIntValue("Minimum Value"),
		       attrib->giveIntValue("Maximum Value"));

	look->setSteps(attrib->giveIntValue("Line Incriment"),
		       attrib->giveIntValue("Page Incriment"));

	*orient = attrib->giveValue("Direction");
	if(*orient == "Vertical")
		look->setOrientation(QSlider::Vertical);
	else
		look->setOrientation(QSlider::Horizontal);

	*tickM = attrib->giveValue("Tick Marks");
	if(*tickM == "NoMarks")
		look->setTickmarks(QSlider::NoMarks);
	else if(*tickM == "Above") 
		look->setTickmarks(QSlider::Above);
	else if(*tickM == "Below")
		look->setTickmarks(QSlider::Below);
	else if(*tickM == "Left")
		look->setTickmarks(QSlider::Left);
	else if(*tickM == "Right")
		look->setTickmarks(QSlider::Right);

        *tooltipData = attrib->giveValue("Tool Tip");

        QColorGroup p(look->foregroundColor(),
			attrib->giveColor("Background Colour"),
                         QColor(255,255,255),QColor(96,96,96),
                         QColor(160,160,160), look->colorGroup().text(),
                         QColor(255,255,255));

        look->setPalette(QPalette(p,
                look->palette().disabled(),look->palette().active()));

	genericInterpAttrib();
}

void
zSlider::writeInits()
{
	QString i;

	i.sprintf("setValue(%d);\n",look->value());
	WRITEATTRIB("Starting Value",i);

	i.sprintf("setRange(%d,%d);\n",look->minValue(),look->maxValue());
	if(isWritten("Minimum Value") || isWritten("Maximum Value"))
	    writeInParent(i);

	i.sprintf("setSteps(%d,%d);\n",look->lineStep(),look->pageStep());
	if(isWritten("Line Incriment") || isWritten("Page Incriment"))
	    writeInParent(i);

        i.sprintf("setOrientation(QSlider::%s);\n",orient->data());
	WRITEATTRIB("Direction",i);

        i.sprintf("setGeometry(%d,%d,%d,%d);\n",x(),y(),width(),height());
        writeInParent(i);

        if(!tooltipData->isEmpty())
        {
                QString p;
                if(noFuncs() == TRUE)
                        p = name();
                else
                        p = "this";

                i.sprintf(
                "QToolTip::add(%s,\"%s\");\n"
                        ,p.data(),tooltipData->data());
                writeLine(i);
        }

	i.sprintf("setTickmarks(QSlider::%s);\n",tickM->data());
	WRITEATTRIB("Tick Marks",i);

        writePalette();

       if(isWritten("Maximum Width") || isWritten("Maximum Height"))
       {
	   i.sprintf("setMaximumSize(%d,%d);\n",
		     look->maximumSize().width(),look->maximumSize().height());
	   writeInParent(i);
       }
       if(isWritten("Minimum Width") || isWritten("Minimum Height"))
       {
	   i.sprintf("setMinimumSize(%d,%d);\n",
		     look->minimumSize().width(),look->minimumSize().height());
	   writeInParent(i);
       }

        if(*uiStyle == "Motif")
               i.sprintf("setStyle(MotifStyle);\n");
        else if(*uiStyle == "Windows")
                i.sprintf("setStyle(WindowsStyle);\n");
        else
                i = "";
        if(!i.isEmpty()) writeInParent(i);
}

void
zSlider::fillMembInfo()
{
        //fills generic virtual protected stuffs
        genericFillMembInfo();

        addMember("destructor()","","",0,2,1);

       addMember("constructor(QWidget *parent=0, char *name=0)",
                        "init(parent,name)","",0,2,1);

        addMember("void valueChange()","","",1,1,1);
	addMember("void stepChange()","","",1,1,1);
	addMember("void rangeChange()","","",1,1,1);
	addMember("void valueChanged(int x)","","",0,3,0);
        addMember("void sliderPressed()","","",0,3,0);
        addMember("void sliderMoved(int x)","","",0,3,0);
        addMember("void sliderReleased()","","",0,3,0);
        addMember("void nextLine()","","",0,3,0);
        addMember("void prevLine()","","",0,3,0);
        addMember("void nextPage()","","",0,3,0);
        addMember("void prevPage()","","",0,3,0);
		
}
